@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: package.sh,v 1.1.2.8 2005/03/24 22:43:02 nickm Exp $
+# $Id: package.sh,v 1.4 2005/03/19 05:07:19 nickm Exp $
 # Copyright 2004 Nick Mathewson.
 # See LICENSE in Tor distribution for licensing information.
 
@@ -129,7 +129,7 @@ groff doc/tor.1 -T ps -m man | ps2pdf - $DOC/tor-reference.pdf
 groff doc/tor-resolve.1 -T ps -m man | ps2pdf - $DOC/tor-resolve.pdf
 
 mkdir $DOC/Advanced
-cp doc/tor-spec.txt doc/rend-spec.txt doc/control-spec.txt doc/socks-extensions.txt $DOC/Advanced
+cp doc/tor-spec.txt doc/rend-spec.txt doc/control-spec.txt doc/socks-extensions.txt doc/version-spec.txt $DOC/Advanced
 cp doc/CLIENTS $DOC/Advanced/CLIENTS.txt
 cp doc/HACKING $DOC/Advanced/HACKING.txt
 cp ChangeLog $DOC/Advanced/ChangeLog.txt
