@@ -1,14 +1,13 @@
 /* Copyright 2003-2004 Roger Dingledine; Copyright 2004 Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c,v 1.22.2.10 2005/02/19 03:42:26 arma Exp $ */
-const char compat_c_id[] = "$Id: compat.c,v 1.22.2.10 2005/02/19 03:42:26 arma Exp $";
+/* $Id: compat.c,v 1.44 2005/03/28 01:11:02 arma Exp $ */
+const char compat_c_id[] = "$Id: compat.c,v 1.44 2005/03/28 01:11:02 arma Exp $";
 
 /* This is required on rh7 to make strptime not complain.
  */
 #define _GNU_SOURCE
 
 #include "orconfig.h"
-#include "fakepoll.h"
 #include "compat.h"
 
 #ifdef MS_WINDOWS
@@ -55,6 +54,9 @@ const char compat_c_id[] = "$Id: compat.c,v 1.22.2.10 2005/02/19 03:42:26 arma E
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h> /* FreeBSD needs this to know what version it is */
 #endif
@@ -63,6 +65,9 @@ const char compat_c_id[] = "$Id: compat.c,v 1.22.2.10 2005/02/19 03:42:26 arma E
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#ifdef HAVE_PTHREAD_H
+#include <pthread.h>
+#endif
 
 #include "log.h"
 #include "util.h"
@@ -357,12 +362,19 @@ tor_socketpair(int family, int type, int protocol, int fd[2])
 /** Get the maximum allowed number of file descriptors. (Some systems
  * have a low soft limit.) Make sure we set it to at least
  * <b>*limit</b>. Return a new limit if we can, or -1 if we fail. */
-int set_max_file_descriptors(int limit, int cap) {
+int
+set_max_file_descriptors(unsigned long limit, unsigned long cap) {
 #ifndef HAVE_GETRLIMIT
   log_fn(LOG_INFO,"This platform is missing getrlimit(). Proceeding.");
+  if (limit > cap) {
+    log(LOG_INFO, "ConnLimit must be at most %d. Capping it.", cap);
+    limit = cap;
+  }
 #else
   struct rlimit rlim;
-  int most;
+  unsigned long most;
+  tor_assert(limit > 0);
+  tor_assert(cap > 0);
 
   if (getrlimit(RLIMIT_NOFILE, &rlim) != 0) {
     log_fn(LOG_WARN, "Could not get maximum number of file descriptors: %s",
@@ -370,13 +382,13 @@ int set_max_file_descriptors(int limit, int cap) {
     return -1;
   }
   if (rlim.rlim_max < limit) {
-    log_fn(LOG_WARN,"We need %d file descriptors available, and we're limited to %lu. Please change your ulimit -n.", limit, (unsigned long int)rlim.rlim_max);
+    log_fn(LOG_WARN,"We need %lu file descriptors available, and we're limited to %lu. Please change your ulimit -n.", limit, (unsigned long)rlim.rlim_max);
     return -1;
   }
-  most = ((rlim.rlim_max > cap) ? cap : rlim.rlim_max);
+  most = (rlim.rlim_max > cap) ? cap : (unsigned) rlim.rlim_max;
   if (most > rlim.rlim_cur) {
-    log_fn(LOG_INFO,"Raising max file descriptors from %lu to %d.",
-           (unsigned long int)rlim.rlim_cur, most);
+    log_fn(LOG_INFO,"Raising max file descriptors from %lu to %lu.",
+           (unsigned long)rlim.rlim_cur, most);
   }
   rlim.rlim_cur = most;
   if (setrlimit(RLIMIT_NOFILE, &rlim) != 0) {
@@ -388,17 +400,6 @@ int set_max_file_descriptors(int limit, int cap) {
   limit = most;
 #endif
 
-#ifdef USE_FAKE_POLL
-  if (limit > 1024) {
-    log(LOG_INFO, "Systems without a working poll() can't set ConnLimit higher than %d in Tor 0.0.9.x. Capping.", 1024);
-    limit = 1024;
-  }
-#endif
-
-  if (limit > cap) { /* this won't be checked above if ndef getrlimit */
-    log(LOG_INFO, "ConnLimit must be at most %d. Capping it.", cap);
-    limit = cap;
-  }
   if (limit < ULIMIT_BUFFER) {
     log_fn(LOG_WARN,"ConnLimit must be at least %d. Failing.", ULIMIT_BUFFER);
     return -1;
@@ -500,6 +501,92 @@ int tor_inet_aton(const char *c, struct in_addr* addr)
 #endif
 }
 
+/** Similar behavior to Unix gethostbyname: resolve <b>name</b>, and set
+ * *addr to the proper IP address, in network byte order.  Returns 0
+ * on success, -1 on failure; 1 on transient failure.
+ *
+ * (This function exists because standard windows gethostbyname
+ * doesn't treat raw IP addresses properly.)
+ */
+int tor_lookup_hostname(const char *name, uint32_t *addr)
+{
+  /* Perhaps eventually this should be replaced by a tor_getaddrinfo or
+   * something.
+   */
+  struct in_addr iaddr;
+  tor_assert(addr);
+  if (!*name) {
+    /* Empty address is an error. */
+    return -1;
+  } else if (tor_inet_aton(name, &iaddr)) {
+    /* It's an IP. */
+    memcpy(addr, &iaddr.s_addr, 4);
+    return 0;
+  } else {
+#ifdef HAVE_GETADDRINFO
+    int err;
+    struct addrinfo *res, *res_p;
+    struct addrinfo hints;
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = PF_INET;
+    hints.ai_socktype = SOCK_STREAM;
+    err = getaddrinfo(name, NULL, NULL, &res);
+    if (!err) {
+      for (res_p = res; res_p; res_p = res_p->ai_next) {
+        if (res_p->ai_family == AF_INET) {
+          struct sockaddr_in *sin = (struct sockaddr_in *)res_p->ai_addr;
+          memcpy(addr, &sin->sin_addr, 4);
+          freeaddrinfo(res);
+          return 0;
+        } else {
+        }
+      }
+      return -1;
+    }
+
+    return (err == EAI_AGAIN) ? 1 : -1;
+#else
+    struct hostent *ent;
+    int err;
+#ifdef HAVE_GETHOSTBYNAME_R_6_ARG
+    char buf[2048];
+    struct hostent hostent;
+    int r;
+    r = gethostbyname_r(name, &hostent, buf, sizeof(buf), &ent, &err);
+#elif defined(HAVE_GETHOSTBYNAME_R_5_ARG)
+    char buf[2048];
+    struct hostent hostent;
+    ent = gethostbyname_r(name, &hostent, buf, sizeof(buf), &err);
+#elif defined(HAVE_GETHOSTBYNAME_R_3_ARG)
+    struct hostent_data data;
+    struct hostent hent;
+    memset(&data, 0, sizeof(data));
+    err = gethostbyname_r(name, &hent, &data);
+    ent = err ? NULL : &hent;
+#else
+    ent = gethostbyname(name);
+#ifdef MS_WINDOWS
+    err = WSAGetLastError();
+#else
+    err = h_errno;
+#endif
+#endif
+    if (ent) {
+      /* break to remind us if we move away from IPv4 */
+      tor_assert(ent->h_length == 4);
+      memcpy(addr, ent->h_addr, 4);
+      return 0;
+    }
+    memset(addr, 0, 4);
+#ifdef MS_WINDOWS
+    return (err == WSATRY_AGAIN) ? 1 : -1;
+#else
+    return (err == TRY_AGAIN) ? 1 : -1;
+#endif
+#endif
+  }
+}
+
 /* Hold the result of our call to <b>uname</b>. */
 static char uname_result[256];
 /* True iff uname_result is set. */
@@ -522,8 +609,55 @@ get_uname(void)
     } else
 #endif
       {
+#ifdef MS_WINDOWS
+        OSVERSIONINFO info;
+        int i;
+        const char *plat = NULL;
+        static struct {
+          int major; int minor; const char *version;
+        } win_version_table[] = {
+          { 5, 2, "Windows Server 2003" },
+          { 5, 1, "Windows XP" },
+          { 5, 0, "Windows 2000" },
+          /* { 4, 0, "Windows NT 4.0" }, */
+          { 4, 90, "Windows Me" },
+          { 4, 10, "Windows 98" },
+          /* { 4, 0, "Windows 95" } */
+          { 3, 51, "Windows NT 3.51" },
+          { -1, -1, NULL }
+        };
+        info.dwOSVersionInfoSize = sizeof(info);
+        GetVersionEx(&info);
+        if (info.dwMajorVersion == 4 && info.dwMinorVersion == 0) {
+          if (info.dwPlatformId == VER_PLATFORM_WIN32_NT)
+            plat = "Windows NT 4.0";
+          else
+            plat = "Windows 95";
+        } else {
+          for (i=0; win_version_table[i].major>=0; ++i) {
+            if (win_version_table[i].major == info.dwMajorVersion &&
+                win_version_table[i].minor == info.dwMinorVersion) {
+              plat = win_version_table[i].version;
+              break;
+            }
+          }
+        }
+        if (plat) {
+          strlcpy(uname_result, plat, sizeof(uname_result));
+        } else {
+          if (info.dwMajorVersion > 5 ||
+              (info.dwMajorVersion==5 && info.dwMinorVersion>2))
+            tor_snprintf(uname_result, sizeof(uname_result),
+                         "Very recent version of Windows [major=%d,minor=%d]",
+                         (int)info.dwMajorVersion,(int)info.dwMinorVersion);
+          else
+            tor_snprintf(uname_result, sizeof(uname_result),
+                         "Unrecognized version of Windows [major=%d,minor=%d]",
+                         (int)info.dwMajorVersion,(int)info.dwMinorVersion);
+        }
+#else
         strlcpy(uname_result, "Unknown platform", sizeof(uname_result));
-/* XXX win32 doesn't have uname, but we can still ifdef windows sprint "windows" */
+#endif
       }
     uname_result_is_set = 1;
   }
@@ -534,6 +668,25 @@ get_uname(void)
  *   Process control
  */
 
+#if defined(USE_PTHREADS)
+struct tor_pthread_data_t {
+  int (*func)(void *);
+  void *data;
+};
+static void *
+tor_pthread_helper_fn(void *_data)
+{
+  struct tor_pthread_data_t *data = _data;
+  int (*func)(void*);
+  void *arg;
+  func = data->func;
+  arg = data->data;
+  tor_free(_data);
+  func(arg);
+  return NULL;
+}
+#endif
+
 /** Minimalist interface to run a void function in the background.  On
  * unix calls fork, on win32 calls beginthread.  Returns -1 on failure.
  * func should not return, but rather should call spawn_exit.
@@ -546,12 +699,23 @@ get_uname(void)
 int
 spawn_func(int (*func)(void *), void *data)
 {
-#ifdef MS_WINDOWS
+#if defined(USE_WIN32_THREADS)
   int rv;
   rv = _beginthread(func, 0, data);
   if (rv == (unsigned long) -1)
     return -1;
   return 0;
+#elif defined(USE_PTHREADS)
+  pthread_t thread;
+  struct tor_pthread_data_t *d;
+  d = tor_malloc(sizeof(struct tor_pthread_data_t));
+  d->data = data;
+  d->func = func;
+  if (pthread_create(&thread,NULL,tor_pthread_helper_fn,d))
+    return -1;
+  if (pthread_detach(thread))
+    return -1;
+  return 0;
 #else
   pid_t pid;
   pid = fork();
@@ -573,10 +737,14 @@ spawn_func(int (*func)(void *), void *data)
  */
 void spawn_exit()
 {
-#ifdef MS_WINDOWS
+#if defined(USE_WIN32_THREADS)
   _endthread();
+#elif defined(USE_PTHREADS)
+  pthread_exit(NULL);
 #else
-  exit(0);
+  /* http://www.erlenstar.demon.co.uk/unix/faq_2.html says we should
+   * call _exit, not exit, from child processes. */
+  _exit(0);
 #endif
 }
 
@@ -602,15 +770,63 @@ void tor_gettimeofday(struct timeval *timeval) {
   return;
 }
 
-#ifndef MS_WINDOWS
-struct tor_mutex_t {
-  int _unused;
-};
-tor_mutex_t *tor_mutex_new(void) { return NULL; }
-void tor_mutex_acquire(tor_mutex_t *m) { }
-void tor_mutex_release(tor_mutex_t *m) { }
-void tor_mutex_free(tor_mutex_t *m) { }
+#if defined(TOR_IS_MULTITHREADED) && !defined(MS_WINDOWS)
+#define TIME_FNS_NEED_LOCKS
+#endif
+
+#ifndef HAVE_LOCALTIME_R
+#ifdef TIME_FNS_NEED_LOCKS
+struct tm *tor_localtime_r(const time_t *timep, struct tm *result)
+{
+  struct tm *r;
+  static tor_mutex_t *m=NULL;
+  if (!m) { m=tor_mutex_new(); }
+  tor_assert(result);
+  tor_mutex_acquire(m);
+  r = localtime(timep);
+  memcpy(result, r, sizeof(struct tm));
+  tor_mutex_release(m);
+  return result;
+}
 #else
+struct tm *tor_localtime_r(const time_t *timep, struct tm *result)
+{
+  struct tm *r;
+  tor_assert(result);
+  r = localtime(timep);
+  memcpy(result, r, sizeof(struct tm));
+  return result;
+}
+#endif
+#endif
+
+#ifndef HAVE_GMTIME_R
+#ifdef TIME_FNS_NEED_LOCKS
+struct tm *tor_gmtime_r(const time_t *timep, struct tm *result)
+{
+  struct tm *r;
+  static tor_mutex_t *m=NULL;
+  if (!m) { m=tor_mutex_new(); }
+  tor_assert(result);
+  tor_mutex_acquire(m);
+  r = gmtime(timep);
+  memcpy(result, r, sizeof(struct tm));
+  tor_mutex_release(m);
+  return result;
+}
+#else
+struct tm *tor_gmtime_r(const time_t *timep, struct tm *result)
+{
+  struct tm *r;
+  tor_assert(result);
+  r = gmtime(timep);
+  memcpy(result, r, sizeof(struct tm));
+  return result;
+}
+#endif
+#endif
+
+#ifdef USE_WIN32_THREADS
 struct tor_mutex_t {
   HANDLE handle;
 };
@@ -650,6 +866,46 @@ void tor_mutex_release(tor_mutex_t *m)
     log_fn(LOG_WARN, "Failed to release mutex: %d", GetLastError());
   }
 }
+unsigned long
+tor_get_thread_id(void)
+{
+  return (unsigned long)GetCurrentThreadId();
+}
+#elif defined(USE_PTHREADS)
+struct tor_mutex_t {
+  pthread_mutex_t mutex;
+};
+tor_mutex_t *tor_mutex_new(void)
+{
+  tor_mutex_t *mutex = tor_malloc_zero(sizeof(tor_mutex_t));
+  pthread_mutex_init(&mutex->mutex, NULL);
+  return mutex;
+}
+void tor_mutex_acquire(tor_mutex_t *m)
+{
+  tor_assert(m);
+  pthread_mutex_lock(&m->mutex);
+}
+void tor_mutex_release(tor_mutex_t *m)
+{
+  tor_assert(m);
+  pthread_mutex_unlock(&m->mutex);
+}
+void tor_mutex_free(tor_mutex_t *m)
+{
+  tor_assert(m);
+  pthread_mutex_destroy(&m->mutex);
+  tor_free(m);
+}
+unsigned long
+tor_get_thread_id(void)
+{
+  return (unsigned long)pthread_self();
+}
+#else
+struct tor_mutex_t {
+  int _unused;
+};
 #endif
 
 /**
@@ -768,7 +1024,9 @@ int network_init(void)
     log_fn(LOG_WARN,"Error initializing windows network layer: code was %d",r);
     return -1;
   }
+  /* WSAData.iMaxSockets might show the max sockets we're allowed to use.
+   * We might use it to complain if we're trying to be a server but have
+   * too few sockets available. */
 #endif
   return 0;
 }
-
