@@ -1,7 +1,7 @@
 /* Copyright 2003-2004 Roger Dingledine; Copyright 2004 Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: container.c,v 1.15.2.1 2005/02/28 02:23:12 nickm Exp $ */
-const char container_c_id[] = "$Id: container.c,v 1.15.2.1 2005/02/28 02:23:12 nickm Exp $";
+/* $Id: container.c,v 1.21 2005/03/22 05:37:59 arma Exp $ */
+const char container_c_id[] = "$Id: container.c,v 1.21 2005/03/22 05:37:59 arma Exp $";
 
 #include "compat.h"
 #include "util.h"
@@ -88,7 +88,9 @@ void smartlist_truncate(smartlist_t *sl, int len)
 /** Append element to the end of the list. */
 void smartlist_add(smartlist_t *sl, void *element) {
   if (sl->num_used >= sl->capacity) {
-    sl->capacity *= 2;
+    int higher = sl->capacity * 2;
+    tor_assert(higher > sl->capacity); /* detect overflow */
+    sl->capacity = higher;
     sl->list = tor_realloc(sl->list, sizeof(void*)*sl->capacity);
   }
   sl->list[sl->num_used++] = element;
@@ -100,8 +102,9 @@ void smartlist_add_all(smartlist_t *sl, const smartlist_t *s2)
   SMARTLIST_FOREACH(s2, void *, element, smartlist_add(sl, element));
 }
 
-/** Remove all elements E from sl such that E==element.  Does not preserve
- * the order of s1.
+/** Remove all elements E from sl such that E==element.  Preserve
+ * the order of any elements before E, but elements after E can be
+ * rearranged.
  */
 void smartlist_remove(smartlist_t *sl, void *element) {
   int i;
@@ -132,6 +135,12 @@ int smartlist_string_isin(const smartlist_t *sl, const char *element) {
   return 0;
 }
 
+int smartlist_string_num_isin(const smartlist_t *sl, int num) {
+  char buf[16];
+  tor_snprintf(buf,sizeof(buf),"%d", num);
+  return smartlist_string_isin(sl, buf);
+}
+
 /** Return true iff some element E of sl2 has smartlist_isin(sl1,E).
  */
 int smartlist_overlap(const smartlist_t *sl1, const smartlist_t *sl2) {
@@ -328,7 +337,6 @@ char *smartlist_join_strings2(smartlist_t *sl, const char *join,
 
   tor_assert(sl);
   tor_assert(join);
-  join_len = strlen(join);
   for (i = 0; i < sl->num_used; ++i) {
     n += strlen(sl->list[i]);
     n += join_len;
@@ -613,7 +621,7 @@ strmap_free(strmap_t *map, void (*free_val)(void*))
     SPLAY_REMOVE(strmap_tree, &map->head, ent);
     tor_free(ent->key);
     if (free_val)
-      tor_free(ent->val);
+      free_val(ent->val);
     tor_free(ent);
   }
   tor_assert(SPLAY_EMPTY(&map->head));
