@@ -1,7 +1,7 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar. */
 /* See LICENSE for licensing information */
-/* $Id: crypto.c,v 1.130.2.2 2005/02/12 21:09:28 nickm Exp $ */
-const char crypto_c_id[] = "$Id: crypto.c,v 1.130.2.2 2005/02/12 21:09:28 nickm Exp $";
+/* $Id: crypto.c,v 1.135 2005/04/01 02:37:10 nickm Exp $ */
+const char crypto_c_id[] = "$Id: crypto.c,v 1.135 2005/04/01 02:37:10 nickm Exp $";
 
 /**
  * \file crypto.c
@@ -55,6 +55,7 @@ const char crypto_c_id[] = "$Id: crypto.c,v 1.130.2.2 2005/02/12 21:09:28 nickm
 #include "aes.h"
 #include "util.h"
 #include "container.h"
+#include "compat.h"
 
 #if OPENSSL_VERSION_NUMBER < 0x00905000l
 #error "We require openssl >= 0.9.5"
@@ -78,6 +79,11 @@ const char crypto_c_id[] = "$Id: crypto.c,v 1.130.2.2 2005/02/12 21:09:28 nickm
 /** Macro: is k a valid RSA private key? */
 #define PRIVATE_KEY_OK(k) ((k) && (k)->key && (k)->key->p)
 
+#ifdef TOR_IS_MULTITHREADED
+static tor_mutex_t **_openssl_mutexes = NULL;
+static int _n_openssl_mutexes = -1;
+#endif
+
 struct crypto_pk_env_t
 {
   int refs; /* reference counting so we don't have to copy keys */
@@ -100,6 +106,8 @@ RSA *_crypto_pk_env_get_rsa(crypto_pk_env_t *env);
 EVP_PKEY *_crypto_pk_env_get_evp_pkey(crypto_pk_env_t *env, int private);
 DH *_crypto_dh_env_get_dh(crypto_dh_env_t *dh);
 
+static int setup_openssl_threading(void);
+
 /** Return the number of bytes added by padding method <b>padding</b>.
  */
 static INLINE int
@@ -157,6 +165,7 @@ int crypto_global_init()
   if (!_crypto_global_initialized) {
       ERR_load_crypto_strings();
       _crypto_global_initialized = 1;
+      setup_openssl_threading();
   }
   return 0;
 }
@@ -166,6 +175,19 @@ int crypto_global_init()
 int crypto_global_cleanup()
 {
   ERR_free_strings();
+#ifdef TOR_IS_MULTITHREADED
+  if (_n_openssl_mutexes) {
+    int n = _n_openssl_mutexes;
+    tor_mutex_t **ms = _openssl_mutexes;
+    int i;
+    _openssl_mutexes = NULL;
+    _n_openssl_mutexes = 0;
+    for (i=0;i<n;++i) {
+      tor_mutex_free(ms[i]);
+    }
+    tor_free(ms);
+  }
+#endif
   return 0;
 }
 
@@ -1626,3 +1648,32 @@ secret_to_key(char *key_out, size_t key_out_len, const char *secret,
   crypto_free_digest_env(d);
 }
 
+#ifdef TOR_IS_MULTITHREADED
+static void
+_openssl_locking_cb(int mode, int n, const char *file, int line)
+{
+  if (!_openssl_mutexes)
+    /* This is not a really good  fix for the
+     * "release-freed-lock-from-separate-thread-on-shutdown" problem, but
+     * it can't hurt. */
+    return;
+  if (mode & CRYPTO_LOCK)
+    tor_mutex_acquire(_openssl_mutexes[n]);
+  else
+    tor_mutex_release(_openssl_mutexes[n]);
+}
+static int
+setup_openssl_threading(void) {
+  int i;
+  int n = CRYPTO_num_locks();
+  _n_openssl_mutexes = n;
+  _openssl_mutexes = tor_malloc(n*sizeof(tor_mutex_t *));
+  for (i=0; i < n; ++i)
+    _openssl_mutexes[i] = tor_mutex_new();
+  CRYPTO_set_locking_callback(_openssl_locking_cb);
+  CRYPTO_set_id_callback(tor_get_thread_id);
+  return 0;
+}
+#else
+static int setup_openssl_threading(void) { return 0; }
+#endif
