@@ -1,6 +1,6 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar. */
 /* See LICENSE for licensing information */
-/* $Id: log.h,v 1.40 2004/11/30 06:17:34 nickm Exp $ */
+/* $Id: log.h,v 1.42 2005/04/01 07:05:20 nickm Exp $ */
 
 /**
  * \file log.h
@@ -9,7 +9,7 @@
  **/
 
 #ifndef __LOG_H
-#define LOG_H_ID "$Id: log.h,v 1.40 2004/11/30 06:17:34 nickm Exp $"
+#define LOG_H_ID "$Id: log.h,v 1.42 2005/04/01 07:05:20 nickm Exp $"
 
 #include "../common/compat.h"
 
@@ -66,6 +66,8 @@ void reset_logs(void);
 void add_temp_log(void);
 void close_temp_logs(void);
 void mark_logs_temp(void);
+void configure_libevent_logging(void);
+void suppress_libevent_log_msg(const char *msg);
 
 /* Outputs a message to stdout */
 void _log(int severity, const char *format, ...) CHECK_PRINTF(2,3);
