@@ -1,6 +1,6 @@
 /* Copyright 2003 Roger Dingledine */
 /* See LICENSE for licensing information */
-/* $Id: torgzip.h,v 1.4.2.1 2005/01/21 02:06:27 nickm Exp $ */
+/* $Id: torgzip.h,v 1.5 2005/01/19 22:40:32 nickm Exp $ */
 
 /**
  * \file torgzip.h
@@ -9,9 +9,11 @@
 
 #ifndef __TORGZIP_H
 #define __TORGZIP_H
-#define TORGZIP_H_ID "$Id: torgzip.h,v 1.4.2.1 2005/01/21 02:06:27 nickm Exp $"
+#define TORGZIP_H_ID "$Id: torgzip.h,v 1.5 2005/01/19 22:40:32 nickm Exp $"
 
-typedef enum { GZIP_METHOD=1, ZLIB_METHOD=2 } compress_method_t;
+typedef enum {
+  GZIP_METHOD=1, ZLIB_METHOD=2, UNKNOWN_METHOD=3
+} compress_method_t;
 
 int
 tor_gzip_compress(char **out, size_t *out_len,
