@@ -1,6 +1,6 @@
 /* Copyright 2003 Roger Dingledine */
 /* See LICENSE for licensing information */
-/* $Id: torint.h,v 1.16 2004/12/05 11:58:30 arma Exp $ */
+/* $Id: torint.h,v 1.18 2005/03/03 06:37:54 nickm Exp $ */
 
 /**
  * \file torint.h
@@ -9,7 +9,7 @@
 
 #ifndef __TORINT_H
 #define __TORINT_H
-#define TORINT_H_ID "$Id: torint.h,v 1.16 2004/12/05 11:58:30 arma Exp $"
+#define TORINT_H_ID "$Id: torint.h,v 1.18 2005/03/03 06:37:54 nickm Exp $"
 
 #include "orconfig.h"
 
@@ -33,6 +33,9 @@
 #include <machine/limits.h>
 #endif
 #endif
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 
 #if (SIZEOF_INT8_T != 0)
 #define HAVE_INT8_T
@@ -232,6 +235,16 @@ typedef uint32_t uintptr_t;
 #endif
 #endif
 
+#ifndef TIME_MAX
+#if (SIZEOF_TIME_T == 4)
+#define TIME_MAX ((time_t)UINT32_MAX)
+#elif (SIZEOF_TIME_T == 8)
+#define TIME_MAX ((time_t)UINT64_MAX)
+#else
+#error "Can't define TIME_MAX"
+#endif
+#endif
+
 /* Any size_t larger than this amount is likely to be an underflow. */
 #define SIZE_T_CEILING (sizeof(char)<<(sizeof(size_t)*8 - 1))
 
