@@ -1,7 +1,7 @@
 /* Copyright 2003 Roger Dingledine. */
 /* See LICENSE for licensing information */
-/* $Id: tortls.c,v 1.82.2.3 2005/04/23 20:36:19 nickm Exp $ */
-const char tortls_c_id[] = "$Id: tortls.c,v 1.82.2.3 2005/04/23 20:36:19 nickm Exp $";
+/* $Id: tortls.c,v 1.89 2005/03/31 07:46:38 arma Exp $ */
+const char tortls_c_id[] = "$Id: tortls.c,v 1.89 2005/03/31 07:46:38 arma Exp $";
 
 /**
  * \file tortls.c
@@ -156,6 +156,17 @@ tor_tls_init(void) {
   }
 }
 
+void
+tor_tls_free_all(void)
+{
+  if (global_tls_context) {
+    SSL_CTX_free(global_tls_context->ctx);
+    SSL_CTX_free(global_tls_context->client_only_ctx);
+    tor_free(global_tls_context);
+    global_tls_context = NULL;
+  }
+}
+
 /** We need to give OpenSSL a callback to verify certificates. This is
  * it: We always accept peer certs and complete the handshake.  We
  * don't validate them until later.
@@ -419,14 +430,25 @@ tor_tls_new(int sock, int isServer, int use_no_cert)
   return result;
 }
 
+/** Return whether this tls initiated the connect (client) or
+ * received it (server). */
+int
+tor_tls_is_server(tor_tls *tls)
+{
+  tor_assert(tls);
+  return tls->isServer;
+}
+
 /** Release resources associated with a TLS object.  Does not close the
  * underlying file descriptor.
  */
 void
 tor_tls_free(tor_tls *tls)
 {
+  tor_assert(tls && tls->ssl);
   SSL_free(tls->ssl);
-  free(tls);
+  tls->ssl = NULL;
+  tor_free(tls);
 }
 
 /** Underlying function for TLS reading.  Reads up to <b>len</b>
@@ -500,17 +522,12 @@ tor_tls_handshake(tor_tls *tls)
   tor_assert(tls);
   tor_assert(tls->ssl);
   tor_assert(tls->state == TOR_TLS_ST_HANDSHAKE);
-  check_no_tls_errors();
   if (tls->isServer) {
     r = SSL_accept(tls->ssl);
   } else {
     r = SSL_connect(tls->ssl);
   }
   r = tor_tls_get_error(tls,r,0, "handshaking", LOG_INFO);
-  if (ERR_peek_error() != 0) {
-    tls_log_errors(LOG_WARN, "handshaking");
-    return TOR_TLS_ERROR;
-  }
   if (r == TOR_TLS_DONE) {
     tls->state = TOR_TLS_ST_OPEN;
   }
@@ -591,8 +608,10 @@ tor_tls_peer_has_cert(tor_tls *tls)
   return 1;
 }
 
-/** Return the nickname (if any) that the peer connected on <b>tls</b>
- * claims to have.
+/** Write the nickname (if any) that the peer connected on <b>tls</b>
+ * claims to have into the first <b>buflen</b> characters of <b>buf</b>.
+ * Truncate the nickname if it is longer than buflen-1 characters.  Always
+ * NUL-terminate.  Return 0 on success, -1 on failure.
  */
 int
 tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
@@ -617,7 +636,10 @@ tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
   if (lenout == -1)
     goto error;
   if (((int)strspn(buf, LEGAL_NICKNAME_CHARACTERS)) < lenout) {
-    log_fn(LOG_WARN, "Peer certificate nickname has illegal characters.");
+    log_fn(LOG_WARN, "Peer certificate nickname '%s' has illegal characters.",
+           buf);
+    if (strchr(buf, '.'))
+      log_fn(LOG_WARN, "  (Maybe it is not really running Tor at its advertised OR port.)");
     goto error;
   }
   X509_free(cert);
@@ -636,6 +658,7 @@ static void log_cert_lifetime(X509 *cert, const char *problem)
   char *s1=NULL, *s2=NULL;
   char mytime[33];
   time_t now = time(NULL);
+  struct tm tm;
 
   if (problem)
     log_fn(LOG_WARN,"Certificate %s: is your system clock set incorrectly?",
@@ -659,7 +682,7 @@ static void log_cert_lifetime(X509 *cert, const char *problem)
   BIO_get_mem_ptr(bio, &buf);
   s2 = tor_strndup(buf->data, buf->length);
 
-  strftime(mytime, 32, "%b %d %H:%M:%S %Y GMT", gmtime(&now));
+  strftime(mytime, 32, "%b %d %H:%M:%S %Y GMT", tor_gmtime_r(&now, &tm));
 
   log_fn(LOG_WARN, "(certificate lifetime runs from %s through %s. Your time is %s.)",s1,s2,mytime);
 
@@ -808,12 +831,14 @@ unsigned long tor_tls_get_n_bytes_written(tor_tls *tls)
 
 /** Implement assert_no_tls_errors: If there are any pending OpenSSL
  * errors, log an error message and assert(0). */
-void _check_no_tls_errors(const char *fname, int line)
+void _assert_no_tls_errors(const char *fname, int line)
 {
   if (ERR_peek_error() == 0)
     return;
-  log_fn(LOG_WARN, "Unhandled OpenSSL errors found at %s:%d: ",
+  log_fn(LOG_ERR, "Unhandled OpenSSL errors found at %s:%d: ",
          fname, line);
-  tls_log_errors(LOG_WARN, NULL);
+  tls_log_errors(LOG_ERR, NULL);
+
+  tor_assert(0);
 }
 
