@@ -1,10 +1,10 @@
 /* Copyright 2003 Roger Dingledine */
 /* See LICENSE for licensing information */
-/* $Id: tortls.h,v 1.22.2.2 2005/04/23 14:32:25 nickm Exp $ */
+/* $Id: tortls.h,v 1.25 2005/03/31 07:46:38 arma Exp $ */
 
 #ifndef _TORTLS_H
 #define _TORTLS_H
-#define TORTLS_H_ID "$Id: tortls.h,v 1.22.2.2 2005/04/23 14:32:25 nickm Exp $"
+#define TORTLS_H_ID "$Id: tortls.h,v 1.25 2005/03/31 07:46:38 arma Exp $"
 
 /**
  * \file tortls.h
@@ -24,9 +24,11 @@ typedef struct tor_tls_st tor_tls;
 #define TOR_TLS_WANTWRITE   -1
 #define TOR_TLS_DONE         0
 
+void tor_tls_free_all(void);
 int tor_tls_context_new(crypto_pk_env_t *rsa, int isServer,
                         const char *nickname, unsigned int key_lifetime);
 tor_tls *tor_tls_new(int sock, int is_server, int use_no_cert);
+int tor_tls_is_server(tor_tls *tls);
 void tor_tls_free(tor_tls *tls);
 int tor_tls_peer_has_cert(tor_tls *tls);
 int tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen);
@@ -43,9 +45,9 @@ unsigned long tor_tls_get_n_bytes_written(tor_tls *tls);
 
 /* Log and abort if there are unhandled TLS errors in OpenSSL's error stack.
  */
-#define check_no_tls_errors() _check_no_tls_errors(_SHORT_FILE_,__LINE__)
+#define assert_no_tls_errors() _assert_no_tls_errors(_SHORT_FILE_,__LINE__)
 
-void _check_no_tls_errors(const char *fname, int line);
+void _assert_no_tls_errors(const char *fname, int line);
 
 #endif
 
