@@ -1,7 +1,7 @@
 /* Copyright 2003 Roger Dingledine */
 /* See LICENSE for licensing information */
-/* $Id: util.c,v 1.195.2.2 2005/03/16 00:14:39 nickm Exp $ */
-const char util_c_id[] = "$Id: util.c,v 1.195.2.2 2005/03/16 00:14:39 nickm Exp $";
+/* $Id: util.c,v 1.206 2005/03/17 12:38:35 nickm Exp $ */
+const char util_c_id[] = "$Id: util.c,v 1.206 2005/03/17 12:38:35 nickm Exp $";
 
 /**
  * \file util.c
@@ -62,9 +62,6 @@ const char util_c_id[] = "$Id: util.c,v 1.195.2.2 2005/03/16 00:14:39 nickm Exp
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -91,22 +88,38 @@ const char util_c_id[] = "$Id: util.c,v 1.195.2.2 2005/03/16 00:14:39 nickm Exp
 /* =====
  * Memory management
  * ===== */
+#ifdef USE_DMALLOC
+ #include <dmalloc.h>
+#else
+ #define dmalloc_strdup(file, line, string, xalloc_b) strdup(string)
+
+ #define dmalloc_malloc(file, line, size, func_id, alignment, xalloc_b) malloc(size)
+ #define DMALLOC_FUNC_MALLOC 0
+
+ #define dmalloc_realloc(file, line, old_pnt, new_size, func_id, xalloc_b) realloc((old_pnt), (new_size))
+ #define DMALLOC_FUNC_REALLOC 0
+#endif
 
 /** Allocate a chunk of <b>size</b> bytes of memory, and return a pointer to
  * result.  On error, log and terminate the process.  (Same as malloc(size),
  * but never returns NULL.)
+ *
+ * <b>file</b> and <b>line</b> are used if dmalloc is enabled, and
+ * ignored otherwise.
  */
-void *tor_malloc(size_t size) {
+void *_tor_malloc(const char *file, const int line, size_t size) {
   void *result;
 
   /* Some libcs don't do the right thing on size==0. Override them. */
   if (size==0) {
     size=1;
   }
-  result = malloc(size);
+  result = dmalloc_malloc(file, line, size, DMALLOC_FUNC_MALLOC, 0, 0);
 
   if (!result) {
     log_fn(LOG_ERR, "Out of memory. Dying.");
+    /* XXX if these functions die within a worker process, they won't
+     * call spawn_exit */
     exit(1);
   }
 //  memset(result,'X',size); /* deadbeef to encourage bugs */
@@ -117,8 +130,8 @@ void *tor_malloc(size_t size) {
  * zero bytes, and return a pointer to the result.  Log and terminate
  * the process on error.  (Same as calloc(size,1), but never returns NULL.)
  */
-void *tor_malloc_zero(size_t size) {
-  void *result = tor_malloc(size);
+void *_tor_malloc_zero(const char *file, const int line, size_t size) {
+  void *result = _tor_malloc(file, line, size);
   memset(result, 0, size);
   return result;
 }
@@ -127,10 +140,10 @@ void *tor_malloc_zero(size_t size) {
  * bytes long; return the new memory block.  On error, log and
  * terminate. (Like realloc(ptr,size), but never returns NULL.)
  */
-void *tor_realloc(void *ptr, size_t size) {
+void *_tor_realloc(const char *file, const int line, void *ptr, size_t size) {
   void *result;
 
-  result = realloc(ptr, size);
+  result = dmalloc_realloc(file, line, ptr, size, DMALLOC_FUNC_REALLOC, 0);
   if (!result) {
     log_fn(LOG_ERR, "Out of memory. Dying.");
     exit(1);
@@ -142,11 +155,11 @@ void *tor_realloc(void *ptr, size_t size) {
  * error, log and terminate.  (Like strdup(s), but never returns
  * NULL.)
  */
-char *tor_strdup(const char *s) {
+char *_tor_strdup(const char *file, const int line, const char *s) {
   char *dup;
   tor_assert(s);
 
-  dup = strdup(s);
+  dup = dmalloc_strdup(file, line, s, 0);
   if (!dup) {
     log_fn(LOG_ERR,"Out of memory. Dying.");
     exit(1);
@@ -160,10 +173,10 @@ char *tor_strdup(const char *s) {
  * always NUL-terminated.  (Like strndup(s,n), but never returns
  * NULL.)
  */
-char *tor_strndup(const char *s, size_t n) {
+char *_tor_strndup(const char *file, const int line, const char *s, size_t n) {
   char *dup;
   tor_assert(s);
-  dup = tor_malloc(n+1);
+  dup = _tor_malloc(file, line, n+1);
   /* Performance note: Ordinarily we prefer strlcpy to strncpy.  But
    * this function gets called a whole lot, and platform strncpy is
    * much faster than strlcpy when strlen(s) is much longer than n.
@@ -218,6 +231,8 @@ int tor_strpartition(char *dest, size_t dest_len,
   tor_assert(dest_len < SIZE_T_CEILING);
   len_in = strlen(s);
   len_ins = strlen(insert);
+  tor_assert(len_in < SIZE_T_CEILING);
+  tor_assert(len_in/n < SIZE_T_CEILING/len_ins); /* avoid overflow */
   len_out = len_in + (len_in/n)*len_ins;
   is_even = (len_in%n) == 0;
   switch (rule)
@@ -588,15 +603,17 @@ static const char *MONTH_NAMES[] =
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 
 void format_rfc1123_time(char *buf, time_t t) {
-  struct tm *tm = gmtime(&t);
+  struct tm tm;
 
-  strftime(buf, RFC1123_TIME_LEN+1, "___, %d ___ %Y %H:%M:%S GMT", tm);
-  tor_assert(tm->tm_wday >= 0);
-  tor_assert(tm->tm_wday <= 6);
-  memcpy(buf, WEEKDAY_NAMES[tm->tm_wday], 3);
-  tor_assert(tm->tm_wday >= 0);
-  tor_assert(tm->tm_mon <= 11);
-  memcpy(buf+8, MONTH_NAMES[tm->tm_mon], 3);
+  tor_gmtime_r(&t, &tm);
+
+  strftime(buf, RFC1123_TIME_LEN+1, "___, %d ___ %Y %H:%M:%S GMT", &tm);
+  tor_assert(tm.tm_wday >= 0);
+  tor_assert(tm.tm_wday <= 6);
+  memcpy(buf, WEEKDAY_NAMES[tm.tm_wday], 3);
+  tor_assert(tm.tm_wday >= 0);
+  tor_assert(tm.tm_mon <= 11);
+  memcpy(buf+8, MONTH_NAMES[tm.tm_mon], 3);
 }
 
 int parse_rfc1123_time(const char *buf, time_t *t) {
@@ -634,11 +651,13 @@ int parse_rfc1123_time(const char *buf, time_t *t) {
 }
 
 void format_local_iso_time(char *buf, time_t t) {
-  strftime(buf, ISO_TIME_LEN+1, "%Y-%m-%d %H:%M:%S", localtime(&t));
+  struct tm tm;
+  strftime(buf, ISO_TIME_LEN+1, "%Y-%m-%d %H:%M:%S", tor_localtime_r(&t, &tm));
 }
 
 void format_iso_time(char *buf, time_t t) {
-  strftime(buf, ISO_TIME_LEN+1, "%Y-%m-%d %H:%M:%S", gmtime(&t));
+  struct tm tm;
+  strftime(buf, ISO_TIME_LEN+1, "%Y-%m-%d %H:%M:%S", tor_gmtime_r(&t, &tm));
 }
 
 int parse_iso_time(const char *cp, time_t *t) {
@@ -693,8 +712,7 @@ int write_all(int fd, const char *buf, size_t count, int isSocket) {
 }
 
 /** Read from <b>fd</b> to <b>buf</b>, until we get <b>count</b> bytes
- * or reach the end of the file.
- * isSocket must be 1 if fd
+ * or reach the end of the file. <b>isSocket</b> must be 1 if fd
  * was returned by socket() or accept(), and 0 if fd was returned by
  * open().  Return the number of bytes read, or -1 on error. Only use
  * if fd is a blocking fd. */
@@ -853,39 +871,53 @@ write_str_to_file(const char *fname, const char *str, int bin)
 int write_bytes_to_file(const char *fname, const char *str, size_t len,
                         int bin)
 {
-  char tempname[1024];
+  size_t tempname_len;
+  char *tempname;
   int fd;
   int result;
-  if ((strlcpy(tempname,fname,1024) >= 1024) ||
-      (strlcat(tempname,".tmp",1024) >= 1024)) {
-    log(LOG_WARN, "Filename %s.tmp too long (>1024 chars)", fname);
-    return -1;
+  tempname_len = strlen(fname)+16;
+  tor_assert(tempname_len > strlen(fname)); /*check for overflow*/
+  tempname = tor_malloc(tempname_len);
+  if (tor_snprintf(tempname, tempname_len, "%s.tmp", fname)<0) {
+    log(LOG_WARN, "Failed to generate filename");
+    goto err;
   }
   if ((fd = open(tempname, O_WRONLY|O_CREAT|O_TRUNC|(bin?O_BINARY:O_TEXT), 0600))
       < 0) {
     log(LOG_WARN, "Couldn't open %s for writing: %s", tempname,
         strerror(errno));
-    return -1;
+    goto err;
   }
   result = write_all(fd, str, len, 0);
   if (result < 0 || (size_t)result != len) {
     log(LOG_WARN, "Error writing to %s: %s", tempname, strerror(errno));
     close(fd);
-    return -1;
+    goto err;
   }
   if (close(fd)) {
     log(LOG_WARN,"Error flushing to %s: %s", tempname, strerror(errno));
-    return -1;
+    goto err;
   }
   if (replace_file(tempname, fname)) {
     log(LOG_WARN, "Error replacing %s: %s", fname, strerror(errno));
-    return -1;
+    goto err;
   }
+  tor_free(tempname);
   return 0;
+ err:
+  tor_free(tempname);
+  return -1;
 }
 
-/** Read the contents of <b>filename</b> into a newly allocated string; return the
- * string on success or NULL on failure.
+/** Read the contents of <b>filename</b> into a newly allocated
+ * string; return the string on success or NULL on failure.
+ */
+/*
+ * This function <em>may</em> return an erroneous result if the file
+ * is modified while it is running, but must not crash or overflow.
+ * Right now, the error case occurs when the file length grows between
+ * the call to stat and the call to read_all: the resulting string will
+ * be truncated.
  */
 char *read_file_to_str(const char *filename, int bin) {
   int fd; /* router file */
@@ -1097,45 +1129,6 @@ int is_local_IP(uint32_t ip) {
   return is_internal_IP(ip);
 }
 
-/** Similar behavior to Unix gethostbyname: resolve <b>name</b>, and set
- * *addr to the proper IP address, in network byte order.  Returns 0
- * on success, -1 on failure; 1 on transient failure.
- *
- * (This function exists because standard windows gethostbyname
- * doesn't treat raw IP addresses properly.)
- */
-int tor_lookup_hostname(const char *name, uint32_t *addr)
-{
-  /* Perhaps eventually this should be replaced by a tor_getaddrinfo or
-   * something.
-   */
-  struct in_addr iaddr;
-  struct hostent *ent;
-  tor_assert(addr);
-  if (!*name) {
-    /* Empty address is an error. */
-    return -1;
-  } else if (tor_inet_aton(name, &iaddr)) {
-    /* It's an IP. */
-    memcpy(addr, &iaddr.s_addr, 4);
-    return 0;
-  } else {
-    ent = gethostbyname(name);
-    if (ent) {
-      /* break to remind us if we move away from IPv4 */
-      tor_assert(ent->h_length == 4);
-      memcpy(addr, ent->h_addr, 4);
-      return 0;
-    }
-    memset(addr, 0, 4);
-#ifdef MS_WINDOWS
-    return (WSAGetLastError() == WSATRY_AGAIN) ? 1 : -1;
-#else
-    return (h_errno == TRY_AGAIN) ? 1 : -1;
-#endif
-  }
-}
-
 /** Parse a string of the form "host[:port]" from <b>addrport</b>.  If
  * <b>address</b> is provided, set *<b>address</b> to a copy of the
  * host portion of the string.  If <b>addr</b> is provided, try to
@@ -1295,6 +1288,37 @@ parse_addr_and_port_range(const char *s, uint32_t *addr_out,
   return -1;
 }
 
+/** Given an IPv4 address <b>in</b> (in network order, as usual),
+ * write it as a string into the <b>buf_len</b>-byte buffer in
+ * <b>buf</b>.
+ */
+int
+tor_inet_ntoa(struct in_addr *in, char *buf, size_t buf_len)
+{
+  uint32_t a = ntohl(in->s_addr);
+  return tor_snprintf(buf, buf_len, "%d.%d.%d.%d",
+                      (int)(uint8_t)((a>>24)&0xff),
+                      (int)(uint8_t)((a>>16)&0xff),
+                      (int)(uint8_t)((a>>8 )&0xff),
+                      (int)(uint8_t)((a    )&0xff));
+}
+
+/* Return true iff <b>name</b> looks like it might be a hostname or IP
+ * address of some kind. */
+int
+is_plausible_address(const char *name)
+{
+  const char *cp;
+  tor_assert(name);
+  /* We could check better here. */
+  for (cp=name; *cp; cp++) {
+    if (*cp != '.' && *cp != '-' && !TOR_ISALNUM(*cp))
+      return 0;
+  }
+
+  return 1;
+}
+
 /* =====
  * Process helpers
  * ===== */
@@ -1418,4 +1442,3 @@ void write_pidfile(char *filename) {
   }
 #endif
 }
-
