@@ -1,6 +1,6 @@
 /* Copyright 2003-2004 Roger Dingledine; Copyright 2004 Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.h,v 1.124.2.1 2004/12/22 02:57:32 nickm Exp $ */
+/* $Id: util.h,v 1.130 2005/02/25 20:46:13 nickm Exp $ */
 
 /**
  * \file util.h
@@ -9,7 +9,7 @@
 
 #ifndef __UTIL_H
 #define __UTIL_H
-#define UTIL_H_ID "$Id: util.h,v 1.124.2.1 2004/12/22 02:57:32 nickm Exp $"
+#define UTIL_H_ID "$Id: util.h,v 1.130 2005/02/25 20:46:13 nickm Exp $"
 
 #include "orconfig.h"
 #include "torint.h"
@@ -47,13 +47,19 @@
 #endif
 
 /* Memory management */
-void *tor_malloc(size_t size);
-void *tor_malloc_zero(size_t size);
-void *tor_realloc(void *ptr, size_t size);
-char *tor_strdup(const char *s);
-char *tor_strndup(const char *s, size_t n);
+void *_tor_malloc(const char *file, const int line, size_t size);
+void *_tor_malloc_zero(const char *file, const int line, size_t size);
+void *_tor_realloc(const char *file, const int line, void *ptr, size_t size);
+char *_tor_strdup(const char *file, const int line, const char *s);
+char *_tor_strndup(const char *file, const int line, const char *s, size_t n);
 #define tor_free(p) do { if (p) {free(p); (p)=NULL;} } while (0)
 
+#define tor_malloc(size)       _tor_malloc(_SHORT_FILE_, __LINE__, size)
+#define tor_malloc_zero(size)  _tor_malloc_zero(_SHORT_FILE_, __LINE__, size)
+#define tor_realloc(ptr, size) _tor_realloc(_SHORT_FILE_, __LINE__, ptr, size)
+#define tor_strdup(s)          _tor_strdup(_SHORT_FILE_, __LINE__, s)
+#define tor_strndup(s, n)      _tor_strndup(_SHORT_FILE_, __LINE__, s, n)
+
 /* String manipulation */
 #define HEX_CHARACTERS "0123456789ABCDEFabcdef"
 void tor_strlower(char *s);
@@ -115,12 +121,14 @@ char *expand_filename(const char *filename);
 /* Net helpers */
 int is_internal_IP(uint32_t ip);
 int is_local_IP(uint32_t ip);
-int tor_lookup_hostname(const char *name, uint32_t *addr);
 int parse_addr_port(const char *addrport, char **address, uint32_t *addr,
                     uint16_t *port);
 int parse_addr_and_port_range(const char *s, uint32_t *addr_out,
                               uint32_t *mask_out, uint16_t *port_min_out,
                               uint16_t *port_max_out);
+#define INET_NTOA_BUF_LEN 16
+int tor_inet_ntoa(struct in_addr *in, char *buf, size_t buf_len);
+int is_plausible_address(const char *name);
 
 /* Process helpers */
 void start_daemon(const char *desired_cwd);
