@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.75.2.2 2005/03/27 07:17:39 arma Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.75.2.2 2005/03/27 07:17:39 arma Exp $";
+/* $Id: circuitbuild.c,v 1.102 2005/04/01 09:23:51 arma Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.102 2005/04/01 09:23:51 arma Exp $";
 
 /**
  * \file circuitbuild.c
@@ -19,13 +19,14 @@ extern circuit_t *global_circuitlist;
 
 /********* END VARIABLES ************/
 
-static int
-circuit_deliver_create_cell(circuit_t *circ, char *payload);
-static cpath_build_state_t *onion_new_cpath_build_state(uint8_t purpose,
-                                                   const char *exit_digest);
-static int onion_extend_cpath(crypt_path_t **head_ptr,
-                       cpath_build_state_t *state, routerinfo_t **router_out);
+static int circuit_deliver_create_cell(circuit_t *circ, char *payload);
+static int onion_pick_cpath_exit(circuit_t *circ, routerinfo_t *exit);
+static crypt_path_t *onion_next_hop_in_cpath(crypt_path_t *cpath);
+static int onion_next_router_in_cpath(circuit_t *circ, routerinfo_t **router);
+static int onion_extend_cpath(uint8_t purpose, crypt_path_t **head_ptr,
+                              cpath_build_state_t *state);
 static int count_acceptable_routers(smartlist_t *routers);
+static int onion_append_hop(crypt_path_t **head_ptr, routerinfo_t *choice);
 
 /** Iterate over values of circ_id, starting from conn-\>next_circ_id,
  * and with the high bit specified by circ_id_type (see
@@ -81,7 +82,9 @@ circuit_list_path(circuit_t *circ, int verbose)
   elements = smartlist_create();
 
   if (verbose) {
-    tor_snprintf(buf, sizeof(buf)-1, "circ (length %d, exit %s):",
+    tor_snprintf(buf, sizeof(buf)-1, "%s%s circ (length %d, exit %s):",
+                 circ->build_state->is_internal ? "internal" : "exit",
+                 circ->build_state->need_uptime ? " (high-uptime)" : "",
                  circ->build_state->desired_path_len,
                  circ->build_state->chosen_exit_name);
     smartlist_add(elements, tor_strdup(buf));
@@ -226,42 +229,78 @@ void circuit_dump_by_conn(connection_t *conn, int severity) {
   }
 }
 
-/** Build a new circuit for <b>purpose</b>. If <b>exit_digest</b>
+/** Pick all the entries in our cpath. Stop and return 0 when we're
+ * happy, or return -1 if an error occurs. */
+static int
+onion_populate_cpath(circuit_t *circ) {
+  int r;
+again:
+  r = onion_extend_cpath(circ->purpose, &circ->cpath, circ->build_state);
+//    || !CIRCUIT_IS_ORIGIN(circ)) { // wtf? -rd
+  if (r < 0) {
+    log_fn(LOG_INFO,"Generating cpath hop failed.");
+    return -1;
+  }
+  if (r == 0)
+    goto again;
+  return 0; /* if r == 1 */
+}
+
+/** Create and return a new circuit. Initialize its purpose and
+ * build-state based on our arguments. */
+circuit_t *
+circuit_init(uint8_t purpose, int need_uptime, int need_capacity, int internal) {
+  circuit_t *circ = circuit_new(0, NULL); /* sets circ->p_circ_id and circ->p_conn */
+  circ->state = CIRCUIT_STATE_OR_WAIT;
+  circ->build_state = tor_malloc_zero(sizeof(cpath_build_state_t));
+  circ->build_state->need_uptime = need_uptime;
+  circ->build_state->need_capacity = need_capacity;
+  circ->build_state->is_internal = internal;
+  circ->purpose = purpose;
+  return circ;
+}
+
+/** Build a new circuit for <b>purpose</b>. If <b>exit</b>
  * is defined, then use that as your exit router, else choose a suitable
  * exit node.
  *
  * Also launch a connection to the first OR in the chosen path, if
  * it's not open already.
  */
-circuit_t *circuit_establish_circuit(uint8_t purpose,
-                                     const char *exit_digest) {
-  routerinfo_t *firsthop;
-  connection_t *n_conn;
+circuit_t *
+circuit_establish_circuit(uint8_t purpose, routerinfo_t *exit,
+                          int need_uptime, int need_capacity, int internal) {
   circuit_t *circ;
 
-  circ = circuit_new(0, NULL); /* sets circ->p_circ_id and circ->p_conn */
-  circ->state = CIRCUIT_STATE_OR_WAIT;
-  circ->build_state = onion_new_cpath_build_state(purpose, exit_digest);
-  circ->purpose = purpose;
+  circ = circuit_init(purpose, need_uptime, need_capacity, internal);
 
-  if (! circ->build_state) {
-    log_fn(LOG_INFO,"Generating cpath failed.");
-    circuit_mark_for_close(circ);
-    return NULL;
-  }
-
-  if (onion_extend_cpath(&circ->cpath, circ->build_state, &firsthop)<0 ||
-      !CIRCUIT_IS_ORIGIN(circ)) {
-    log_fn(LOG_INFO,"Generating first cpath hop failed.");
+  if (onion_pick_cpath_exit(circ, exit) < 0 ||
+      onion_populate_cpath(circ) < 0) {
     circuit_mark_for_close(circ);
     return NULL;
   }
 
   control_event_circuit_status(circ, CIRC_EVENT_LAUNCHED);
 
-  /* now see if we're already connected to the first OR in 'route' */
+  if (circuit_handle_first_hop(circ) < 0) {
+    circuit_mark_for_close(circ);
+    return NULL;
+  }
+  return circ;
+}
 
+/** Start establishing the first hop of our circuit. Figure out what
+ * OR we should connect to, and if necessary start the connection to
+ * it. If we're already connected, then send the 'create' cell.
+ * Return 0 for ok, -1 if circ should be marked-for-close. */
+int circuit_handle_first_hop(circuit_t *circ) {
+  routerinfo_t *firsthop;
+  connection_t *n_conn;
+
+  onion_next_router_in_cpath(circ, &firsthop);
   tor_assert(firsthop);
+
+  /* now see if we're already connected to the first OR in 'route' */
   log_fn(LOG_DEBUG,"Looking for firsthop '%s:%u'",
       firsthop->address,firsthop->or_port);
   /* imprint the circuit with its future n_conn->id */
@@ -277,8 +316,7 @@ circuit_t *circuit_establish_circuit(uint8_t purpose,
                                      firsthop->identity_digest);
       if (!n_conn) { /* connect failed, forget the whole thing */
         log_fn(LOG_INFO,"connect to firsthop failed. Closing.");
-        circuit_mark_for_close(circ);
-        return NULL;
+        return -1;
       }
     }
 
@@ -286,7 +324,7 @@ circuit_t *circuit_establish_circuit(uint8_t purpose,
     /* return success. The onion/circuit/etc will be taken care of automatically
      * (may already have been) whenever n_conn reaches OR_CONN_STATE_OPEN.
      */
-    return circ;
+    return 0;
   } else { /* it's already open. use it. */
     circ->n_addr = n_conn->addr;
     circ->n_port = n_conn->port;
@@ -294,11 +332,10 @@ circuit_t *circuit_establish_circuit(uint8_t purpose,
     log_fn(LOG_DEBUG,"Conn open. Delivering first onion skin.");
     if (circuit_send_next_onion_skin(circ) < 0) {
       log_fn(LOG_INFO,"circuit_send_next_onion_skin failed.");
-      circuit_mark_for_close(circ);
-      return NULL;
+      return -1;
     }
   }
-  return circ;
+  return 0;
 }
 
 /** Find circuits that are waiting on <b>or_conn</b> to become open,
@@ -420,25 +457,29 @@ int circuit_send_next_onion_skin(circuit_t *circ) {
     tor_assert(circ->cpath->state == CPATH_STATE_OPEN);
     tor_assert(circ->state == CIRCUIT_STATE_BUILDING);
     log_fn(LOG_DEBUG,"starting to send subsequent skin.");
-    r = onion_extend_cpath(&circ->cpath, circ->build_state, &router);
-    if (r==1) {
+    r = onion_next_router_in_cpath(circ, &router);
+    if (r > 0) {
       /* done building the circuit. whew. */
       circ->state = CIRCUIT_STATE_OPEN;
       log_fn(LOG_INFO,"circuit built!");
       circuit_reset_failure_count(0);
       if (!has_completed_circuit) {
+        or_options_t *options = get_options();
         has_completed_circuit=1;
         log_fn(LOG_NOTICE,"Tor has successfully opened a circuit. Looks like it's working.");
         /* XXX009 Log a count of known routers here */
+        if (server_mode(options) && !check_whether_orport_reachable())
+          log_fn(LOG_NOTICE,"Now checking whether ORPort %s%s reachable... (this may take several minutes)",
+                 options->DirPort ? "and DirPort " : "",
+                 options->DirPort ? "are" : "is");
       }
       circuit_rep_hist_note_result(circ);
       circuit_has_opened(circ); /* do other actions as necessary */
       return 0;
-    } else if (r<0) {
-      log_fn(LOG_INFO,"Unable to extend circuit path.");
+    } else if (r < 0) {
       return -1;
     }
-    hop = circ->cpath->prev;
+    hop = onion_next_hop_in_cpath(circ->cpath);
 
     *(uint32_t*)payload = htonl(hop->addr);
     *(uint16_t*)(payload+4) = htons(hop->port);
@@ -464,6 +505,15 @@ int circuit_send_next_onion_skin(circuit_t *circ) {
   return 0;
 }
 
+/** Our clock just jumped forward by <b>seconds_elapsed</b>. Assume
+ * something has also gone wrong with our network: notify the user,
+ * and abandon all not-yet-used circuits. */
+void circuit_note_clock_jumped(int seconds_elapsed) {
+  log_fn(LOG_NOTICE,"Your clock just jumped %d seconds forward; assuming established circuits no longer work.", seconds_elapsed);
+  has_completed_circuit=0; /* so it'll log when it works again */
+  circuit_mark_all_unused_circs();
+}
+
 /** Take the 'extend' cell, pull out addr/port plus the onion skin. Make
  * sure we're connected to the next hop, and pass it the onion skin in
  * a create cell.
@@ -473,7 +523,6 @@ int circuit_extend(cell_t *cell, circuit_t *circ) {
   relay_header_t rh;
   char *onionskin;
   char *id_digest=NULL;
-  routerinfo_t *router;
 
   if (circ->n_conn) {
     log_fn(LOG_WARN,"n_conn already set. Bug/attack. Closing.");
@@ -499,11 +548,11 @@ int circuit_extend(cell_t *cell, circuit_t *circ) {
      * router twice in a row in the path. I think that's ok.
      */
     struct in_addr in;
+    char tmpbuf[INET_NTOA_BUF_LEN];
     in.s_addr = htonl(circ->n_addr);
+    tor_inet_ntoa(&in,tmpbuf,sizeof(tmpbuf));
     log_fn(LOG_INFO,"Next router (%s:%d) not connected. Connecting.",
-           inet_ntoa(in), circ->n_port);
-
-    router = router_get_by_digest(id_digest);
+           tmpbuf, circ->n_port);
 
     memcpy(circ->onionskin, onionskin, ONIONSKIN_CHALLENGE_LEN);
     circ->state = CIRCUIT_STATE_OR_WAIT;
@@ -610,10 +659,8 @@ int circuit_finish_handshake(circuit_t *circ, char *reply) {
   if (circ->cpath->state == CPATH_STATE_AWAITING_KEYS)
     hop = circ->cpath;
   else {
-    for (hop=circ->cpath->next;
-        hop != circ->cpath && hop->state == CPATH_STATE_OPEN;
-        hop=hop->next) ;
-    if (hop == circ->cpath) { /* got an extended when we're all done? */
+    hop = onion_next_hop_in_cpath(circ->cpath);
+    if (!hop) { /* got an extended when we're all done? */
       log_fn(LOG_WARN,"got extended when circ already built? Closing.");
       return -1;
     }
@@ -636,7 +683,7 @@ int circuit_finish_handshake(circuit_t *circ, char *reply) {
   }
 
   hop->state = CPATH_STATE_OPEN;
-  log_fn(LOG_INFO,"Finished building circuit:");
+  log_fn(LOG_INFO,"Finished building circuit hop:");
   circuit_log_path(LOG_INFO,circ);
   control_event_circuit_status(circ, CIRC_EVENT_EXTENDED);
 
@@ -676,8 +723,7 @@ int circuit_truncated(circuit_t *circ, crypt_path_t *layer) {
         /* no need to send 'end' relay cells,
          * because the other side's already dead
          */
-        stream->has_sent_end = 1;
-        connection_mark_for_close(stream);
+        connection_close_unattached_ap(stream, END_STREAM_REASON_DESTROY);
       }
     }
 
@@ -698,6 +744,7 @@ int onionskin_answer(circuit_t *circ, unsigned char *payload, unsigned char *key
   crypt_path_t *tmp_cpath;
 
   tmp_cpath = tor_malloc_zero(sizeof(crypt_path_t));
+  tmp_cpath->magic = CRYPT_PATH_MAGIC;
 
   memset(&cell, 0, sizeof(cell_t));
   cell.command = CELL_CREATED;
@@ -720,6 +767,7 @@ int onionskin_answer(circuit_t *circ, unsigned char *payload, unsigned char *key
   circ->n_crypto = tmp_cpath->f_crypto;
   circ->p_digest = tmp_cpath->b_digest;
   circ->p_crypto = tmp_cpath->b_crypto;
+  tmp_cpath->magic = 0;
   tor_free(tmp_cpath);
 
   memcpy(circ->handshake_digest, cell.payload+DH_KEY_LEN, DIGEST_LEN);
@@ -727,6 +775,14 @@ int onionskin_answer(circuit_t *circ, unsigned char *payload, unsigned char *key
   connection_or_write_cell_to_buf(&cell, circ->p_conn);
   log_fn(LOG_DEBUG,"Finished sending 'created' cell.");
 
+  if (!is_local_IP(circ->p_conn->addr) &&
+      tor_tls_is_server(circ->p_conn->tls)) {
+    /* record that we could process create cells from a non-local conn
+     * that we didn't initiate; presumably this means that create cells
+     * can reach us too. */
+    router_orport_found_reachable();
+  }
+
   return 0;
 }
 
@@ -749,6 +805,8 @@ static int new_route_len(double cw, uint8_t purpose, smartlist_t *routers) {
 #else
   if (purpose == CIRCUIT_PURPOSE_C_GENERAL)
     routelen = 3;
+  else if (purpose == CIRCUIT_PURPOSE_TESTING)
+    routelen = 3;
   else if (purpose == CIRCUIT_PURPOSE_C_INTRODUCING)
     routelen = 4;
   else if (purpose == CIRCUIT_PURPOSE_C_ESTABLISH_REND)
@@ -759,6 +817,9 @@ static int new_route_len(double cw, uint8_t purpose, smartlist_t *routers) {
     routelen = 4;
   else {
     log_fn(LOG_WARN,"Bug: unhandled purpose %d", purpose);
+#ifdef TOR_FRAGILE
+    tor_assert(0);
+#endif
     return -1;
   }
 #endif
@@ -811,13 +872,24 @@ circuit_get_unhandled_ports(time_t now) {
 
 /** Return 1 if we already have circuits present or on the way for
  * all anticipated ports. Return 0 if we should make more.
+ *
+ * If we're returning 0, set need_uptime and need_capacity to
+ * indicate any requirements that the unhandled ports have.
  */
 int
-circuit_all_predicted_ports_handled(time_t now) {
-  int enough;
+circuit_all_predicted_ports_handled(time_t now, int *need_uptime,
+                                    int *need_capacity) {
+  int i, enough;
+  uint16_t *port;
   smartlist_t *sl = circuit_get_unhandled_ports(now);
+  smartlist_t *LongLivedServices = get_options()->LongLivedPorts;
   enough = (smartlist_len(sl) == 0);
-  SMARTLIST_FOREACH(sl, uint16_t *, cp, tor_free(cp));
+  for (i = 0; i < smartlist_len(sl); ++i) {
+    port = smartlist_get(sl, i);
+    if (smartlist_string_num_isin(LongLivedServices, *port))
+      *need_uptime = 1;
+    tor_free(port);
+  }
   smartlist_free(sl);
   return enough;
 }
@@ -831,10 +903,11 @@ router_handles_some_port(routerinfo_t *router, smartlist_t *needed_ports) {
   uint16_t port;
 
   for (i = 0; i < smartlist_len(needed_ports); ++i) {
+    addr_policy_result_t r;
     port = *(uint16_t *)smartlist_get(needed_ports, i);
     tor_assert(port);
-    if (router_compare_addr_to_addr_policy(0, port, router->exit_policy) !=
-          ADDR_POLICY_REJECTED)
+    r = router_compare_addr_to_addr_policy(0, port, router->exit_policy);
+    if (r != ADDR_POLICY_REJECTED && r != ADDR_POLICY_PROBABLY_REJECTED)
       return 1;
   }
   return 0;
@@ -845,6 +918,17 @@ router_handles_some_port(routerinfo_t *router, smartlist_t *needed_ports) {
  */
 #define MIN_CIRCUITS_HANDLING_STREAM 2
 
+static int
+ap_stream_wants_exit_attention(connection_t *conn) {
+  if (conn->type == CONN_TYPE_AP &&
+      conn->state == AP_CONN_STATE_CIRCUIT_WAIT &&
+      !conn->marked_for_close &&
+      !connection_edge_is_rendezvous_stream(conn) &&
+      !circuit_stream_is_being_handled(conn, 0, MIN_CIRCUITS_HANDLING_STREAM))
+    return 1;
+  return 0;
+}
+
 /** Return a pointer to a suitable router to be the exit node for the
  * general-purpose circuit we're about to build.
  *
@@ -853,7 +937,9 @@ router_handles_some_port(routerinfo_t *router, smartlist_t *needed_ports) {
  *
  * Return NULL if we can't find any suitable routers.
  */
-static routerinfo_t *choose_good_exit_server_general(routerlist_t *dir)
+static routerinfo_t *
+choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
+                                int need_capacity)
 {
   int *n_supported;
   int i, j;
@@ -875,10 +961,7 @@ static routerinfo_t *choose_good_exit_server_general(routerlist_t *dir)
    * We use this for log messages now, but in the future we may depend on it.
    */
   for (i = 0; i < n_connections; ++i) {
-    if (carray[i]->type == CONN_TYPE_AP &&
-        carray[i]->state == AP_CONN_STATE_CIRCUIT_WAIT &&
-        !carray[i]->marked_for_close &&
-        !circuit_stream_is_being_handled(carray[i], 0, MIN_CIRCUITS_HANDLING_STREAM))
+    if (ap_stream_wants_exit_attention(carray[i]))
       ++n_pending_connections;
   }
 //  log_fn(LOG_DEBUG, "Choosing exit node; %d connections are pending",
@@ -905,9 +988,13 @@ static routerinfo_t *choose_good_exit_server_general(routerlist_t *dir)
 //             router->nickname, i);
       continue; /* skip routers that are known to be down */
     }
+    if (router_is_unreliable(router, need_uptime, need_capacity)) {
+      n_supported[i] = -1;
+      continue; /* skip routers that are not suitable */
+    }
     if (!router->is_verified &&
         (!(options->_AllowUnverified & ALLOW_UNVERIFIED_EXIT) ||
-         router_is_unreliable_router(router, 1, 1))) {
+         router_is_unreliable(router, 1, 1))) {
       /* if it's unverified, and either we don't want it or it's unsuitable */
       n_supported[i] = -1;
 //      log_fn(LOG_DEBUG,"Skipping node %s (index %d) -- unverified router.",
@@ -928,10 +1015,7 @@ static routerinfo_t *choose_good_exit_server_general(routerlist_t *dir)
     }
     n_supported[i] = 0;
     for (j = 0; j < n_connections; ++j) { /* iterate over connections */
-      if (carray[j]->type != CONN_TYPE_AP ||
-          carray[j]->state != AP_CONN_STATE_CIRCUIT_WAIT ||
-          carray[j]->marked_for_close ||
-          circuit_stream_is_being_handled(carray[j], 0, MIN_CIRCUITS_HANDLING_STREAM))
+      if (!ap_stream_wants_exit_attention(carray[j]))
         continue; /* Skip everything but APs in CIRCUIT_WAIT */
       if (connection_ap_can_use_exit(carray[j], router)) {
         ++n_supported[i];
@@ -1035,64 +1119,93 @@ static routerinfo_t *choose_good_exit_server_general(routerlist_t *dir)
  * For client-side rendezvous circuits, choose a random node, weighted
  * toward the preferences in 'options'.
  */
-static routerinfo_t *choose_good_exit_server(uint8_t purpose, routerlist_t *dir)
+static routerinfo_t *
+choose_good_exit_server(uint8_t purpose, routerlist_t *dir,
+                        int need_uptime, int need_capacity)
 {
   routerinfo_t *r;
   or_options_t *options = get_options();
   switch (purpose) {
     case CIRCUIT_PURPOSE_C_GENERAL:
-      return choose_good_exit_server_general(dir);
+      return choose_good_exit_server_general(dir, need_uptime, need_capacity);
     case CIRCUIT_PURPOSE_C_ESTABLISH_REND:
       r = router_choose_random_node(options->RendNodes, options->RendExcludeNodes,
-          NULL, 0, 1, options->_AllowUnverified & ALLOW_UNVERIFIED_RENDEZVOUS, 0);
+          NULL, need_uptime, need_capacity,
+          options->_AllowUnverified & ALLOW_UNVERIFIED_RENDEZVOUS, 0);
       return r;
   }
   log_fn(LOG_WARN,"Bug: unhandled purpose %d", purpose);
+#ifdef TOR_FRAGILE
+  tor_assert(0);
+#endif
   return NULL;
 }
 
-/** Allocate a cpath_build_state_t, populate it based on
- * <b>purpose</b> and <b>exit_digest</b> (if specified), and
- * return it.
- */
-static cpath_build_state_t *
-onion_new_cpath_build_state(uint8_t purpose, const char *exit_digest)
-{
+/** Decide a suitable length for circ's cpath, and pick an exit
+ * router (or use <b>exit</b> if provided). Store these in the
+ * cpath. Return 0 if ok, -1 if circuit should be closed. */
+static int
+onion_pick_cpath_exit(circuit_t *circ, routerinfo_t *exit) {
+  cpath_build_state_t *state = circ->build_state;
+
   routerlist_t *rl;
   int r;
-  cpath_build_state_t *info;
-  routerinfo_t *exit;
 
   router_get_routerlist(&rl);
-  if (!rl)
-    return NULL;
-  r = new_route_len(get_options()->PathlenCoinWeight, purpose, rl->routers);
+  if (!rl) {
+    log_fn(LOG_WARN,"router_get_routerlist returned empty list; closing circ.");
+    return -1;
+  }
+  r = new_route_len(get_options()->PathlenCoinWeight, circ->purpose, rl->routers);
   if (r < 1) /* must be at least 1 */
-    return NULL;
-  info = tor_malloc_zero(sizeof(cpath_build_state_t));
-  info->desired_path_len = r;
-  if (exit_digest) { /* the circuit-builder pre-requested one */
-    memcpy(info->chosen_exit_digest, exit_digest, DIGEST_LEN);
-    exit = router_get_by_digest(exit_digest);
-    if (exit) {
-      info->chosen_exit_name = tor_strdup(exit->nickname);
-    } else {
-      info->chosen_exit_name = tor_malloc(HEX_DIGEST_LEN+1);
-      base16_encode(info->chosen_exit_name, HEX_DIGEST_LEN+1,
-                    exit_digest, DIGEST_LEN);
-    }
-    log_fn(LOG_INFO,"Using requested exit node '%s'", info->chosen_exit_name);
+    return -1;
+  state->desired_path_len = r;
+
+  if (exit) { /* the circuit-builder pre-requested one */
+    log_fn(LOG_INFO,"Using requested exit node '%s'", exit->nickname);
   } else { /* we have to decide one */
-    exit = choose_good_exit_server(purpose, rl);
+    exit = choose_good_exit_server(circ->purpose, rl,
+                                   state->need_uptime, state->need_capacity);
     if (!exit) {
       log_fn(LOG_WARN,"failed to choose an exit server");
-      tor_free(info);
-      return NULL;
+      return -1;
     }
-    memcpy(info->chosen_exit_digest, exit->identity_digest, DIGEST_LEN);
-    info->chosen_exit_name = tor_strdup(exit->nickname);
   }
-  return info;
+  memcpy(state->chosen_exit_digest, exit->identity_digest, DIGEST_LEN);
+  state->chosen_exit_name = tor_strdup(exit->nickname);
+  return 0;
+}
+
+/** Give <b>circ</b> a new exit destination to <b>exit</b>, and add a
+ * hop to the cpath reflecting this. Don't send the next extend cell --
+ * the caller will do this if it wants to.
+ */
+int
+circuit_append_new_exit(circuit_t *circ, routerinfo_t *exit) {
+  tor_assert(exit);
+  tor_assert(circ && CIRCUIT_IS_ORIGIN(circ));
+  tor_free(circ->build_state->chosen_exit_name);
+  circ->build_state->chosen_exit_name = tor_strdup(exit->nickname);
+  memcpy(circ->build_state->chosen_exit_digest, exit->identity_digest, DIGEST_LEN);
+  ++circ->build_state->desired_path_len;
+  onion_append_hop(&circ->cpath, exit);
+  return 0;
+}
+
+/** Take the open circ originating here, give it a new exit destination
+ * to <b>exit</b>, and get it to send the next extend cell. If you can't
+ * send the extend cell, mark the circuit for close and return -1, else
+ * return 0. */
+int circuit_extend_to_new_exit(circuit_t *circ, routerinfo_t *exit) {
+  circuit_append_new_exit(circ, exit);
+  circ->state = CIRCUIT_STATE_BUILDING;
+  if (circuit_send_next_onion_skin(circ)<0) {
+    log_fn(LOG_WARN, "Couldn't extend circuit to new point '%s'.",
+           circ->build_state->chosen_exit_name);
+    circuit_mark_for_close(circ);
+    return -1;
+  }
+  return 0;
 }
 
 /** Return the number of routers in <b>routers</b> that are currently up
@@ -1143,7 +1256,26 @@ void onion_append_to_cpath(crypt_path_t **head_ptr, crypt_path_t *new_hop)
   }
 }
 
-static routerinfo_t *choose_good_middle_server(cpath_build_state_t *state,
+/** Add to sl all routers with platform version less than cutoff. */
+static void
+excluded_add_obsolete(smartlist_t *sl, const char *atleast, const char *nomore) {
+  routerlist_t *rl;
+  int i;
+  routerinfo_t *router;
+
+  router_get_routerlist(&rl);
+  if (!rl) return;
+
+  for (i = 0; i < smartlist_len(rl->routers); ++i) { /* iterate over routers */
+    router = smartlist_get(rl->routers, i);
+    if (!tor_version_as_new_as(router->platform, atleast) ||
+        tor_version_as_new_as(router->platform, nomore))
+      smartlist_add(sl, router);
+  }
+}
+
+static routerinfo_t *choose_good_middle_server(uint8_t purpose,
+                                               cpath_build_state_t *state,
                                                crypt_path_t *head,
                                                int cur_len)
 {
@@ -1168,8 +1300,11 @@ static routerinfo_t *choose_good_middle_server(cpath_build_state_t *state,
       routerlist_add_family(excluded, r);
     }
   }
+  if (purpose == CIRCUIT_PURPOSE_TESTING)
+    excluded_add_obsolete(excluded, "0.0.9.7", "0.0.9.10");
   choice = router_choose_random_node(NULL, get_options()->ExcludeNodes, excluded,
-           0, 1, get_options()->_AllowUnverified & ALLOW_UNVERIFIED_MIDDLE, 0);
+           state->need_uptime, state->need_capacity,
+           get_options()->_AllowUnverified & ALLOW_UNVERIFIED_MIDDLE, 0);
   smartlist_free(excluded);
   return choice;
 }
@@ -1179,7 +1314,6 @@ static routerinfo_t *choose_good_entry_server(cpath_build_state_t *state)
   routerinfo_t *r, *choice;
   smartlist_t *excluded = smartlist_create();
   or_options_t *options = get_options();
-  char buf[16];
 
   if ((r = router_get_by_digest(state->chosen_exit_digest))) {
     smartlist_add(excluded, r);
@@ -1200,33 +1334,65 @@ static routerinfo_t *choose_good_entry_server(cpath_build_state_t *state)
 
     for (i=0; i < smartlist_len(rl->routers); i++) {
       r = smartlist_get(rl->routers, i);
-      tor_snprintf(buf, sizeof(buf), "%d", r->or_port);
-      if (!smartlist_string_isin(options->FirewallPorts, buf))
+      if (!smartlist_string_num_isin(options->FirewallPorts, r->or_port))
          smartlist_add(excluded, r);
     }
   }
   choice = router_choose_random_node(options->EntryNodes, options->ExcludeNodes,
-           excluded, 0, 1, options->_AllowUnverified & ALLOW_UNVERIFIED_ENTRY,
+           excluded, state->need_uptime, state->need_capacity,
+           options->_AllowUnverified & ALLOW_UNVERIFIED_ENTRY,
            options->StrictEntryNodes);
   smartlist_free(excluded);
   return choice;
 }
 
-/** Choose a suitable next hop in the cpath <b>head_ptr</b>,
- * based on <b>state</b>. Add the hop info to head_ptr, and return a
- * pointer to the chosen router in <b>router_out</b>.
+/** Return the first non-open hop in cpath, or return NULL if all
+ * hops are open. */
+static crypt_path_t *
+onion_next_hop_in_cpath(crypt_path_t *cpath) {
+  crypt_path_t *hop = cpath;
+  do {
+    if (hop->state != CPATH_STATE_OPEN)
+      return hop;
+    hop = hop->next;
+  } while (hop != cpath);
+  return NULL;
+}
+
+/** Find the router corresponding to the first non-open hop in
+ * circ->cpath. Make sure it's state closed. Return 1 if all
+ * hops are open (the circuit is complete), 0 if we find a router
+ * (and set it to *router), and -1 if we fail to lookup the router.
  */
 static int
-onion_extend_cpath(crypt_path_t **head_ptr, cpath_build_state_t
-                   *state, routerinfo_t **router_out)
+onion_next_router_in_cpath(circuit_t *circ, routerinfo_t **router) {
+  routerinfo_t *r;
+  crypt_path_t *hop = onion_next_hop_in_cpath(circ->cpath);
+  if (!hop) /* all hops are open */
+    return 1;
+  tor_assert(hop->state == CPATH_STATE_CLOSED);
+  r = router_get_by_digest(hop->identity_digest);
+  if (!r) {
+    log_fn(LOG_WARN,"Circuit intended to extend to a hop whose routerinfo we've lost. Cancelling circuit.");
+    return -1;
+  }
+  *router = r;
+  return 0;
+}
+
+/** Choose a suitable next hop in the cpath <b>head_ptr</b>,
+ * based on <b>state</b>. Append the hop info to head_ptr.
+ */
+static int
+onion_extend_cpath(uint8_t purpose, crypt_path_t **head_ptr,
+                   cpath_build_state_t *state)
 {
   int cur_len;
-  crypt_path_t *cpath, *hop;
+  crypt_path_t *cpath;
   routerinfo_t *choice;
   smartlist_t *excludednodes;
 
   tor_assert(head_ptr);
-  tor_assert(router_out);
 
   if (!*head_ptr) {
     cur_len = 0;
@@ -1236,11 +1402,13 @@ onion_extend_cpath(crypt_path_t **head_ptr, cpath_build_state_t
       ++cur_len;
     }
   }
+
   if (cur_len >= state->desired_path_len) {
     log_fn(LOG_DEBUG, "Path is complete: %d steps long",
            state->desired_path_len);
     return 1;
   }
+
   log_fn(LOG_DEBUG, "Path is %d long; we want %d", cur_len,
          state->desired_path_len);
 
@@ -1252,7 +1420,7 @@ onion_extend_cpath(crypt_path_t **head_ptr, cpath_build_state_t
   } else if (cur_len == 0) { /* picking first node */
     choice = choose_good_entry_server(state);
   } else {
-    choice = choose_good_middle_server(state, *head_ptr, cur_len);
+    choice = choose_good_middle_server(purpose, state, *head_ptr, cur_len);
   }
 
   smartlist_free(excludednodes);
@@ -1264,11 +1432,21 @@ onion_extend_cpath(crypt_path_t **head_ptr, cpath_build_state_t
   log_fn(LOG_DEBUG,"Chose router %s for hop %d (exit is %s)",
          choice->nickname, cur_len+1, state->chosen_exit_name);
 
-  hop = tor_malloc_zero(sizeof(crypt_path_t));
+  onion_append_hop(head_ptr, choice);
+  return 0;
+}
+
+/** Create a new hop, annotate it with information about its
+ * corresponding router <b>choice</b>, and append it to the
+ * end of the cpath <b>head_ptr</b>. */
+static int
+onion_append_hop(crypt_path_t **head_ptr, routerinfo_t *choice) {
+  crypt_path_t *hop = tor_malloc_zero(sizeof(crypt_path_t));
 
   /* link hop into the cpath, at the end. */
   onion_append_to_cpath(head_ptr, hop);
 
+  hop->magic = CRYPT_PATH_MAGIC;
   hop->state = CPATH_STATE_CLOSED;
 
   hop->port = choice->or_port;
@@ -1278,10 +1456,6 @@ onion_extend_cpath(crypt_path_t **head_ptr, cpath_build_state_t
   hop->package_window = CIRCWINDOW_START;
   hop->deliver_window = CIRCWINDOW_START;
 
-  log_fn(LOG_DEBUG, "Extended circuit path with %s for hop %d",
-         choice->nickname, cur_len+1);
-
-  *router_out = choice;
   return 0;
 }
 
