@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c,v 1.22 2004/11/29 22:25:29 nickm Exp $ */
-const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.22 2004/11/29 22:25:29 nickm Exp $";
+/* $Id: circuitlist.c,v 1.34 2005/03/29 00:35:35 arma Exp $ */
+const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.34 2005/03/29 00:35:35 arma Exp $";
 
 /**
  * \file circuitlist.c
@@ -76,7 +76,8 @@ void circuit_close_all_marked(void)
  */
 circuit_t *circuit_new(uint16_t p_circ_id, connection_t *p_conn) {
   circuit_t *circ;
-  static uint32_t n_circuits_allocated = 0;
+  static uint32_t n_circuits_allocated = 1;
+    /* never zero, since a global ID of 0 is treated specially by the controller */
 
   circ = tor_malloc_zero(sizeof(circuit_t));
   circ->magic = CIRCUIT_MAGIC;
@@ -149,6 +150,24 @@ static void circuit_free_cpath(crypt_path_t *cpath) {
   circuit_free_cpath_node(cpath);
 }
 
+/** Release all storage held by circuits. */
+void
+circuit_free_all(void)
+{
+  circuit_t *next;
+  while (global_circuitlist) {
+    next = global_circuitlist->next;
+    while (global_circuitlist->resolving_streams) {
+      connection_t *next;
+      next = global_circuitlist->resolving_streams->next_stream;
+      connection_free(global_circuitlist->resolving_streams);
+      global_circuitlist->resolving_streams = next;
+    }
+    circuit_free(global_circuitlist);
+    global_circuitlist = next;
+  }
+}
+
 /** Deallocate space associated with the cpath node <b>victim</b>. */
 static void
 circuit_free_cpath_node(crypt_path_t *victim) {
@@ -162,9 +181,27 @@ circuit_free_cpath_node(crypt_path_t *victim) {
     crypto_free_digest_env(victim->b_digest);
   if (victim->handshake_state)
     crypto_dh_free(victim->handshake_state);
+  victim->magic = 0xDEADBEEFu;
   tor_free(victim);
 }
 
+/** Return the circuit whose global ID is <b>id</b>, or NULL if no
+ * such circuit exists. */
+circuit_t *
+circuit_get_by_global_id(uint32_t id)
+{
+  circuit_t *circ;
+  for (circ=global_circuitlist;circ;circ = circ->next) {
+    if (circ->global_identifier == id) {
+      if (circ->marked_for_close)
+        return NULL;
+      else
+        return circ;
+    }
+  }
+  return NULL;
+}
+
 /** Return a circ such that:
  *  - circ-\>n_circ_id or circ-\>p_circ_id is equal to <b>circ_id</b>, and
  *  - circ is attached to <b>conn</b>, either as p_conn, n-conn, or
@@ -292,41 +329,49 @@ circuit_t *circuit_get_rendezvous(const char *cookie)
   return NULL;
 }
 
-/** Count the number of circs originating here that aren't open, and
- * that have the specified <b>purpose</b>. */
-int circuit_count_building(uint8_t purpose) {
-  circuit_t *circ;
-  int num=0;
-
-  for (circ=global_circuitlist;circ;circ = circ->next) {
-    if (CIRCUIT_IS_ORIGIN(circ) &&
-       circ->state != CIRCUIT_STATE_OPEN &&
-       circ->purpose == purpose &&
-       !circ->marked_for_close)
-      num++;
-  }
-  return num;
-}
-
-/** Return the circuit that is open, has specified <b>purpose</b>,
- * has a timestamp_dirty value of 0, and was created most recently,
- * or NULL if no circuit fits this description.
+/** Return a circuit that is open, has specified <b>purpose</b>,
+ * has a timestamp_dirty value of 0, and is uptime/capacity/internal
+ * if required; or NULL if no circuit fits this description.
+ *
+ * Avoid returning need_uptime circuits if not necessary.
+ * FFFF As a more important goal, not yet implemented, avoid returning
+ * internal circuits if not necessary.
  */
 circuit_t *
-circuit_get_youngest_clean_open(uint8_t purpose) {
+circuit_get_clean_open(uint8_t purpose, int need_uptime,
+                       int need_capacity, int internal) {
   circuit_t *circ;
-  circuit_t *youngest=NULL;
+  circuit_t *best=NULL;
 
-  for (circ=global_circuitlist;circ;circ = circ->next) {
+  log_fn(LOG_DEBUG,"Hunting for a circ to cannibalize: purpose %d, uptime %d, capacity %d, internal %d", purpose, need_uptime, need_capacity, internal);
+
+  for (circ=global_circuitlist; circ; circ = circ->next) {
     if (CIRCUIT_IS_ORIGIN(circ) &&
         circ->state == CIRCUIT_STATE_OPEN &&
         !circ->marked_for_close &&
         circ->purpose == purpose &&
         !circ->timestamp_dirty &&
-        (!youngest || youngest->timestamp_created < circ->timestamp_created))
-      youngest = circ;
+        (!need_uptime || circ->build_state->need_uptime) &&
+        (!need_capacity || circ->build_state->need_capacity) &&
+        (!internal || circ->build_state->is_internal)) {
+      if (!best || (best->build_state->need_uptime && !need_uptime))
+        best = circ;
+    }
+  }
+  return best;
+}
+
+/** Go through the circuitlist; mark-for-close each circuit that starts
+ *  at us but has not yet been used. */
+void circuit_mark_all_unused_circs(void) {
+  circuit_t *circ;
+
+  for (circ=global_circuitlist; circ; circ = circ->next) {
+    if (CIRCUIT_IS_ORIGIN(circ) &&
+        !circ->marked_for_close &&
+        !circ->timestamp_dirty)
+      circuit_mark_for_close(circ);
   }
-  return youngest;
 }
 
 /** Mark <b>circ</b> to be closed next time we call
@@ -383,9 +428,13 @@ int _circuit_mark_for_close(circuit_t *circ) {
   while (circ->resolving_streams) {
     conn = circ->resolving_streams;
     circ->resolving_streams = conn->next_stream;
-    connection_dns_remove(conn); /* remove it from resolve lists */
-    log_fn(LOG_INFO,"Freeing resolving-conn.");
-    connection_free(conn);
+    if (!conn->marked_for_close) {
+      /* The other side will see a DESTROY, and infer that the connections
+       * are closing because the circuit is getting torn down.  No need
+       * to send an end cell*/
+      conn->has_sent_end = 1; /* we're closing the circuit, nothing to send to */
+      connection_mark_for_close(conn);
+    }
   }
   if (circ->p_conn)
     connection_send_destroy(circ->p_circ_id, circ->p_conn);
@@ -409,6 +458,8 @@ void assert_cpath_layer_ok(const crypt_path_t *cp)
 {
 //  tor_assert(cp->addr); /* these are zero for rendezvous extra-hops */
 //  tor_assert(cp->port);
+  tor_assert(cp);
+  tor_assert(cp->magic == CRYPT_PATH_MAGIC);
   switch (cp->state)
     {
     case CPATH_STATE_OPEN:
