@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuituse.c,v 1.38.2.2 2005/01/17 18:50:45 arma Exp $ */
-const char circuituse_c_id[] = "$Id: circuituse.c,v 1.38.2.2 2005/01/17 18:50:45 arma Exp $";
+/* $Id: circuituse.c,v 1.64 2005/04/01 08:42:26 arma Exp $ */
+const char circuituse_c_id[] = "$Id: circuituse.c,v 1.64 2005/04/01 08:42:26 arma Exp $";
 
 /**
  * \file circuituse.c
@@ -35,6 +35,9 @@ static int circuit_is_acceptable(circuit_t *circ,
                                  time_t now)
 {
   routerinfo_t *exitrouter;
+  tor_assert(circ);
+  tor_assert(conn);
+  tor_assert(conn->socks_request);
 
   if (!CIRCUIT_IS_ORIGIN(circ))
     return 0; /* this circ doesn't start at us */
@@ -46,9 +49,9 @@ static int circuit_is_acceptable(circuit_t *circ,
   /* if this circ isn't our purpose, skip. */
   if (purpose == CIRCUIT_PURPOSE_C_REND_JOINED && !must_be_open) {
     if (circ->purpose != CIRCUIT_PURPOSE_C_ESTABLISH_REND &&
-       circ->purpose != CIRCUIT_PURPOSE_C_REND_READY &&
-       circ->purpose != CIRCUIT_PURPOSE_C_REND_READY_INTRO_ACKED &&
-       circ->purpose != CIRCUIT_PURPOSE_C_REND_JOINED)
+        circ->purpose != CIRCUIT_PURPOSE_C_REND_READY &&
+        circ->purpose != CIRCUIT_PURPOSE_C_REND_READY_INTRO_ACKED &&
+        circ->purpose != CIRCUIT_PURPOSE_C_REND_JOINED)
       return 0;
   } else if (purpose == CIRCUIT_PURPOSE_C_INTRODUCE_ACK_WAIT && !must_be_open) {
     if (circ->purpose != CIRCUIT_PURPOSE_C_INTRODUCING &&
@@ -61,40 +64,36 @@ static int circuit_is_acceptable(circuit_t *circ,
 
   if (purpose == CIRCUIT_PURPOSE_C_GENERAL)
     if (circ->timestamp_dirty &&
-       circ->timestamp_dirty+get_options()->NewCircuitPeriod <= now)
+       circ->timestamp_dirty+get_options()->MaxCircuitDirtiness <= now)
       return 0;
 
-  if (conn) {
-    /* decide if this circ is suitable for this conn */
+  /* decide if this circ is suitable for this conn */
 
-    /* for rend circs, circ->cpath->prev is not the last router in the
-     * circuit, it's the magical extra bob hop. so just check the nickname
-     * of the one we meant to finish at.
-     */
-    exitrouter = router_get_by_digest(circ->build_state->chosen_exit_digest);
+  /* for rend circs, circ->cpath->prev is not the last router in the
+   * circuit, it's the magical extra bob hop. so just check the nickname
+   * of the one we meant to finish at.
+   */
+  exitrouter = router_get_by_digest(circ->build_state->chosen_exit_digest);
 
-    if (!exitrouter) {
-      log_fn(LOG_INFO,"Skipping broken circ (exit router vanished)");
-      return 0; /* this circuit is screwed and doesn't know it yet */
+  if (!exitrouter) {
+    log_fn(LOG_INFO,"Skipping broken circ (exit router vanished)");
+    return 0; /* this circuit is screwed and doesn't know it yet */
+  }
+
+  if (!circ->build_state->need_uptime &&
+      smartlist_string_num_isin(get_options()->LongLivedPorts,
+                                conn->socks_request->port))
+    return 0;
+
+  if (purpose == CIRCUIT_PURPOSE_C_GENERAL) {
+    if (!connection_ap_can_use_exit(conn, exitrouter)) {
+      /* can't exit from this router */
+      return 0;
     }
-
-    if (conn->socks_request &&
-        conn->socks_request->command == SOCKS_COMMAND_RESOLVE) {
-      /* 0.0.8 servers have buggy resolve support. */
-      if (!tor_version_as_new_as(exitrouter->platform, "0.0.9pre1"))
-        return 0;
-    } else if (purpose == CIRCUIT_PURPOSE_C_GENERAL) {
-      if (!connection_ap_can_use_exit(conn, exitrouter)) {
-        /* can't exit from this router */
-        return 0;
-      }
-    } else { /* not general */
-      if (rend_cmp_service_ids(conn->rend_query, circ->rend_query) &&
-          (circ->rend_query[0] || purpose != CIRCUIT_PURPOSE_C_REND_JOINED)) {
-        /* this circ is not for this conn, and it's not suitable
-         * for cannibalizing either */
-        return 0;
-      }
+  } else { /* not general */
+    if (rend_cmp_service_ids(conn->rend_query, circ->rend_query)) {
+      /* this circ is not for this conn */
+      return 0;
     }
   }
   return 1;
@@ -116,6 +115,7 @@ static int circuit_is_better(circuit_t *a, circuit_t *b, uint8_t purpose)
           return 1;
       } else {
         if (a->timestamp_dirty ||
+            b->build_state->is_internal ||
             a->timestamp_created > b->timestamp_created)
           return 1;
       }
@@ -176,9 +176,8 @@ circuit_get_best(connection_t *conn, int must_be_open, uint8_t purpose)
   return best;
 }
 
-/** Circuits that were born at the end of their second might be expired
- * after 30.1 seconds; circuits born at the beginning might be expired
- * after closer to 31 seconds.
+/** If we find a circuit that isn't open yet and was born this many
+ * seconds ago, then assume something went wrong, and cull it.
  */
 #define MIN_SECONDS_BEFORE_EXPIRING_CIRC 30
 
@@ -198,6 +197,7 @@ void circuit_expire_building(time_t now) {
     if (victim->timestamp_created + MIN_SECONDS_BEFORE_EXPIRING_CIRC > now)
       continue; /* it's young still, don't mess with it */
 
+#if 0
     /* some debug logs, to help track bugs */
     if (victim->purpose >= CIRCUIT_PURPOSE_C_INTRODUCING &&
         victim->purpose <= CIRCUIT_PURPOSE_C_REND_READY_INTRO_ACKED) {
@@ -213,6 +213,7 @@ void circuit_expire_building(time_t now) {
                victim->n_circ_id,
                (int)(now - victim->timestamp_dirty));
     }
+#endif
 
     /* if circ is !open, or if it's open but purpose is a non-finished
      * intro or rend, then mark it for close */
@@ -279,27 +280,95 @@ int circuit_stream_is_being_handled(connection_t *conn, uint16_t port, int min)
   routerinfo_t *exitrouter;
   int num=0;
   time_t now = time(NULL);
+  int need_uptime = smartlist_string_num_isin(get_options()->LongLivedPorts,
+                                   conn ? conn->socks_request->port : port);
 
   for (circ=global_circuitlist;circ;circ = circ->next) {
     if (CIRCUIT_IS_ORIGIN(circ) &&
         !circ->marked_for_close &&
         circ->purpose == CIRCUIT_PURPOSE_C_GENERAL &&
         (!circ->timestamp_dirty ||
-         circ->timestamp_dirty + get_options()->NewCircuitPeriod < now)) {
+         circ->timestamp_dirty + get_options()->MaxCircuitDirtiness < now)) {
       exitrouter = router_get_by_digest(circ->build_state->chosen_exit_digest);
       if (exitrouter &&
-          ((conn && connection_ap_can_use_exit(conn, exitrouter)) ||
-           (!conn &&
-            router_compare_addr_to_addr_policy(0, port, exitrouter->exit_policy) !=
-              ADDR_POLICY_REJECTED))) {
-        if (++num >= min)
-          return 1;
+          (!need_uptime || circ->build_state->need_uptime)) {
+        int ok;
+        if (conn) {
+          ok = connection_ap_can_use_exit(conn, exitrouter);
+        } else {
+          addr_policy_result_t r =
+           router_compare_addr_to_addr_policy(0, port, exitrouter->exit_policy);
+          ok = r != ADDR_POLICY_REJECTED && r != ADDR_POLICY_PROBABLY_REJECTED;
+        }
+        if (ok) {
+          if (++num >= min)
+            return 1;
+        }
       }
     }
   }
   return 0;
 }
 
+/** Don't keep more than 10 unused open circuits around. */
+#define MAX_UNUSED_OPEN_CIRCUITS 10
+
+/** Figure out how many circuits we have open that are clean. Make
+ * sure it's enough for all the upcoming behaviors we predict we'll have.
+ * But if we have too many, close the not-so-useful ones.
+ */
+static void
+circuit_predict_and_launch_new(void)
+{
+  circuit_t *circ;
+  int num=0, num_internal=0, num_uptime_internal=0;
+  int hidserv_needs_uptime=0, hidserv_needs_capacity=1;
+  int port_needs_uptime=0, port_needs_capacity=1;
+  int need_ports, need_hidserv;
+  time_t now = time(NULL);
+
+  /* check if we know of a port that's been requested recently
+   * and no circuit is currently available that can handle it. */
+  need_ports = !circuit_all_predicted_ports_handled(now, &port_needs_uptime,
+                                                    &port_needs_capacity);
+
+  need_hidserv = rep_hist_get_predicted_hidserv(now, &hidserv_needs_uptime,
+                                                &hidserv_needs_capacity);
+
+  for (circ=global_circuitlist;circ;circ = circ->next) {
+    if (!CIRCUIT_IS_ORIGIN(circ))
+      continue;
+    if (circ->marked_for_close)
+      continue; /* don't mess with marked circs */
+    if (circ->timestamp_dirty)
+      continue; /* only count clean circs */
+    if (circ->purpose != CIRCUIT_PURPOSE_C_GENERAL)
+      continue; /* only pay attention to general-purpose circs */
+    num++;
+    if (circ->build_state->is_internal)
+      num_internal++;
+    if (circ->build_state->need_uptime && circ->build_state->is_internal)
+      num_uptime_internal++;
+  }
+
+  if (num < MAX_UNUSED_OPEN_CIRCUITS) {
+    /* perhaps we want another */
+    if (need_ports) {
+      log_fn(LOG_INFO,"Have %d clean circs (%d internal), need another exit circ.",
+        num, num_internal);
+      circuit_launch_by_router(CIRCUIT_PURPOSE_C_GENERAL, NULL,
+                               port_needs_uptime, port_needs_capacity, 0);
+    } else if (need_hidserv &&
+               ((num_uptime_internal<2 && hidserv_needs_uptime) ||
+                num_internal<2)) {
+      log_fn(LOG_INFO,"Have %d clean circs (%d uptime-internal, %d internal),"
+        " need another hidserv circ.", num, num_uptime_internal, num_internal);
+      circuit_launch_by_router(CIRCUIT_PURPOSE_C_GENERAL, NULL,
+                               hidserv_needs_uptime, hidserv_needs_capacity, 1);
+    }
+  }
+}
+
 /** Build a new test circuit every 5 minutes */
 #define TESTING_CIRCUIT_INTERVAL 300
 
@@ -310,7 +379,6 @@ int circuit_stream_is_being_handled(connection_t *conn, uint16_t port, int min)
  */
 void circuit_build_needed_circs(time_t now) {
   static long time_to_new_circuit = 0;
-  circuit_t *circ;
 
   /* launch a new circ for any pending streams that need one */
   connection_ap_attach_pending();
@@ -319,45 +387,24 @@ void circuit_build_needed_circs(time_t now) {
   if (has_fetched_directory)
     rend_services_introduce();
 
-  circ = circuit_get_youngest_clean_open(CIRCUIT_PURPOSE_C_GENERAL);
-
   if (time_to_new_circuit < now) {
     circuit_reset_failure_count(1);
     time_to_new_circuit = now + get_options()->NewCircuitPeriod;
     if (proxy_mode(get_options()))
-      client_dns_clean();
+      addressmap_clean(now);
     circuit_expire_old_circuits();
 
+#if 0 /* disable for now, until predict-and-launch-new can cull leftovers */
+    circ = circuit_get_youngest_clean_open(CIRCUIT_PURPOSE_C_GENERAL);
     if (get_options()->RunTesting &&
         circ &&
         circ->timestamp_created + TESTING_CIRCUIT_INTERVAL < now) {
       log_fn(LOG_INFO,"Creating a new testing circuit.");
-      circuit_launch_by_identity(CIRCUIT_PURPOSE_C_GENERAL, NULL);
+      circuit_launch_by_router(CIRCUIT_PURPOSE_C_GENERAL, NULL, 0, 0, 0);
     }
-  }
-
-#if 0
-/** How many simultaneous in-progress general-purpose circuits do we
- * want to be building at once, if there are no open general-purpose
- * circuits?
- */
-#define CIRCUIT_MIN_BUILDING_GENERAL 5
-  /* if there's no open circ, and less than 5 are on the way,
-   * go ahead and try another. */
-  if (!circ && circuit_count_building(CIRCUIT_PURPOSE_C_GENERAL)
-               < CIRCUIT_MIN_BUILDING_GENERAL) {
-    circuit_launch_by_identity(CIRCUIT_PURPOSE_C_GENERAL, NULL);
-  }
 #endif
-
-  /* if we know of a port that's been requested recently and no
-   * circuit is currently available that can handle it, start one
-   * for that too. */
-  if (!circuit_all_predicted_ports_handled(now)) {
-    circuit_launch_by_identity(CIRCUIT_PURPOSE_C_GENERAL, NULL);
   }
-
-  /* XXX count idle rendezvous circs and build more */
+  circuit_predict_and_launch_new();
 }
 
 /** If the stream <b>conn</b> is a member of any of the linked
@@ -432,10 +479,8 @@ void circuit_about_to_close_connection(connection_t *conn) {
 
   switch (conn->type) {
     case CONN_TYPE_OR:
-      if (conn->state != OR_CONN_STATE_OPEN) {
-        /* Inform any pending (not attached) circs that they should give up. */
-        circuit_n_conn_done(conn, 0);
-      }
+      /* Inform any pending (not attached) circs that they should give up. */
+      circuit_n_conn_done(conn, 0);
       /* Now close all the attached circuits on it. */
       while ((circ = circuit_get_by_conn(conn))) {
         if (circ->n_conn == conn) /* it's closing in front of us */
@@ -462,24 +507,14 @@ void circuit_about_to_close_connection(connection_t *conn) {
   } /* end switch */
 }
 
-/** Don't keep more than 10 unused open circuits around. */
-#define MAX_UNUSED_OPEN_CIRCUITS 10
-
 /** Find each circuit that has been dirty for too long, and has
  * no streams on it: mark it for close.
- *
- * Also, if there are more than MAX_UNUSED_OPEN_CIRCUITS open and
- * unused circuits, then mark the excess circs for close.
  */
 static void
 circuit_expire_old_circuits(void)
 {
   circuit_t *circ;
   time_t now = time(NULL);
-  smartlist_t *unused_open_circs;
-  int i;
-
-  unused_open_circs = smartlist_create();
 
   for (circ = global_circuitlist; circ; circ = circ->next) {
     if (circ->marked_for_close)
@@ -488,8 +523,8 @@ circuit_expire_old_circuits(void)
      * on it, mark it for close.
      */
     if (circ->timestamp_dirty &&
-        circ->timestamp_dirty + get_options()->NewCircuitPeriod < now &&
-        !circ->p_conn && /* we're the origin */
+        circ->timestamp_dirty + get_options()->MaxCircuitDirtiness < now &&
+        CIRCUIT_IS_ORIGIN(circ) &&
         !circ->p_streams /* nothing attached */ ) {
       log_fn(LOG_DEBUG,"Closing n_circ_id %d (dirty %d secs ago, purp %d)",circ->n_circ_id,
              (int)(now - circ->timestamp_dirty), circ->purpose);
@@ -505,23 +540,31 @@ circuit_expire_old_circuits(void)
         log_fn(LOG_DEBUG,"Closing circuit that has been unused for %d seconds.",
                (int)(now - circ->timestamp_created));
         circuit_mark_for_close(circ);
-      } else {
-        /* Also, gather a list of open unused general circuits that we created.
-         * Because we add elements to the front of global_circuitlist,
-         * the last elements of unused_open_circs will be the oldest
-         * ones.
-         */
-        smartlist_add(unused_open_circs, circ);
       }
     }
   }
-  for (i = MAX_UNUSED_OPEN_CIRCUITS; i < smartlist_len(unused_open_circs); ++i) {
-    circuit_t *circ = smartlist_get(unused_open_circs, i);
-    log_fn(LOG_DEBUG,"Expiring excess clean circ (n_circ_id %d, purp %d)",
-           circ->n_circ_id, circ->purpose);
-    circuit_mark_for_close(circ);
-  }
-  smartlist_free(unused_open_circs);
+}
+
+/** A testing circuit has completed. Take whatever stats we want. */
+static void
+circuit_testing_opened(circuit_t *circ) {
+  /* For now, we only use testing circuits to see if our ORPort is
+     reachable. But we remember reachability in onionskin_answer(),
+     so there's no need to record anything here. Just close the circ. */
+  circuit_mark_for_close(circ);
+}
+
+/** A testing circuit has failed to build. Take whatever stats we want. */
+static void
+circuit_testing_failed(circuit_t *circ, int at_last_hop) {
+#if 0
+  routerinfo_t *me = router_get_my_routerinfo();
+
+  if (!at_last_hop)
+    circuit_launch_by_router(CIRCUIT_PURPOSE_TESTING, me, 0, 1, 1);
+  else
+#endif
+  log_fn(LOG_INFO,"Our testing circuit (to see if your ORPort is reachable) has failed. I'll try again later.");
 }
 
 /** The circuit <b>circ</b> has just become open. Take the next
@@ -537,6 +580,7 @@ void circuit_has_opened(circuit_t *circ) {
   switch (circ->purpose) {
     case CIRCUIT_PURPOSE_C_ESTABLISH_REND:
       rend_client_rendcirc_has_opened(circ);
+      connection_ap_attach_pending();
       break;
     case CIRCUIT_PURPOSE_C_INTRODUCING:
       rend_client_introcirc_has_opened(circ);
@@ -554,6 +598,9 @@ void circuit_has_opened(circuit_t *circ) {
       /* at Bob, connecting to rend point */
       rend_service_rendezvous_has_opened(circ);
       break;
+    case CIRCUIT_PURPOSE_TESTING:
+      circuit_testing_opened(circ);
+      break;
     default:
       log_fn(LOG_ERR,"unhandled purpose %d",circ->purpose);
       tor_assert(0);
@@ -585,6 +632,9 @@ void circuit_build_failed(circuit_t *circ) {
         circuit_increment_failure_count();
       }
       break;
+    case CIRCUIT_PURPOSE_TESTING:
+      circuit_testing_failed(circ, failed_at_last_hop);
+      break;
     case CIRCUIT_PURPOSE_S_ESTABLISH_INTRO:
       /* at Bob, waiting for introductions */
       if (circ->state != CIRCUIT_STATE_OPEN) {
@@ -612,13 +662,10 @@ void circuit_build_failed(circuit_t *circ) {
       /* at Bob, connecting to rend point */
       /* Don't increment failure count, since Alice may have picked
        * the rendezvous point maliciously */
-      if (failed_at_last_hop) {
-        log_fn(LOG_INFO,"Couldn't connect to Alice's chosen rend point %s. Sucks to be Alice.", circ->build_state->chosen_exit_name);
-      } else {
-        log_fn(LOG_INFO,"Couldn't connect to Alice's chosen rend point %s, because an earlier node failed.",
-               circ->build_state->chosen_exit_name);
-        rend_service_relaunch_rendezvous(circ);
-      }
+      log_fn(LOG_INFO,"Couldn't connect to Alice's chosen rend point %s (%s hop failed).",
+             failed_at_last_hop?"last":"non-last",
+             circ->build_state->chosen_exit_name);
+      rend_service_relaunch_rendezvous(circ);
       break;
     default:
       /* Other cases are impossible, since this function is only called with
@@ -637,13 +684,59 @@ static int did_circs_fail_last_period = 0;
  * success. */
 #define MAX_CIRCUIT_FAILURES 5
 
-circuit_t *circuit_launch_by_identity(uint8_t purpose, const char *exit_digest)
+/** Launch a new circuit based on our arguments. */
+circuit_t *
+circuit_launch_by_router(uint8_t purpose, routerinfo_t *exit,
+                         int need_uptime, int need_capacity, int internal)
 {
+  circuit_t *circ;
+
   if (!has_fetched_directory) {
     log_fn(LOG_DEBUG,"Haven't fetched directory yet; canceling circuit launch.");
     return NULL;
   }
 
+  if (purpose != CIRCUIT_PURPOSE_C_GENERAL &&
+      purpose != CIRCUIT_PURPOSE_TESTING) {
+    /* see if there are appropriate circs available to cannibalize. */
+    if ((circ = circuit_get_clean_open(CIRCUIT_PURPOSE_C_GENERAL, need_uptime,
+                                       need_capacity, internal))) {
+      log_fn(LOG_INFO,"Cannibalizing circ '%s' for purpose %d",
+             circ->build_state->chosen_exit_name, purpose);
+      circ->purpose = purpose;
+      /* reset the birth date of this circ, else expire_building
+       * will see it and think it's been trying to build since it
+       * began. */
+      circ->timestamp_created = time(NULL);
+      switch (purpose) {
+        case CIRCUIT_PURPOSE_C_ESTABLISH_REND:
+          /* it's ready right now */
+          /* XXX should we call control_event_circuit_status() here? */
+          rend_client_rendcirc_has_opened(circ);
+          break;
+        case CIRCUIT_PURPOSE_S_ESTABLISH_INTRO:
+          /* it's ready right now */
+          rend_service_intro_has_opened(circ);
+          break;
+        case CIRCUIT_PURPOSE_C_INTRODUCING:
+        case CIRCUIT_PURPOSE_S_CONNECT_REND:
+          /* need to add a new hop */
+          tor_assert(exit);
+          if (circuit_extend_to_new_exit(circ, exit) < 0)
+            return NULL;
+          break;
+        default:
+          log_fn(LOG_WARN,"Bug: unexpected purpose %d when cannibalizing a general circ.",
+                 purpose);
+#ifdef TOR_FRAGILE
+          tor_assert(0);
+#endif
+          return NULL;
+      }
+      return circ;
+    }
+  }
+
   if (did_circs_fail_last_period &&
       n_circuit_failures > MAX_CIRCUIT_FAILURES) {
     /* too many failed circs in a row. don't try. */
@@ -652,23 +745,26 @@ circuit_t *circuit_launch_by_identity(uint8_t purpose, const char *exit_digest)
   }
 
   /* try a circ. if it fails, circuit_mark_for_close will increment n_circuit_failures */
-  return circuit_establish_circuit(purpose, exit_digest);
+  return circuit_establish_circuit(purpose, exit,
+                                   need_uptime, need_capacity, internal);
 }
 
 /** Launch a new circuit and return a pointer to it. Return NULL if you failed. */
-circuit_t *circuit_launch_by_nickname(uint8_t purpose, const char *exit_nickname)
+circuit_t *
+circuit_launch_by_nickname(uint8_t purpose, const char *exit_nickname,
+                           int need_uptime, int need_capacity, int internal)
 {
-  const char *digest = NULL;
+  routerinfo_t *router = NULL;
 
   if (exit_nickname) {
-    routerinfo_t *r = router_get_by_nickname(exit_nickname);
-    if (!r) {
+    router = router_get_by_nickname(exit_nickname);
+    if (!router) {
       log_fn(LOG_WARN, "No such OR as '%s'", exit_nickname);
       return NULL;
     }
-    digest = r->identity_digest;
   }
-  return circuit_launch_by_identity(purpose, digest);
+  return circuit_launch_by_router(purpose, router,
+                                  need_uptime, need_capacity, internal);
 }
 
 /** Record another failure at opening a general circuit. When we have
@@ -702,8 +798,8 @@ circuit_get_open_circ_or_launch(connection_t *conn,
                                 uint8_t desired_circuit_purpose,
                                 circuit_t **circp) {
   circuit_t *circ;
-  uint32_t addr;
   int is_resolve;
+  int need_uptime;
 
   tor_assert(conn);
   tor_assert(circp);
@@ -729,11 +825,17 @@ circuit_get_open_circ_or_launch(connection_t *conn,
     return 0;
   }
 
+  need_uptime = smartlist_string_num_isin(get_options()->LongLivedPorts,
+                                          conn->socks_request->port);
 
   /* Do we need to check exit policy? */
   if (!is_resolve && !connection_edge_is_rendezvous_stream(conn)) {
-    addr = client_dns_lookup_entry(conn->socks_request->address);
-    if (router_exit_policy_all_routers_reject(addr, conn->socks_request->port)) {
+    struct in_addr in;
+    uint32_t addr = 0;
+    if (tor_inet_aton(conn->socks_request->address, &in))
+      addr = ntohl(in.s_addr);
+    if (router_exit_policy_all_routers_reject(addr, conn->socks_request->port,
+                                              need_uptime)) {
       log_fn(LOG_NOTICE,"No Tor server exists that allows exit to %s:%d. Rejecting.",
              conn->socks_request->address, conn->socks_request->port);
       return -1;
@@ -745,21 +847,26 @@ circuit_get_open_circ_or_launch(connection_t *conn,
   if (!circ) {
     char *exitname=NULL;
     uint8_t new_circ_purpose;
+    int is_internal;
 
     if (desired_circuit_purpose == CIRCUIT_PURPOSE_C_INTRODUCE_ACK_WAIT) {
       /* need to pick an intro point */
+try_an_intro_point:
       exitname = rend_client_get_random_intro(conn->rend_query);
       if (!exitname) {
-        log_fn(LOG_WARN,"Couldn't get an intro point for '%s'. Closing.",
+        log_fn(LOG_INFO,"No intro points for '%s': refetching service descriptor.",
                conn->rend_query);
-        return -1;
+        rend_client_refetch_renddesc(conn->rend_query);
+        conn->state = AP_CONN_STATE_RENDDESC_WAIT;
+        return 0;
       }
       if (!router_get_by_nickname(exitname)) {
-        log_fn(LOG_WARN,"Advertised intro point '%s' is not known. Closing.", exitname);
+        log_fn(LOG_NOTICE,"Advertised intro point '%s' is not recognized for '%s'. Skipping over.",
+               exitname, conn->rend_query);
+        rend_client_remove_intro_point(exitname, conn->rend_query);
         tor_free(exitname);
-        return -1;
+        goto try_an_intro_point;
       }
-      /* XXX if we failed, then refetch the descriptor */
       log_fn(LOG_INFO,"Chose %s as intro point for %s.", exitname, conn->rend_query);
     }
 
@@ -770,7 +877,7 @@ circuit_get_open_circ_or_launch(connection_t *conn,
       if (conn->chosen_exit_name) {
         exitname = tor_strdup(conn->chosen_exit_name);
         if (!router_get_by_nickname(exitname)) {
-          log_fn(LOG_WARN,"Requested exit point '%s' is not known. Closing.", exitname);
+          log_fn(LOG_NOTICE,"Requested exit point '%s' is not known. Closing.", exitname);
           tor_free(exitname);
           return -1;
         }
@@ -784,13 +891,18 @@ circuit_get_open_circ_or_launch(connection_t *conn,
     else
       new_circ_purpose = desired_circuit_purpose;
 
-    circ = circuit_launch_by_nickname(new_circ_purpose, exitname);
+    is_internal = (new_circ_purpose != CIRCUIT_PURPOSE_C_GENERAL || is_resolve);
+    circ = circuit_launch_by_nickname(new_circ_purpose, exitname, need_uptime,
+                                      1, is_internal);
     tor_free(exitname);
 
-    if (circ &&
-        desired_circuit_purpose != CIRCUIT_PURPOSE_C_GENERAL) {
-      /* then write the service_id into circ */
-      strlcpy(circ->rend_query, conn->rend_query, sizeof(circ->rend_query));
+    if (desired_circuit_purpose != CIRCUIT_PURPOSE_C_GENERAL) {
+      /* help predict this next time */
+      rep_hist_note_used_hidserv(time(NULL), need_uptime, 1);
+      if (circ) {
+        /* write the service_id into circ */
+        strlcpy(circ->rend_query, conn->rend_query, sizeof(circ->rend_query));
+      }
     }
   }
   if (!circ)
@@ -819,6 +931,87 @@ static void link_apconn_to_circ(connection_t *apconn, circuit_t *circ) {
   apconn->cpath_layer = circ->cpath->prev;
 }
 
+/** If an exit wasn't specifically chosen, save the history for future
+ * use */
+static void
+consider_recording_trackhost(connection_t *conn, circuit_t *circ) {
+  int found_needle = 0;
+  char *str;
+  or_options_t *options = get_options();
+  size_t len;
+  char *new_address;
+
+  /* Search the addressmap for this conn's destination. */
+  /* If he's not in the address map.. */
+  if (!options->TrackHostExits ||
+      addressmap_already_mapped(conn->socks_request->address))
+    return; /* nothing to track, or already mapped */
+
+  SMARTLIST_FOREACH(options->TrackHostExits, const char *, cp, {
+    if (cp[0] == '.') { /* match end */
+      /* XXX strstr is probably really bad here */
+      if ((str = strstr(conn->socks_request->address, &cp[1]))) {
+        if (str == conn->socks_request->address
+          || strcmp(str, &cp[1]) == 0) {
+          found_needle = 1;
+        }
+      }
+    } else if (strcmp(cp, conn->socks_request->address) == 0) {
+      found_needle = 1;
+    }
+  });
+
+  if (!found_needle)
+    return;
+
+  /* Add this exit/hostname pair to the addressmap. */
+  len = strlen(conn->socks_request->address) + 1 /* '.' */ +
+        strlen(circ->build_state->chosen_exit_name) + 1 /* '.' */ +
+        strlen("exit") + 1 /* '\0' */;
+  new_address = tor_malloc(len);
+
+  tor_snprintf(new_address, len, "%s.%s.exit",
+               conn->socks_request->address,
+               circ->build_state->chosen_exit_name);
+
+  addressmap_register(conn->socks_request->address, new_address,
+                      time(NULL) + options->TrackHostExitsExpire);
+}
+
+/** Attempt to attach the connection <b>conn</b> to <b>circ</b>, and
+ * send a begin or resolve cell as appropriate.  Return values are as
+ * for connection_ap_handshake_attach_circuit. */
+int
+connection_ap_handshake_attach_chosen_circuit(connection_t *conn,
+                                              circuit_t *circ)
+{
+  tor_assert(conn);
+  tor_assert(conn->type == CONN_TYPE_AP);
+  tor_assert(conn->state == AP_CONN_STATE_CIRCUIT_WAIT ||
+             conn->state == AP_CONN_STATE_CONTROLLER_WAIT);
+  tor_assert(conn->socks_request);
+  tor_assert(circ);
+  tor_assert(circ->state == CIRCUIT_STATE_OPEN);
+
+  conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
+
+  if (!circ->timestamp_dirty)
+    circ->timestamp_dirty = time(NULL);
+
+  link_apconn_to_circ(conn, circ);
+  tor_assert(conn->socks_request);
+  if (conn->socks_request->command == SOCKS_COMMAND_CONNECT) {
+    consider_recording_trackhost(conn, circ);
+    if (connection_ap_handshake_send_begin(conn, circ)<0)
+      return -1;
+  } else {
+    if (connection_ap_handshake_send_resolve(conn, circ)<0)
+      return -1;
+  }
+
+  return 1;
+}
+
 /** Try to find a safe live circuit for CONN_TYPE_AP connection conn. If
  * we don't find one: if conn cannot be handled by any known nodes,
  * warn and return -1 (conn needs to die);
@@ -863,25 +1056,14 @@ int connection_ap_handshake_attach_circuit(connection_t *conn) {
     if (retval < 1)
       return retval;
 
-    /* We have found a suitable circuit for our conn. Hurray. */
-    tor_assert(circ);
-
-    log_fn(LOG_DEBUG,"Attaching apconn to general circ %d (stream %d sec old).",
+    log_fn(LOG_DEBUG,"Attaching apconn to circ %d (stream %d sec old).",
            circ->n_circ_id, conn_age);
     /* here, print the circ's path. so people can figure out which circs are sucking. */
     circuit_log_path(LOG_INFO,circ);
 
-    if (!circ->timestamp_dirty)
-      circ->timestamp_dirty = time(NULL);
+    /* We have found a suitable circuit for our conn. Hurray. */
+    return connection_ap_handshake_attach_chosen_circuit(conn, circ);
 
-    link_apconn_to_circ(conn, circ);
-    tor_assert(conn->socks_request);
-    if (conn->socks_request->command == SOCKS_COMMAND_CONNECT)
-      connection_ap_handshake_send_begin(conn, circ);
-    else
-      connection_ap_handshake_send_resolve(conn, circ);
-
-    return 1;
   } else { /* we're a rendezvous conn */
     circuit_t *rendcirc=NULL, *introcirc=NULL;
 
@@ -936,7 +1118,6 @@ int connection_ap_handshake_attach_circuit(connection_t *conn) {
       if (introcirc->state == CIRCUIT_STATE_OPEN) {
         log_fn(LOG_INFO,"found open intro circ %d (rend %d); sending introduction. (stream %d sec old)",
                introcirc->n_circ_id, rendcirc->n_circ_id, conn_age);
-        /* XXX here we should cannibalize the rend circ if it's a zero service id */
         if (rend_client_send_introduction(introcirc, rendcirc) < 0) {
           return -1;
         }
@@ -954,4 +1135,3 @@ int connection_ap_handshake_attach_circuit(connection_t *conn) {
     return 0;
   }
 }
-
