@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: config.c,v 1.286.2.11 2005/02/19 03:42:28 arma Exp $ */
-const char config_c_id[] = "$Id: config.c,v 1.286.2.11 2005/02/19 03:42:28 arma Exp $";
+/* $Id: config.c,v 1.330 2005/04/01 07:16:52 nickm Exp $ */
+const char config_c_id[] = "$Id: config.c,v 1.330 2005/04/01 07:16:52 nickm Exp $";
 
 /**
  * \file config.c
@@ -53,6 +53,7 @@ static config_abbrev_t config_abbrevs[] = {
   PLURAL(EntryNode),
   PLURAL(ExcludeNode),
   PLURAL(FirewallPort),
+  PLURAL(LongLivedPort),
   PLURAL(HiddenServiceNode),
   PLURAL(HiddenServiceExcludeNode),
   PLURAL(RendNode),
@@ -98,17 +99,18 @@ static config_var_t config_vars[] = {
   VAR("AuthoritativeDirectory",BOOL,   AuthoritativeDir,     "0"),
   VAR("BandwidthRate",       MEMUNIT,  BandwidthRate,        "1 MB"),
   VAR("BandwidthBurst",      MEMUNIT,  BandwidthBurst,       "5 MB"),
+  VAR("MaxAdvertisedBandwidth",MEMUNIT,MaxAdvertisedBandwidth,"128 TB"),
   VAR("ClientOnly",          BOOL,     ClientOnly,           "0"),
   VAR("ContactInfo",         STRING,   ContactInfo,          NULL),
   VAR("ControlPort",         UINT,     ControlPort,          "0"),
   VAR("CookieAuthentication",BOOL,     CookieAuthentication, "0"),
   VAR("DebugLogFile",        STRING,   DebugLogFile,         NULL),
   VAR("DataDirectory",       STRING,   DataDirectory,        NULL),
+  VAR("DirAllowPrivateAddresses",BOOL, DirAllowPrivateAddresses, NULL),
   VAR("DirPort",             UINT,     DirPort,              "0"),
   VAR("DirBindAddress",      LINELIST, DirBindAddress,       NULL),
-/* XXX we'd like dirfetchperiod to be higher for people with dirport not
- * set, but low for people with dirport set. how to have two defaults? */
-  VAR("DirFetchPeriod",      INTERVAL, DirFetchPeriod,       "1 hour"),
+  /** DOCDOC **/
+  VAR("DirFetchPeriod",      INTERVAL, DirFetchPeriod,       "0 seconds"),
   VAR("DirPostPeriod",       INTERVAL, DirPostPeriod,        "20 minutes"),
   VAR("RendPostPeriod",      INTERVAL, RendPostPeriod,       "20 minutes"),
   VAR("DirPolicy",           LINELIST, DirPolicy,            NULL),
@@ -119,6 +121,9 @@ static config_var_t config_vars[] = {
   VAR("StrictEntryNodes",    BOOL,     StrictEntryNodes,     "0"),
   VAR("ExitPolicy",          LINELIST, ExitPolicy,           NULL),
   VAR("ExcludeNodes",        STRING,   ExcludeNodes,         NULL),
+  VAR("TrackHostExits",      CSV,      TrackHostExits,       NULL),
+  VAR("TrackHostExitsExpire",INTERVAL, TrackHostExitsExpire, "30 minutes"),
+  VAR("MapAddress",          LINELIST, AddressMap,           NULL),
   VAR("FascistFirewall",     BOOL,     FascistFirewall,      "0"),
   VAR("FirewallPorts",       CSV,      FirewallPorts,        "80,443"),
   VAR("MyFamily",            STRING,   MyFamily,             NULL),
@@ -126,6 +131,7 @@ static config_var_t config_vars[] = {
   VAR("Group",               STRING,   Group,                NULL),
   VAR("HashedControlPassword",STRING,  HashedControlPassword, NULL),
   VAR("HttpProxy",           STRING,   HttpProxy,            NULL),
+  VAR("HttpsProxy",          STRING,   HttpsProxy,           NULL),
   VAR("HiddenServiceOptions",LINELIST_V, RendConfigLines,    NULL),
   VAR("HiddenServiceDir",    LINELIST_S, RendConfigLines,    NULL),
   VAR("HiddenServicePort",   LINELIST_S, RendConfigLines,    NULL),
@@ -144,11 +150,13 @@ static config_var_t config_vars[] = {
   VAR("AccountingMax",       MEMUNIT,   AccountingMax,        "0 bytes"),
   VAR("Nickname",            STRING,   Nickname,             NULL),
   VAR("NewCircuitPeriod",    INTERVAL, NewCircuitPeriod,     "30 seconds"),
+  VAR("MaxCircuitDirtiness", INTERVAL, MaxCircuitDirtiness,  "10 minutes"),
   VAR("NumCpus",             UINT,     NumCpus,              "1"),
   VAR("ORPort",              UINT,     ORPort,               "0"),
   VAR("ORBindAddress",       LINELIST, ORBindAddress,        NULL),
   VAR("OutboundBindAddress", STRING,   OutboundBindAddress,  NULL),
   VAR("PidFile",             STRING,   PidFile,              NULL),
+  VAR("LongLivedPorts",      CSV,      LongLivedPorts,       "21,22,706,1863,5050,5190,5222,5223,6667,8300,8888"),
   VAR("PathlenCoinWeight",   DOUBLE,   PathlenCoinWeight,    "0.3"),
   VAR("RedirectExit",        LINELIST, RedirectExit,         NULL),
   OBSOLETE("RouterFile"),
@@ -157,15 +165,16 @@ static config_var_t config_vars[] = {
   VAR("RecommendedVersions", LINELIST, RecommendedVersions,  NULL),
   VAR("RendNodes",           STRING,   RendNodes,            NULL),
   VAR("RendExcludeNodes",    STRING,   RendExcludeNodes,     NULL),
+  VAR("ShutdownWaitLength",  INTERVAL, ShutdownWaitLength,   "30 seconds"),
   VAR("SocksPort",           UINT,     SocksPort,            "9050"),
   VAR("SocksBindAddress",    LINELIST, SocksBindAddress,     NULL),
   VAR("SocksPolicy",         LINELIST, SocksPolicy,          NULL),
-/* XXX as with dirfetchperiod, we want this to be 15 minutes for people
- * with a dirport open, but higher for people without a dirport open. */
-  VAR("StatusFetchPeriod",   INTERVAL, StatusFetchPeriod,    "15 minutes"),
+  /** DOCDOC */
+  VAR("StatusFetchPeriod",   INTERVAL, StatusFetchPeriod,    "0 seconds"),
   VAR("SysLog",              LINELIST_S, OldLogOptions,      NULL),
   OBSOLETE("TrafficShaping"),
   VAR("User",                STRING,   User,                 NULL),
+  VAR("__LeaveStreamsUnattached", BOOL,LeaveStreamsUnattached, "0"),
   { NULL, CONFIG_TYPE_OBSOLETE, 0, NULL }
 };
 #undef VAR
@@ -183,6 +192,7 @@ static or_options_t *options_dup(or_options_t *old);
 static int options_validate(or_options_t *options);
 static int options_transition_allowed(or_options_t *old, or_options_t *new);
 static int check_nickname_list(const char *lst, const char *name);
+static void config_register_addressmaps(or_options_t *options);
 
 static int parse_dir_server_line(const char *line, int validate_only);
 static int parse_redirect_line(smartlist_t *result,
@@ -220,15 +230,7 @@ get_options(void) {
 }
 
 /** Change the current global options to contain <b>new_val</b> instead
- * of their current value; free the old value as necessary.  Where
- * <b>new_val</b> is different from the old value, update the process to
- * use the new value instead.
- *
- * Note 1: <b>new_val</b> must have previously been validated with
- * options_validate(), or Tor may freak out and exit.
- *
- * Note 2: We haven't moved all the "act on new configuration" logic
- * here yet.  Some is still in do_hup() and other places.
+ * of their current value; free the old value as necessary.
  */
 void
 set_options(or_options_t *new_val) {
@@ -237,27 +239,63 @@ set_options(or_options_t *new_val) {
   global_options = new_val;
 }
 
+void
+config_free_all(void)
+{
+  options_free(global_options);
+  tor_free(config_fname);
+}
+
 /** Fetch the active option list, and take actions based on it. All
  * of the things we do should survive being done repeatedly.
  * Return 0 if all goes well, return -1 if it's time to die.
+ *
+ * Note 1: <b>new_val</b> must have previously been validated with
+ * options_validate(), or Tor may freak out and exit.
+ *
+ * Note 2: We haven't moved all the "act on new configuration" logic
+ * here yet.  Some is still in do_hup() and other places.
  */
 int
 options_act(void) {
   struct config_line_t *cl;
   or_options_t *options = get_options();
+  static int libevent_initialized = 0;
+
+  /* XXXX009 We once had a reason to separate start_daemon and finish_daemon:
+   *    It let us have the parent process stick around until we were sure Tor
+   *    was started.  Should we make start_daemon get called earlier? -NM */
+  if (options->RunAsDaemon) {
+    start_daemon(options->DataDirectory);
+  }
+  if (!libevent_initialized) {
+    configure_libevent_logging();
+    suppress_libevent_log_msg("function not implemented");
+    event_init();
+    suppress_libevent_log_msg(NULL);
+#if defined(HAVE_EVENT_GET_VERSION) && defined(HAVE_EVENT_GET_METHOD)
+    /* Making this a NOTICE for now so we can link bugs to a libevent versions
+     * or methods better. */
+    log_fn(LOG_NOTICE, "Initialized libevent version %s using method %s",
+           event_get_version(), event_get_method());
+#else
+    log_fn(LOG_NOTICE, "Initialized old libevent (version 1.0b or earlier)");
+#endif
+    libevent_initialized = 1;
+  }
 
   clear_trusted_dir_servers();
   for (cl = options->DirServers; cl; cl = cl->next) {
     if (parse_dir_server_line(cl->value, 0)<0) {
       log_fn(LOG_ERR,
-             "Previously validated DirServer line could not be added!");
+             "Bug: Previously validated DirServer line could not be added!");
       return -1;
     }
   }
 
   if (rend_config_services(options, 0)<0) {
     log_fn(LOG_ERR,
-           "Previously validated hidden services line could not be added!");
+           "Bug: Previously validated hidden services line could not be added!");
     return -1;
   }
 
@@ -289,7 +327,7 @@ options_act(void) {
   add_callback_log(LOG_NOTICE, LOG_ERR, control_event_logmsg);
 
   options->_ConnLimit =
-    set_max_file_descriptors(options->ConnLimit, MAXCONNECTIONS);
+    set_max_file_descriptors((unsigned)options->ConnLimit, MAXCONNECTIONS);
   if (options->_ConnLimit < 0)
     return -1;
 
@@ -304,13 +342,6 @@ options_act(void) {
 
   /* Start backgrounding the process, if requested. */
 
-  /* XXXX009 We once had a reason to separate start_daemon and finish_daemon:
-   *    It let us have the parent process stick around until we were sure Tor
-   *    was started.  Should we make start_daemon get called earlier? -NM */
-  if (options->RunAsDaemon) {
-    start_daemon(options->DataDirectory);
-  }
-
   /* Finish backgrounding the process */
   if (options->RunAsDaemon) {
     /* We may be calling this for the n'th time (on SIGHUP), but it's safe. */
@@ -322,6 +353,9 @@ options_act(void) {
   if (options->PidFile)
     write_pidfile(options->PidFile);
 
+  /* Register addressmap directives */
+  config_register_addressmaps(options);
+
   /* Update address policies. */
   parse_socks_policy();
   parse_dir_policy();
@@ -330,7 +364,7 @@ options_act(void) {
 
   /* reload keys as needed for rendezvous services. */
   if (rend_service_load_keys()<0) {
-    log_fn(LOG_ERR,"Error reloading rendezvous service keys");
+    log_fn(LOG_ERR,"Error loading rendezvous service keys");
     return -1;
   }
 
@@ -507,6 +541,9 @@ config_free_lines(struct config_line_t *front)
 static config_var_t *config_find_option(const char *key)
 {
   int i;
+  size_t keylen = strlen(key);
+  if (!keylen)
+    return NULL; /* if they say "--" on the commandline, it's not an option */
   /* First, check for an exact (case-insensitive) match */
   for (i=0; config_vars[i].name; ++i) {
     if (!strcasecmp(key, config_vars[i].name))
@@ -514,7 +551,7 @@ static config_var_t *config_find_option(const char *key)
   }
   /* If none, check for an abbreviated match */
   for (i=0; config_vars[i].name; ++i) {
-    if (!strncasecmp(key, config_vars[i].name, strlen(key))) {
+    if (!strncasecmp(key, config_vars[i].name, keylen)) {
       log_fn(LOG_WARN, "The abbreviation '%s' is deprecated. "
           "Tell Nick and Roger to make it official, or just use '%s' instead",
              key, config_vars[i].name);
@@ -907,6 +944,8 @@ resolve_my_address(const char *address, uint32_t *addr)
   struct hostent *rent;
   char hostname[256];
   int explicit_ip=1;
+  char tmpbuf[INET_NTOA_BUF_LEN];
+  static uint32_t old_addr=0;
 
   tor_assert(addr);
 
@@ -936,20 +975,26 @@ resolve_my_address(const char *address, uint32_t *addr)
     memcpy(&in.s_addr, rent->h_addr, rent->h_length);
   }
 
+  tor_inet_ntoa(&in,tmpbuf,sizeof(tmpbuf));
   if (!explicit_ip && is_internal_IP(htonl(in.s_addr))) {
     log_fn(LOG_WARN,"Address '%s' resolves to private IP '%s'. "
            "Please set the Address config option to be the IP you want to use.",
-           hostname, inet_ntoa(in));
+           hostname, tmpbuf);
     return -1;
   }
 
-  log_fn(LOG_DEBUG, "Resolved Address to %s.", inet_ntoa(in));
+  log_fn(LOG_DEBUG, "Resolved Address to %s.", tmpbuf);
   *addr = ntohl(in.s_addr);
+  if (old_addr && old_addr != *addr) {
+    log_fn(LOG_NOTICE,"Your IP seems to have changed. Updating.");
+    server_has_changed_ip();
+  }
+  old_addr = *addr;
   return 0;
 }
 
 /** Called when we don't have a nickname set.  Try to guess a good
- * nickname based on the hostname, and return it. */
+ * nickname based on the hostname, and return it in a newly allocated string. */
 static char *
 get_default_nickname(void)
 {
@@ -1122,6 +1167,9 @@ config_dump_options(or_options_t *options, int minimal)
     if (config_vars[i].type == CONFIG_TYPE_OBSOLETE ||
         config_vars[i].type == CONFIG_TYPE_LINELIST_S)
       continue;
+    /* Don't save 'hidden' control variables. */
+    if (!strcmpstart(config_vars[i].name, "__"))
+      continue;
     if (minimal && option_is_same(options, defaults, config_vars[i].name))
       continue;
     line = config_get_assigned_option(options, config_vars[i].name);
@@ -1144,13 +1192,32 @@ config_dump_options(or_options_t *options, int minimal)
   return result;
 }
 
+static int
+validate_ports_csv(smartlist_t *sl, const char *name) {
+  int i;
+  int result = 0;
+  tor_assert(name);
+
+  if (!sl)
+    return 0;
+
+  SMARTLIST_FOREACH(sl, const char *, cp,
+  {
+    i = atoi(cp);
+    if (i < 1 || i > 65535) {
+      log(LOG_WARN, "Port '%s' out of range in %s", cp, name);
+      result=-1;
+    }
+  });
+  return result;
+}
+
 /** Return 0 if every setting in <b>options</b> is reasonable.  Else
  * warn and return -1.  Should have no side effects, except for
  * normalizing the contents of <b>options</b>. */
 static int
 options_validate(or_options_t *options)
 {
-  int i;
   int result = 0;
   struct config_line_t *cl;
   addr_policy_t *addr_policy=NULL;
@@ -1262,22 +1329,26 @@ options_validate(or_options_t *options)
     result = -1;
   }
 
+  if (options->ConnLimit <= 0) {
+    log(LOG_WARN, "ConnLimit must be greater than 0, but was set to %d",
+        options->ConnLimit);
+    result = -1;
+  }
+
   if (options->_AccountingMaxKB) {
     log(LOG_WARN, "AccountingMaxKB is deprecated.  Say 'AccountingMax %d KB' instead.", options->_AccountingMaxKB);
     options->AccountingMax = U64_LITERAL(1024)*options->_AccountingMaxKB;
     options->_AccountingMaxKB = 0;
   }
 
-  if (options->FirewallPorts) {
-    SMARTLIST_FOREACH(options->FirewallPorts, const char *, cp,
-    {
-      i = atoi(cp);
-      if (i < 1 || i > 65535) {
-        log(LOG_WARN, "Port '%s' out of range in FirewallPorts", cp);
-        result=-1;
-      }
-    });
-  }
+  if (validate_ports_csv(options->FirewallPorts,
+                         "FirewallPorts") < 0)
+    result = -1;
+
+  if (validate_ports_csv(options->LongLivedPorts,
+                         "LongLivedPorts") < 0)
+    result = -1;
+
   options->_AllowUnverified = 0;
   if (options->AllowUnverifiedNodes) {
     SMARTLIST_FOREACH(options->AllowUnverifiedNodes, const char *, cp, {
@@ -1294,7 +1365,7 @@ options_validate(or_options_t *options)
         else {
           log(LOG_WARN, "Unrecognized value '%s' in AllowUnverifiedNodes",
               cp);
-          result=-1;
+          result = -1;
         }
       });
   }
@@ -1310,16 +1381,17 @@ options_validate(or_options_t *options)
 #define MIN_REND_POST_PERIOD 300
 #define MIN_STATUS_FETCH_PERIOD 60
 
-  /* After 0.0.8 is dead, change this to MIN_ONION_KEY_LIFETIME. */
-#define MAX_DIR_PERIOD (OLD_MIN_ONION_KEY_LIFETIME/2)
+#define MAX_DIR_PERIOD (MIN_ONION_KEY_LIFETIME/2)
 #define MAX_CACHE_DIR_FETCH_PERIOD 3600
 #define MAX_CACHE_STATUS_FETCH_PERIOD 900
 
-  if (options->DirFetchPeriod < MIN_DIR_FETCH_PERIOD) {
+  if (options->DirFetchPeriod &&
+      options->DirFetchPeriod < MIN_DIR_FETCH_PERIOD) {
     log(LOG_WARN, "DirFetchPeriod option must be at least %d seconds. Clipping.", MIN_DIR_FETCH_PERIOD);
     options->DirFetchPeriod = MIN_DIR_FETCH_PERIOD;
   }
-  if (options->StatusFetchPeriod < MIN_STATUS_FETCH_PERIOD) {
+  if (options->StatusFetchPeriod &&
+      options->StatusFetchPeriod < MIN_STATUS_FETCH_PERIOD) {
     log(LOG_WARN, "StatusFetchPeriod option must be at least %d seconds. Clipping.", MIN_STATUS_FETCH_PERIOD);
     options->StatusFetchPeriod = MIN_STATUS_FETCH_PERIOD;
   }
@@ -1367,10 +1439,6 @@ options_validate(or_options_t *options)
     result = -1;
   }
 
-  if (options->BandwidthRate > options->BandwidthBurst) {
-    log(LOG_WARN,"BandwidthBurst must be at least equal to BandwidthRate.");
-    result = -1;
-  }
   if (options->BandwidthRate > INT_MAX) {
     log(LOG_WARN,"BandwidthRate must be less than %d",INT_MAX);
     result = -1;
@@ -1379,6 +1447,20 @@ options_validate(or_options_t *options)
     log(LOG_WARN,"BandwidthBurst must be less than %d",INT_MAX);
     result = -1;
   }
+  if (server_mode(options) &&
+      options->BandwidthRate < ROUTER_REQUIRED_MIN_BANDWIDTH*2) {
+    log(LOG_WARN,"BandwidthRate is set to %d bytes/second. For servers, it must be at least %d.", (int)options->BandwidthRate, ROUTER_REQUIRED_MIN_BANDWIDTH*2);
+    result = -1;
+  }
+  if (options->BandwidthRate > options->BandwidthBurst) {
+    log(LOG_WARN,"BandwidthBurst must be at least equal to BandwidthRate.");
+    result = -1;
+  }
+#if 0
+  if (2*options->BandwidthRate > options->BandwidthBurst) {
+    log(LOG_NOTICE,"You have chosen a BandwidthBurst less than twice BandwidthRate. Please consider setting your BandwidthBurst higher (at least %d), to provide better service to the Tor network.", (int)(2*options->BandwidthRate));
+  }
+#endif
 
   if (options->_MonthlyAccountingStart) {
     if (options->AccountingStart) {
@@ -1411,6 +1493,17 @@ options_validate(or_options_t *options)
     }
   }
 
+  if (options->HttpsProxy) { /* parse it now */
+    if (parse_addr_port(options->HttpsProxy, NULL,
+                        &options->HttpsProxyAddr, &options->HttpsProxyPort) < 0) {
+      log(LOG_WARN,"HttpsProxy failed to parse or resolve. Please fix.");
+      result = -1;
+    }
+    if (options->HttpsProxyPort == 0) { /* give it a default */
+      options->HttpsProxyPort = 443;
+    }
+  }
+
   if (options->HashedControlPassword) {
     if (decode_hashed_password(NULL, options->HashedControlPassword)<0) {
       log_fn(LOG_WARN,"Bad HashedControlPassword: wrong length or bad base64");
@@ -1443,6 +1536,11 @@ options_validate(or_options_t *options)
     log_fn(LOG_WARN, "Error in Exit Policy entry.");
     result = -1;
   }
+  if (server_mode(options)) {
+    exit_policy_implicitly_allows_local_networks(addr_policy, 1);
+  }
+  /* The rest of these calls *append* to addr_policy. So don't actually
+   * use the results for anything other than checking if they parse! */
   if (config_parse_addr_policy(options->DirPolicy, &addr_policy)) {
     log_fn(LOG_WARN, "Error in DirPolicy entry.");
     result = -1;
@@ -1742,6 +1840,39 @@ init_from_config(int argc, char **argv)
   return -1;
 }
 
+static void
+config_register_addressmaps(or_options_t *options) {
+  smartlist_t *elts;
+  struct config_line_t *opt;
+  char *from, *to;
+
+  addressmap_clear_configured();
+  elts = smartlist_create();
+  for (opt = options->AddressMap; opt; opt = opt->next) {
+    smartlist_split_string(elts, opt->value, NULL,
+                           SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 2);
+    if (smartlist_len(elts) >= 2) {
+      from = smartlist_get(elts,0);
+      to = smartlist_get(elts,1);
+      if (!is_plausible_address(from)) {
+        log_fn(LOG_WARN,"Skipping invalid argument '%s' to MapAddress",from);
+      } else if (!is_plausible_address(to)) {
+        log_fn(LOG_WARN,"Skipping invalid argument '%s' to MapAddress",to);
+      } else {
+        addressmap_register(from, tor_strdup(to), 0);
+        if (smartlist_len(elts)>2) {
+          log_fn(LOG_WARN,"Ignoring extra arguments to MapAddress.");
+        }
+      }
+    } else {
+      log_fn(LOG_WARN,"MapAddress '%s' has too few arguments. Ignoring.", opt->value);
+    }
+    SMARTLIST_FOREACH(elts, char*, cp, tor_free(cp));
+    smartlist_clear(elts);
+  }
+  smartlist_free(elts);
+}
+
 /** If <b>range</b> is of the form MIN-MAX, for MIN and MAX both
  * recognized log severity levels, set *<b>min_out</b> to MIN and
  * *<b>max_out</b> to MAX and return 0.  Else, if <b>range<b> is of
@@ -2222,7 +2353,6 @@ validate_data_directory(or_options_t *options) {
 static int
 write_configuration_file(const char *fname, or_options_t *options)
 {
-  char fn_tmp[1024];
   char *old_val=NULL, *new_val=NULL, *new_conf=NULL;
   int rename_old = 0, r;
   size_t len;
@@ -2255,9 +2385,14 @@ write_configuration_file(const char *fname, or_options_t *options)
 
   if (rename_old) {
     int i = 1;
+    size_t fn_tmp_len = strlen(fname)+32;
+    char *fn_tmp;
+    tor_assert(fn_tmp_len > strlen(fname)); /*check for overflow*/
+    fn_tmp = tor_malloc(fn_tmp_len);
     while (1) {
-      if (tor_snprintf(fn_tmp, sizeof(fn_tmp), "%s.orig.%d", fname, i)<0) {
-        log_fn(LOG_WARN, "Filename too long");
+      if (tor_snprintf(fn_tmp, fn_tmp_len, "%s.orig.%d", fname, i)<0) {
+        log_fn(LOG_WARN, "tor_snprintf failed inexplicably");
+        tor_free(fn_tmp);
         goto err;
       }
       if (file_status(fn_tmp) == FN_NOENT)
@@ -2266,6 +2401,7 @@ write_configuration_file(const char *fname, or_options_t *options)
     }
     log_fn(LOG_NOTICE, "Renaming old configuration file to %s", fn_tmp);
     rename(fname, fn_tmp);
+    tor_free(fn_tmp);
   }
 
   write_str_to_file(fname, new_val, 0);
@@ -2408,7 +2544,6 @@ print_cvs_version(void)
   extern const char compat_c_id[];
   extern const char container_c_id[];
   extern const char crypto_c_id[];
-  extern const char fakepoll_c_id[];
   extern const char log_c_id[];
   extern const char torgzip_c_id[];
   extern const char tortls_c_id[];
@@ -2445,7 +2580,6 @@ print_cvs_version(void)
   puts(COMPAT_H_ID);
   puts(CONTAINER_H_ID);
   puts(CRYPTO_H_ID);
-  puts(FAKEPOLL_H_ID);
   puts(LOG_H_ID);
   puts(TORGZIP_H_ID);
   puts(TORINT_H_ID);
@@ -2455,7 +2589,6 @@ print_cvs_version(void)
   puts(compat_c_id);
   puts(container_c_id);
   puts(crypto_c_id);
-  puts(fakepoll_c_id);
   puts(log_c_id);
   puts(torgzip_c_id);
   puts(tortls_c_id);
