@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c,v 1.310.2.13 2005/03/27 07:17:23 arma Exp $ */
-const char connection_c_id[] = "$Id: connection.c,v 1.310.2.13 2005/03/27 07:17:23 arma Exp $";
+/* $Id: connection.c,v 1.350 2005/03/31 17:36:24 arma Exp $ */
+const char connection_c_id[] = "$Id: connection.c,v 1.350 2005/03/31 17:36:24 arma Exp $";
 
 /**
  * \file connection.c
@@ -56,10 +56,11 @@ const char *conn_state_to_string[][_CONN_TYPE_MAX+1] = {
     "", /* 4 */
     "awaiting dest info",         /* app, 5 */
     "waiting for rendezvous desc",     /* 6 */
-    "waiting for safe circuit",        /* 7 */
-    "waiting for connected",           /* 8 */
-    "waiting for resolve",             /* 9 */
-    "open" },                          /* 10 */
+    "waiting for controller",          /* 7 */
+    "waiting for safe circuit",        /* 8 */
+    "waiting for connected",           /* 9 */
+    "waiting for resolve",             /* 10 */
+    "open" },                          /* 11 */
   { "ready" }, /* dir listener, 0 */
   { "",                           /* dir, 0 */
     "connecting",                      /* 1 */
@@ -110,6 +111,7 @@ static int connection_bucket_read_limit(connection_t *conn);
  * Initialize conn's timestamps to now.
  */
 connection_t *connection_new(int type) {
+  static uint32_t n_connections_allocated = 0;
   connection_t *conn;
   time_t now = time(NULL);
 
@@ -117,6 +119,7 @@ connection_t *connection_new(int type) {
   conn->magic = CONNECTION_MAGIC;
   conn->s = -1; /* give it a default of 'not used' */
   conn->poll_index = -1; /* also default to 'not used' */
+  conn->global_identifier = n_connections_allocated++;
 
   conn->type = type;
   if (!connection_is_listener(conn)) { /* listeners never use their buf */
@@ -136,12 +139,28 @@ connection_t *connection_new(int type) {
   return conn;
 }
 
+/** Tell libevent that we don't care about <b>conn</b> any more. */
+void
+connection_unregister(connection_t *conn)
+{
+  if (conn->read_event) {
+    if (event_del(conn->read_event))
+      log_fn(LOG_WARN, "Error removing read event for %d", (int)conn->s);
+    tor_free(conn->read_event);
+  }
+  if (conn->write_event) {
+    if (event_del(conn->write_event))
+      log_fn(LOG_WARN, "Error removing write event for %d", (int)conn->s);
+    tor_free(conn->write_event);
+  }
+}
+
 /** Deallocate memory used by <b>conn</b>. Deallocate its buffers if necessary,
  * close its socket if necessary, and mark the directory as dirty if <b>conn</b>
  * is an OR or OP connection.
  */
-void connection_free(connection_t *conn) {
-  tor_assert(conn);
+static void
+_connection_free(connection_t *conn) {
   tor_assert(conn->magic == CONNECTION_MAGIC);
 
   if (!connection_is_listener(conn)) {
@@ -154,8 +173,10 @@ void connection_free(connection_t *conn) {
   if (connection_speaks_cells(conn)) {
     if (conn->state == OR_CONN_STATE_OPEN)
       directory_set_dirty();
-    if (conn->tls)
+    if (conn->tls) {
       tor_tls_free(conn->tls);
+      conn->tls = NULL;
+    }
   }
 
   if (conn->identity_pkey)
@@ -163,17 +184,32 @@ void connection_free(connection_t *conn) {
   tor_free(conn->nickname);
   tor_free(conn->socks_request);
 
+  connection_unregister(conn);
+
   if (conn->s >= 0) {
     log_fn(LOG_INFO,"closing fd %d.",conn->s);
     tor_close_socket(conn->s);
   }
+
   memset(conn, 0xAA, sizeof(connection_t)); /* poison memory */
   tor_free(conn);
 }
 
-/** Call connection_free() on every connection in our array.
+/** Make sure <b>conn</b> isn't in any of the global conn lists; then free it.
+ */
+void connection_free(connection_t *conn) {
+  tor_assert(conn);
+  tor_assert(!connection_is_on_closeable_list(conn));
+  tor_assert(!connection_in_array(conn));
+  _connection_free(conn);
+}
+
+/** Call _connection_free() on every connection in our array.
  * This is used by cpuworkers and dnsworkers when they fork,
  * so they don't keep resources held open (especially sockets).
+ *
+ * Don't do the checks in connection_free(), because they will
+ * fail.
  */
 void connection_free_all(void) {
   int i, n;
@@ -181,7 +217,7 @@ void connection_free_all(void) {
 
   get_connection_array(&carray,&n);
   for (i=0;i<n;i++)
-    connection_free(carray[i]);
+    _connection_free(carray[i]);
 }
 
 /** Do any cleanup needed:
@@ -195,12 +231,17 @@ void connection_free_all(void) {
  */
 void connection_about_to_close_connection(connection_t *conn)
 {
+  circuit_t *circ;
 
   assert(conn->marked_for_close);
 
-  if (conn->type == CONN_TYPE_AP || conn->type == CONN_TYPE_EXIT) {
-    if (!conn->has_sent_end)
+  if (CONN_IS_EDGE(conn)) {
+    if (!conn->has_sent_end) {
       log_fn(LOG_WARN,"Harmless bug: Edge connection (marked at %s:%d) hasn't sent end yet?", conn->marked_for_close_file, conn->marked_for_close);
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
+    }
   }
 
   switch (conn->type) {
@@ -211,7 +252,7 @@ void connection_about_to_close_connection(connection_t *conn)
         connection_dir_connect_failed(conn);
       }
       if (conn->purpose == DIR_PURPOSE_FETCH_RENDDESC)
-        rend_client_desc_fetched(conn->rend_query, 0);
+        rend_client_desc_here(conn->rend_query); /* give it a try */
       break;
     case CONN_TYPE_OR:
       /* Remember why we're closing this connection. */
@@ -239,19 +280,19 @@ void connection_about_to_close_connection(connection_t *conn)
       break;
     case CONN_TYPE_AP:
       if (conn->socks_request->has_finished == 0) {
-        log_fn(LOG_INFO,"Cleaning up AP -- sending socks reject.");
-        conn->hold_open_until_flushed = 1;
-        /* XXX this socks_reply never gets sent, since conn
-         * gets removed right after this function finishes. */
-        connection_ap_handshake_socks_reply(conn, NULL, 0, -1);
-        conn->socks_request->has_finished = 1;
+        /* since conn gets removed right after this function finishes,
+         * there's no point trying to send back a reply at this point. */
+        log_fn(LOG_WARN,"Bug: Closing stream (marked at %s:%d) without sending back a socks reply.",
+               conn->marked_for_close_file, conn->marked_for_close);
       } else {
         control_event_stream_status(conn, STREAM_EVENT_CLOSED);
       }
       break;
     case CONN_TYPE_EXIT:
       if (conn->state == EXIT_CONN_STATE_RESOLVING) {
-        circuit_detach_stream(circuit_get_by_conn(conn), conn);
+        circ = circuit_get_by_conn(conn);
+        if (circ)
+          circuit_detach_stream(circ, conn);
         connection_dns_remove(conn);
       }
       break;
@@ -272,6 +313,9 @@ void connection_close_immediate(connection_t *conn)
   assert_connection_ok(conn,0);
   if (conn->s < 0) {
     log_fn(LOG_WARN,"Bug: Attempt to close already-closed connection.");
+#ifdef TOR_FRAGILE
+    tor_assert(0);
+#endif
     return;
   }
   if (conn->outbuf_flushlen) {
@@ -279,6 +323,9 @@ void connection_close_immediate(connection_t *conn)
            conn->s, CONN_TYPE_TO_STRING(conn->type),
            conn->state, (int)conn->outbuf_flushlen);
   }
+
+  connection_unregister(conn);
+
   tor_close_socket(conn->s);
   conn->s = -1;
   if (!connection_is_listener(conn)) {
@@ -296,10 +343,14 @@ _connection_mark_for_close(connection_t *conn)
 
   if (conn->marked_for_close) {
     log(LOG_WARN, "Bug: Double mark-for-close on connection.");
+#ifdef TOR_FRAGILE
+    tor_assert(0);
+#endif
     return -1;
   }
 
   conn->marked_for_close = 1;
+  add_connection_to_closeable_list(conn);
 
   /* in case we're going to be held-open-til-flushed, reset
    * the number of seconds since last successful write, so
@@ -352,7 +403,9 @@ static int connection_create_listener(const char *bindaddress, uint16_t bindport
   uint16_t usePort;
   uint32_t addr;
   int s; /* the socket we're going to make */
+#ifndef MS_WINDOWS
   int one=1;
+#endif
 
   memset(&bindaddr,0,sizeof(struct sockaddr_in));
   if (parse_addr_port(bindaddress, NULL, &addr, &usePort)<0) {
@@ -376,7 +429,13 @@ static int connection_create_listener(const char *bindaddress, uint16_t bindport
     return -1;
   }
 
+#ifndef MS_WINDOWS
+  /* REUSEADDR on normal places means you can rebind to the port
+   * right after somebody else has let it go. But REUSEADDR on win32
+   * means to let you bind to the port _even when somebody else
+   * already has it bound_. So, don't do that on Win32. */
   setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (void*) &one, sizeof(one));
+#endif
 
   if (bind(s,(struct sockaddr *)&bindaddr,sizeof(bindaddr)) < 0) {
     log_fn(LOG_WARN,"Could not bind to port %u: %s",usePort,
@@ -409,29 +468,6 @@ static int connection_create_listener(const char *bindaddress, uint16_t bindport
   return 0;
 }
 
-static int
-check_sockaddr_in(struct sockaddr *sa, int len, int level)
-{
-  int ok = 1;
-  struct sockaddr_in *sin=(struct sockaddr_in*)sa;
-
-  if (len != sizeof(struct sockaddr_in)) {
-    log_fn(level, "Length of address not as expected: %d vs %d",
-           len,(int)sizeof(struct sockaddr_in));
-    ok = 0;
-  }
-  if (sa->sa_family != AF_INET) {
-    log_fn(level, "Family of address not as expected: %d vs %d",
-           sa->sa_family, AF_INET);
-    ok = 0;
-  }
-  if (sin->sin_addr.s_addr == 0 || sin->sin_port == 0) {
-    log_fn(level, "Address for new connection has address/port equal to zero.");
-    ok = 0;
-  }
-  return ok ? 0 : -1;
-}
-
 /** The listener connection <b>conn</b> told poll() it wanted to read.
  * Call accept() on conn-\>s, and add the new connection if necessary.
  */
@@ -440,13 +476,11 @@ static int connection_handle_listener_read(connection_t *conn, int new_type) {
   connection_t *newconn;
   /* information about the remote peer when connecting to other routers */
   struct sockaddr_in remote;
-  char addrbuf[256];
   /* length of the remote address. Must be an int, since accept() needs that. */
-  int remotelen=256;
-  tor_assert(remotelen >= sizeof(struct sockaddr_in));
-  memset(addrbuf, 0, sizeof(addrbuf));
+  int remotelen = sizeof(struct sockaddr_in);
+  char tmpbuf[INET_NTOA_BUF_LEN];
 
-  news = accept(conn->s,(struct sockaddr *)addrbuf,&remotelen);
+  news = accept(conn->s,(struct sockaddr *)&remote,&remotelen);
   if (!SOCKET_IS_POLLABLE(news)) {
     /* accept() error, or too many conns to poll */
     int e;
@@ -473,29 +507,14 @@ static int connection_handle_listener_read(connection_t *conn, int new_type) {
   log(LOG_INFO,"Connection accepted on socket %d (child of fd %d).",news, conn->s);
 
   set_socket_nonblocking(news);
-  if (check_sockaddr_in((struct sockaddr*)addrbuf, remotelen, LOG_INFO)<0) {
-    log_fn(LOG_INFO, "accept() returned a strange address; trying getsockname().");
-    remotelen=256;
-    memset(addrbuf, 0, sizeof(addrbuf));
-    if (getsockname(news, (struct sockaddr*)addrbuf, &remotelen)<0) {
-      log_fn(LOG_WARN, "getsockname() failed.");
-    } else {
-      if (check_sockaddr_in((struct sockaddr*)addrbuf, remotelen, LOG_WARN)<0) {
-        log_fn(LOG_WARN,"Something's wrong with this conn. Closing it.");
-        tor_close_socket(news);
-        return 0;
-      }
-    }
-  }
-
-  memcpy(&remote, addrbuf, sizeof(struct sockaddr_in));
 
   /* process entrance policies here, before we even create the connection */
   if (new_type == CONN_TYPE_AP) {
     /* check sockspolicy to see if we should accept it */
     if (socks_policy_permits_address(ntohl(remote.sin_addr.s_addr)) == 0) {
+      tor_inet_ntoa(&remote.sin_addr, tmpbuf, sizeof(tmpbuf));
       log_fn(LOG_NOTICE,"Denying socks connection from untrusted address %s.",
-             inet_ntoa(remote.sin_addr));
+             tmpbuf);
       tor_close_socket(news);
       return 0;
     }
@@ -503,8 +522,9 @@ static int connection_handle_listener_read(connection_t *conn, int new_type) {
   if (new_type == CONN_TYPE_DIR) {
     /* check dirpolicy to see if we should accept it */
     if (dir_policy_permits_address(ntohl(remote.sin_addr.s_addr)) == 0) {
+      tor_inet_ntoa(&remote.sin_addr, tmpbuf, sizeof(tmpbuf));
       log_fn(LOG_NOTICE,"Denying dir connection from address %s.",
-             inet_ntoa(remote.sin_addr));
+             tmpbuf);
       tor_close_socket(news);
       return 0;
     }
@@ -514,13 +534,11 @@ static int connection_handle_listener_read(connection_t *conn, int new_type) {
   newconn->s = news;
 
   /* remember the remote address */
-  newconn->address = tor_strdup(inet_ntoa(remote.sin_addr));
+  newconn->address = tor_malloc(INET_NTOA_BUF_LEN);
+  tor_inet_ntoa(&remote.sin_addr, newconn->address, INET_NTOA_BUF_LEN);
+
   newconn->addr = ntohl(remote.sin_addr.s_addr);
   newconn->port = ntohs(remote.sin_port);
-  if (!newconn->addr || !newconn->port) {
-    log_fn(LOG_WARN,"New conn (type %d) has zero addr/port: addr %s, port %d.",
-           new_type, newconn->address, newconn->port);
-  }
 
   if (connection_add(newconn) < 0) { /* no space, forget it */
     connection_free(newconn);
@@ -767,19 +785,21 @@ static int connection_bucket_read_limit(connection_t *conn) {
     if (at_most > conn->receiver_bucket)
       at_most = conn->receiver_bucket;
 
+  if (at_most < 0)
+    return 0;
   return at_most;
 }
 
 /** We just read num_read onto conn. Decrement buckets appropriately. */
 static void connection_read_bucket_decrement(connection_t *conn, int num_read) {
-  global_read_bucket -= num_read; tor_assert(global_read_bucket >= 0);
+  global_read_bucket -= num_read; //tor_assert(global_read_bucket >= 0);
   if (connection_speaks_cells(conn) && conn->state == OR_CONN_STATE_OPEN) {
-    conn->receiver_bucket -= num_read; tor_assert(conn->receiver_bucket >= 0);
+    conn->receiver_bucket -= num_read; //tor_assert(conn->receiver_bucket >= 0);
   }
 }
 
 static void connection_consider_empty_buckets(connection_t *conn) {
-  if (global_read_bucket == 0) {
+  if (global_read_bucket <= 0) {
     log_fn(LOG_DEBUG,"global bucket exhausted. Pausing.");
     conn->wants_to_read = 1;
     connection_stop_reading(conn);
@@ -787,7 +807,7 @@ static void connection_consider_empty_buckets(connection_t *conn) {
   }
   if (connection_speaks_cells(conn) &&
       conn->state == OR_CONN_STATE_OPEN &&
-      conn->receiver_bucket == 0) {
+      conn->receiver_bucket <= 0) {
     log_fn(LOG_DEBUG,"receiver bucket exhausted. Pausing.");
     conn->wants_to_read = 1;
     connection_stop_reading(conn);
@@ -810,11 +830,11 @@ void connection_bucket_refill(struct timeval *now) {
   or_options_t *options = get_options();
 
   /* refill the global buckets */
-  if ((unsigned)global_read_bucket < options->BandwidthBurst) {
+  if (global_read_bucket < (int)options->BandwidthBurst) {
     global_read_bucket += (int)options->BandwidthRate;
     log_fn(LOG_DEBUG,"global_read_bucket now %d.", global_read_bucket);
   }
-  if ((unsigned)global_write_bucket < options->BandwidthBurst) {
+  if (global_write_bucket < (int)options->BandwidthBurst) {
     global_write_bucket += (int)options->BandwidthRate;
     log_fn(LOG_DEBUG,"global_write_bucket now %d.", global_write_bucket);
   }
@@ -859,7 +879,6 @@ static int connection_receiver_bucket_should_increase(connection_t *conn) {
   if (conn->state != OR_CONN_STATE_OPEN)
     return 0; /* only open connections play the rate limiting game */
 
-  tor_assert(conn->bandwidth > 0);
   if (conn->receiver_bucket >= conn->bandwidth)
     return 0;
 
@@ -900,10 +919,8 @@ loop_again:
   if (connection_read_to_buf(conn, &max_to_read) < 0) {
     /* There's a read error; kill the connection.*/
     connection_close_immediate(conn); /* Don't flush; connection is dead. */
-    if (conn->type == CONN_TYPE_AP || conn->type == CONN_TYPE_EXIT) {
-      connection_edge_end(conn, (char)(connection_state_is_open(conn) ?
-                          END_STREAM_REASON_MISC : END_STREAM_REASON_CONNECTREFUSED),
-                          conn->cpath_layer);
+    if (CONN_IS_EDGE(conn)) {
+      connection_edge_end_errno(conn, conn->cpath_layer);
     }
     connection_mark_for_close(conn);
     return -1;
@@ -916,7 +933,8 @@ loop_again:
     }
     if (!conn->marked_for_close &&
         connection_is_reading(conn) &&
-        !conn->inbuf_reached_eof)
+        !conn->inbuf_reached_eof &&
+        max_to_read > 0)
       goto loop_again; /* try reading again, in case more is here now */
   }
   /* one last try, packaging partial cells and all. */
@@ -950,7 +968,8 @@ static int connection_read_to_buf(connection_t *conn, int *max_to_read) {
     at_most = connection_bucket_read_limit(conn);
   }
 
-  if (connection_speaks_cells(conn) && conn->state != OR_CONN_STATE_CONNECTING) {
+  if (connection_speaks_cells(conn) && conn->state > OR_CONN_STATE_PROXY_READING) {
+    int pending;
     if (conn->state == OR_CONN_STATE_HANDSHAKING) {
       /* continue handshaking even if global token bucket is empty */
       return connection_tls_continue_handshake(conn);
@@ -978,12 +997,28 @@ static int connection_read_to_buf(connection_t *conn, int *max_to_read) {
       case TOR_TLS_DONE: /* no data read, so nothing to process */
         result = 0;
         break; /* so we call bucket_decrement below */
+      default:
+        break;
     }
+    pending = tor_tls_get_pending_bytes(conn->tls);
+    if (pending) {
+      /* XXXX If we have any pending bytes, read them now.  This *can*
+       * take us over our read alotment, but really we shouldn't be
+       * believing that SSL bytes are the same as TCP bytes anyway. */
+      int r2 = read_to_buf_tls(conn->tls, pending, conn->inbuf);
+      if (r2<0) {
+        log_fn(LOG_WARN, "Bug: apparently, reading pending bytes can fail.");
+        return -1;
+      } else {
+        result += r2;
+      }
+    }
+
   } else {
     result = read_to_buf(conn->s, at_most, conn->inbuf,
                          &conn->inbuf_reached_eof);
 
-//  log(LOG_DEBUG,"connection_read_to_buf(): read_to_buf returned %d.",read_result);
+//  log_fn(LOG_DEBUG,"read_to_buf returned %d.",read_result);
 
     if (result < 0)
       return -1;
@@ -1052,7 +1087,8 @@ int connection_handle_write(connection_t *conn) {
   if (connection_state_is_connecting(conn)) {
     if (getsockopt(conn->s, SOL_SOCKET, SO_ERROR, (void*)&e, &len) < 0) {
       log_fn(LOG_WARN,"getsockopt() syscall failed?! Please report to tor-ops.");
-      connection_close_immediate(conn);
+      if (CONN_IS_EDGE(conn))
+        connection_edge_end_errno(conn, conn->cpath_layer);
       connection_mark_for_close(conn);
       return -1;
     }
@@ -1061,14 +1097,14 @@ int connection_handle_write(connection_t *conn) {
       if (!ERRNO_IS_CONN_EINPROGRESS(e)) {
         log_fn(LOG_INFO,"in-progress connect failed. Removing.");
         if (CONN_IS_EDGE(conn))
-          connection_edge_end(conn, END_STREAM_REASON_MISC, conn->cpath_layer);
+          connection_edge_end_errno(conn, conn->cpath_layer);
 
         connection_close_immediate(conn);
         connection_mark_for_close(conn);
         /* it's safe to pass OPs to router_mark_as_down(), since it just
          * ignores unrecognized routers
          */
-        if (conn->type == CONN_TYPE_OR)
+        if (conn->type == CONN_TYPE_OR && !get_options()->HttpsProxy)
           router_mark_as_down(conn->identity_digest);
         return -1;
       } else {
@@ -1076,10 +1112,11 @@ int connection_handle_write(connection_t *conn) {
       }
     }
     /* The connection is successful. */
-    return connection_finished_connecting(conn);
+    if (connection_finished_connecting(conn)<0)
+      return -1;
   }
 
-  if (connection_speaks_cells(conn)) {
+  if (connection_speaks_cells(conn) && conn->state > OR_CONN_STATE_PROXY_READING) {
     if (conn->state == OR_CONN_STATE_HANDSHAKING) {
       connection_stop_writing(conn);
       if (connection_tls_continue_handshake(conn) < 0) {
@@ -1124,8 +1161,10 @@ int connection_handle_write(connection_t *conn) {
   } else {
     result = flush_buf(conn->s, conn->outbuf, &conn->outbuf_flushlen);
     if (result < 0) {
+      if (CONN_IS_EDGE(conn))
+        connection_edge_end_errno(conn, conn->cpath_layer);
+
       connection_close_immediate(conn); /* Don't flush; connection is dead. */
-      conn->has_sent_end = 1;
       connection_mark_for_close(conn);
       return -1;
     }
@@ -1158,7 +1197,7 @@ void connection_write_to_buf(const char *string, size_t len, connection_t *conn)
     return;
 
   if (write_to_buf(string, len, conn->outbuf) < 0) {
-    if (conn->type == CONN_TYPE_AP || conn->type == CONN_TYPE_EXIT) {
+    if (CONN_IS_EDGE(conn)) {
       /* if it failed, it means we have our package/delivery windows set
          wrong compared to our max outbuf size. close the whole circuit. */
       log_fn(LOG_WARN,"write_to_buf failed. Closing circuit (fd %d).", conn->s);
@@ -1210,6 +1249,28 @@ connection_t *connection_get_by_identity_digest(const char *digest, int type)
   return best;
 }
 
+/** Return the connection with id <b>id</b> if it is not already
+ * marked for close.
+ */
+connection_t *
+connection_get_by_global_id(uint32_t id) {
+  int i, n;
+  connection_t *conn;
+  connection_t **carray;
+
+  get_connection_array(&carray,&n);
+  for (i=0;i<n;i++) {
+    conn = carray[i];
+    if (conn->global_identifier == id) {
+      if (!conn->marked_for_close)
+        return conn;
+      else
+        return NULL;
+    }
+  }
+  return NULL;
+}
+
 /** Return a connection of type <b>type</b> that is not marked for
  * close.
  */
@@ -1244,6 +1305,23 @@ connection_t *connection_get_by_type_state(int type, int state) {
   return NULL;
 }
 
+/** Return a connection of type <b>type</b> that has purpose <b>purpose</b>,
+ * and that is not marked for close.
+ */
+connection_t *connection_get_by_type_purpose(int type, int purpose) {
+  int i, n;
+  connection_t *conn;
+  connection_t **carray;
+
+  get_connection_array(&carray,&n);
+  for (i=0;i<n;i++) {
+    conn = carray[i];
+    if (conn->type == type && conn->purpose == purpose && !conn->marked_for_close)
+      return conn;
+  }
+  return NULL;
+}
+
 /** Return the connection of type <b>type</b> that is in state
  * <b>state</b>, that was written to least recently, and that is not
  * marked for close.
@@ -1264,9 +1342,11 @@ connection_t *connection_get_by_type_state_lastwritten(int type, int state) {
 }
 
 /** Return a connection of type <b>type</b> that has rendquery equal
- * to <b>rendquery</b>, and that is not marked for close.
+ * to <b>rendquery</b>, and that is not marked for close. If state
+ * is non-zero, conn must be of that state too.
  */
-connection_t *connection_get_by_type_rendquery(int type, const char *rendquery) {
+connection_t *
+connection_get_by_type_state_rendquery(int type, int state, const char *rendquery) {
   int i, n;
   connection_t *conn;
   connection_t **carray;
@@ -1276,6 +1356,7 @@ connection_t *connection_get_by_type_rendquery(int type, const char *rendquery)
     conn = carray[i];
     if (conn->type == type &&
         !conn->marked_for_close &&
+        (!state || state == conn->state) &&
         !rend_cmp_service_ids(rendquery, conn->rend_query))
       return conn;
   }
@@ -1374,6 +1455,9 @@ static int connection_process_inbuf(connection_t *conn, int package_partial) {
       return connection_control_process_inbuf(conn);
     default:
       log_fn(LOG_WARN,"Bug: got unexpected conn type %d.", conn->type);
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
       return -1;
   }
 }
@@ -1406,6 +1490,9 @@ static int connection_finished_flushing(connection_t *conn) {
       return connection_control_finished_flushing(conn);
     default:
       log_fn(LOG_WARN,"Bug: got unexpected conn type %d.", conn->type);
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
       return -1;
   }
 }
@@ -1429,6 +1516,9 @@ static int connection_finished_connecting(connection_t *conn)
       return connection_dir_finished_connecting(conn);
     default:
       log_fn(LOG_WARN,"Bug: got unexpected conn type %d.", conn->type);
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
       return -1;
   }
 }
@@ -1451,6 +1541,9 @@ static int connection_reached_eof(connection_t *conn)
       return connection_control_reached_eof(conn);
     default:
       log_fn(LOG_WARN,"Bug: got unexpected conn type %d.", conn->type);
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
       return -1;
   }
 }
@@ -1507,15 +1600,15 @@ void assert_connection_ok(connection_t *conn, time_t now)
        * gave a bad cert/etc, then we won't have assigned bandwidth,
        * yet it will be open. -RD
        */
-      tor_assert(conn->receiver_bucket >= 0);
+//      tor_assert(conn->receiver_bucket >= 0);
     }
 //    tor_assert(conn->addr && conn->port);
     tor_assert(conn->address);
-    if (conn->state != OR_CONN_STATE_CONNECTING)
+    if (conn->state > OR_CONN_STATE_PROXY_READING)
       tor_assert(conn->tls);
   }
 
-  if (conn->type != CONN_TYPE_EXIT && conn->type != CONN_TYPE_AP) {
+  if (! CONN_IS_EDGE(conn)) {
     tor_assert(!conn->stream_id);
     tor_assert(!conn->next_stream);
     tor_assert(!conn->cpath_layer);
@@ -1585,8 +1678,6 @@ void assert_connection_ok(connection_t *conn, time_t now)
       tor_assert(conn->state <= _CONTROL_CONN_STATE_MAX);
       break;
     default:
-      log_fn(LOG_WARN,"Unexpected conn type %d", conn->type);
       tor_assert(0);
   }
 }
-
