@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: cpuworker.c,v 1.61.2.5 2005/04/23 20:58:44 arma Exp $ */
-const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.61.2.5 2005/04/23 20:58:44 arma Exp $";
+/* $Id: cpuworker.c,v 1.67 2005/02/22 08:18:35 nickm Exp $ */
+const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.67 2005/02/22 08:18:35 nickm Exp $";
 
 /**
  * \file cpuworker.c
@@ -67,13 +67,15 @@ static void tag_pack(char *tag, uint32_t addr, uint16_t port, uint16_t circ_id)
  */
 static void tag_unpack(const char *tag, uint32_t *addr, uint16_t *port, uint16_t *circ_id) {
   struct in_addr in;
+  char addrbuf[INET_NTOA_BUF_LEN];
 
   *addr    = *(const uint32_t *)tag;
   *port    = *(const uint16_t *)(tag+4);
   *circ_id = *(const uint16_t *)(tag+6);
 
   in.s_addr = htonl(*addr);
-  log_fn(LOG_DEBUG,"onion was from %s:%d, circ_id %d.", inet_ntoa(in), *port, *circ_id);
+  tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
+  log_fn(LOG_DEBUG,"onion was from %s:%d, circ_id %d.", addrbuf, *port, *circ_id);
 }
 
 /** Called when the onion key has changed and we need to spawn new
@@ -220,10 +222,16 @@ static int cpuworker_main(void *data) {
   dup_onion_keys(&onion_key, &last_onion_key);
 
   for (;;) {
+    int r;
 
-    if (recv(fd, &question_type, 1, 0) != 1) {
+    if ((r = recv(fd, &question_type, 1, 0)) != 1) {
 //      log_fn(LOG_ERR,"read type failed. Exiting.");
-      log_fn(LOG_INFO,"cpuworker exiting because tor process closed connection (either rotated keys or died).");
+      if (r == 0) {
+        log_fn(LOG_INFO,"CPU worker exiting because Tor process closed connection (either rotated keys or died).");
+      } else {
+        log_fn(LOG_INFO,"CPU worker editing because of error on connection to Tor process.");
+        log_fn(LOG_INFO,"(Error on %d was %s)", fd, tor_socket_strerror(tor_socket_errno(fd)));
+      }
       goto end;
     }
     tor_assert(question_type == CPUWORKER_TASK_ONION);
@@ -347,34 +355,6 @@ static void process_pending_task(connection_t *cpuworker) {
     log_fn(LOG_WARN,"assign_to_cpuworker failed. Ignoring.");
 }
 
-#define CPUWORKER_BUSY_TIMEOUT 3600 /* seconds */
-
-/** We have a bug that I can't find. Sometimes, very rarely, cpuworkers
- * get stuck in the 'busy' state, even though the cpuworker process
- * thinks of itself as idle. I don't know why. But here's a workaround
- * to kill any cpuworker that's been busy for more than 3600 seconds. */
-static void
-cull_wedged_cpuworkers(void) {
-  connection_t **carray;
-  connection_t *conn;
-  int n_conns, i;
-  time_t now = time(NULL);
-
-  get_connection_array(&carray, &n_conns);
-  for (i = 0; i < n_conns; ++i) {
-    conn = carray[i];
-    if (!conn->marked_for_close &&
-        conn->type == CONN_TYPE_CPUWORKER &&
-        conn->state == CPUWORKER_STATE_BUSY_ONION &&
-        conn->timestamp_lastwritten + CPUWORKER_BUSY_TIMEOUT < now) {
-      log_fn(LOG_NOTICE,"Bug: closing wedged cpuworker. Can somebody find the bug?");
-      num_cpuworkers_busy--;
-      num_cpuworkers--;
-      connection_mark_for_close(conn);
-    }
-  }
-}
-
 /** if cpuworker is defined, assert that he's idle, and use him. else,
  * look for an idle cpuworker and use him. if none idle, queue task onto
  * the pending onion list and return.
@@ -388,9 +368,6 @@ int assign_to_cpuworker(connection_t *cpuworker, unsigned char question_type,
 
   tor_assert(question_type == CPUWORKER_TASK_ONION);
 
-  cull_wedged_cpuworkers();
-  spawn_enough_cpuworkers();
-
   if (question_type == CPUWORKER_TASK_ONION) {
     circ = task;
 
