@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.181.2.16 2005/03/19 20:30:05 arma Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.181.2.16 2005/03/19 20:30:05 arma Exp $";
+/* $Id: directory.c,v 1.221 2005/03/22 18:43:24 nickm Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.221 2005/03/22 18:43:24 nickm Exp $";
 
 #include "or.h"
 
@@ -29,18 +29,15 @@ const char directory_c_id[] = "$Id: directory.c,v 1.181.2.16 2005/03/19 20:30:05
  */
 
 static void
-directory_initiate_command_router(routerinfo_t *router, uint8_t purpose,
-                                  const char *resource,
-                                  const char *payload, size_t payload_len);
-static void
 directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
-                                      uint8_t purpose, const char *resource,
+                                      uint8_t purpose, int private_connection,
+                                      const char *resource,
                                       const char *payload, size_t payload_len);
 static void
 directory_initiate_command(const char *address, uint32_t addr, uint16_t port,
                            const char *platform,
                            const char *digest, uint8_t purpose,
-                           const char *resource,
+                           int private_connection, const char *resource,
                            const char *payload, size_t payload_len);
 
 static void
@@ -49,6 +46,7 @@ directory_send_command(connection_t *conn, const char *platform,
                        const char *payload, size_t payload_len);
 static int directory_handle_command(connection_t *conn);
 static int body_is_plausible(const char *body, size_t body_len, int purpose);
+static int purpose_is_private(uint8_t purpose);
 
 /********* START VARIABLES **********/
 
@@ -62,9 +60,6 @@ char rend_publish_string[] = "/tor/rendezvous/publish";
 char rend_fetch_url[] = "/tor/rendezvous/";
 #endif
 
-#define MAX_HEADERS_SIZE 50000
-#define MAX_BODY_SIZE 500000
-
 #define ALLOW_DIRECTORY_TIME_SKEW 30*60
 
 /********* END VARIABLES ************/
@@ -88,6 +83,13 @@ parse_dir_policy(void)
   }
 }
 
+void
+free_dir_policy(void)
+{
+  addr_policy_free(dir_policy);
+  dir_policy = NULL;
+}
+
 /** Return 1 if <b>addr</b> is permitted to connect to our dir port,
  * based on <b>dir_policy</b>. Else return 0.
  */
@@ -98,15 +100,23 @@ int dir_policy_permits_address(uint32_t addr)
   if (!dir_policy) /* 'no dir policy' means 'accept' */
     return 1;
   a = router_compare_addr_to_addr_policy(addr, 1, dir_policy);
-  if (a==-1)
+  if (a==ADDR_POLICY_REJECTED)
     return 0;
-  else if (a==0)
+  else if (a==ADDR_POLICY_ACCEPTED)
     return 1;
-  tor_assert(a==1);
   log_fn(LOG_WARN, "Bug: got unexpected 'maybe' answer from dir policy");
   return 0;
 }
 
+static int
+purpose_is_private(uint8_t purpose) {
+  if (purpose == DIR_PURPOSE_FETCH_DIR ||
+      purpose == DIR_PURPOSE_UPLOAD_DIR ||
+      purpose == DIR_PURPOSE_FETCH_RUNNING_LIST)
+    return 0;
+  return 1;
+}
+
 /** Start a connection to every known directory server, using
  * connection purpose 'purpose' and uploading the payload 'payload'
  * (length 'payload_len').  The purpose should be one of
@@ -117,7 +127,6 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
                              size_t payload_len)
 {
   smartlist_t *dirservers;
-  char buf[16];
 
   router_get_trusted_dir_servers(&dirservers);
   tor_assert(dirservers);
@@ -131,12 +140,11 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
        * descriptor -- those use Tor. */
       if (get_options()->FascistFirewall && purpose == DIR_PURPOSE_UPLOAD_DIR &&
           !get_options()->HttpProxy) {
-        tor_snprintf(buf,sizeof(buf),"%d",ds->dir_port);
-        if (!smartlist_string_isin(get_options()->FirewallPorts, buf))
+        if (!smartlist_string_num_isin(get_options()->FirewallPorts, ds->dir_port))
           continue;
       }
-      directory_initiate_command_trusted_dir(ds, purpose, NULL,
-                                             payload, payload_len);
+      directory_initiate_command_trusted_dir(ds, purpose, purpose_is_private(purpose),
+                                             NULL, payload, payload_len);
     });
 }
 
@@ -157,6 +165,7 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
   int directconn = purpose == DIR_PURPOSE_FETCH_DIR ||
                    purpose == DIR_PURPOSE_FETCH_RUNNING_LIST;
   int fetch_fresh_first = advertised_server_mode();
+  int priv = purpose_is_private(purpose);
 
   if (directconn) {
     if (fetch_fresh_first) {
@@ -184,13 +193,13 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
   }
 
   if (r)
-    directory_initiate_command_router(r, purpose, resource, NULL, 0);
+    directory_initiate_command_router(r, purpose, priv, resource, NULL, 0);
   else if (ds)
-    directory_initiate_command_trusted_dir(ds, purpose, resource, NULL, 0);
+    directory_initiate_command_trusted_dir(ds, purpose, priv, resource, NULL, 0);
   else {
     log_fn(LOG_NOTICE,"No running dirservers known. Not trying. (purpose %d)",
            purpose);
-    if(directconn) {
+    if (directconn) {
       /* remember we tried them all and failed. */
       directory_all_unreachable(time(NULL));
     }
@@ -208,25 +217,28 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
  * When fetching a rendezvous descriptor, <b>resource</b> is the service ID we
  * want to fetch.
  */
-static void
+void
 directory_initiate_command_router(routerinfo_t *router, uint8_t purpose,
-                                  const char *resource,
+                                  int private_connection, const char *resource,
                                   const char *payload, size_t payload_len)
 {
   directory_initiate_command(router->address, router->addr, router->dir_port,
                              router->platform, router->identity_digest,
-                             purpose, resource, payload, payload_len);
+                             purpose, private_connection, resource,
+                             payload, payload_len);
 }
 
 /** As directory_initiate_command_router, but send the command to a trusted
  * directory server <b>dirserv</b>. **/
 static void
 directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
-                                       uint8_t purpose, const char *resource,
+                                       uint8_t purpose, int private_connection,
+                                       const char *resource,
                                        const char *payload, size_t payload_len)
 {
   directory_initiate_command(dirserv->address, dirserv->addr,dirserv->dir_port,
-               NULL, dirserv->digest, purpose, resource, payload, payload_len);
+               NULL, dirserv->digest, purpose, private_connection, resource,
+               payload, payload_len);
 }
 
 /** Called when we are unable to complete our connection to a
@@ -245,7 +257,7 @@ connection_dir_connect_failed(connection_t *conn)
   }
 }
 
-/** Helper for directory_initiate_command(router|trusted_dir): send the
+/** Helper for directory_initiate_command_(router|trusted_dir): send the
  * command to a server whose address is <b>address</b>, whose IP is
  * <b>addr</b>, whose directory port is <b>dir_port</b>, whose tor version is
  * <b>platform</b>, and whose identity key digest is <b>digest</b>. The
@@ -254,7 +266,7 @@ static void
 directory_initiate_command(const char *address, uint32_t addr,
                            uint16_t dir_port, const char *platform,
                            const char *digest, uint8_t purpose,
-                           const char *resource,
+                           int private_connection, const char *resource,
                            const char *payload, size_t payload_len)
 {
   connection_t *conn;
@@ -290,17 +302,7 @@ directory_initiate_command(const char *address, uint32_t addr,
   /* set up conn so it's got all the data we need to remember */
   conn->addr = addr;
   conn->port = dir_port;
-
-  if (get_options()->HttpProxy) {
-    addr = get_options()->HttpProxyAddr;
-    dir_port = get_options()->HttpProxyPort;
-  }
-
   conn->address = tor_strdup(address);
-  /* conn->nickname = tor_strdup(router->nickname); */
-  /* tor_assert(router->identity_pkey); */
-  /* conn->identity_pkey = crypto_pk_dup_key(router->identity_pkey); */
-  /* crypto_pk_get_digest(conn->identity_pkey, conn->identity_digest); */
   memcpy(conn->identity_digest, digest, DIGEST_LEN);
 
   conn->purpose = purpose;
@@ -308,10 +310,14 @@ directory_initiate_command(const char *address, uint32_t addr,
   /* give it an initial state */
   conn->state = DIR_CONN_STATE_CONNECTING;
 
-  if (purpose == DIR_PURPOSE_FETCH_DIR ||
-      purpose == DIR_PURPOSE_UPLOAD_DIR ||
-      purpose == DIR_PURPOSE_FETCH_RUNNING_LIST) {
+  if (!private_connection) {
     /* then we want to connect directly */
+
+    if (get_options()->HttpProxy) {
+      addr = get_options()->HttpProxyAddr;
+      dir_port = get_options()->HttpProxyPort;
+    }
+
     switch (connection_connect(conn, conn->address, addr, dir_port)) {
       case -1:
         connection_dir_connect_failed(conn);
@@ -324,7 +330,7 @@ directory_initiate_command(const char *address, uint32_t addr,
         /* queue the command on the outbuf */
         directory_send_command(conn, platform, purpose, resource,
                                payload, payload_len);
-        connection_watch_events(conn, POLLIN | POLLOUT | POLLERR);
+        connection_watch_events(conn, EV_READ | EV_WRITE);
         /* writable indicates finish, readable indicates broken link,
            error indicates broken link in windowsland. */
     }
@@ -345,7 +351,7 @@ directory_initiate_command(const char *address, uint32_t addr,
     /* queue the command on the outbuf */
     directory_send_command(conn, platform, purpose, resource,
                            payload, payload_len);
-    connection_watch_events(conn, POLLIN | POLLOUT | POLLERR);
+    connection_watch_events(conn, EV_READ | EV_WRITE);
   }
 }
 
@@ -484,14 +490,21 @@ parse_http_url(char *headers, char **url)
 
 /** Parse an HTTP response string <b>headers</b> of the form
  * "HTTP/1.\%d \%d\%s\r\n...".
- * If it's well-formed, assign *<b>code</b>, point and return 0.
- * If <b>date</b> is provided, set *date to the Date header in the
- * http headers, or 0 if no such header is found.
- * Otherwise, return -1.
+ *
+ * If it's well-formed, assign the status code to *<b>code</b> and
+ * return 0.  Otherwise, return -1.
+ *
+ * On success: If <b>date</b> is provided, set *date to the Date
+ * header in the http headers, or 0 if no such header is found.  If
+ * <b>compression</b> is provided, set *<b>compression</b> to the
+ * compression method given in the Content-Encoding header, or 0 if no
+ * such header is found, or -1 if the value of the header is not
+ * recognized.  If <b>reason</b> is provided, strdup the reason string
+ * into it.
  */
-static int
+int
 parse_http_response(const char *headers, int *code, time_t *date,
-                    int *compression)
+                    int *compression, char **reason)
 {
   int n1, n2;
   char datestr[RFC1123_TIME_LEN+1];
@@ -512,13 +525,27 @@ parse_http_response(const char *headers, int *code, time_t *date,
   parsed_headers = smartlist_create();
   smartlist_split_string(parsed_headers, headers, "\n",
                          SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, -1);
+  if (reason) {
+    smartlist_t *status_line_elements = smartlist_create();
+    tor_assert(smartlist_len(parsed_headers));
+    smartlist_split_string(status_line_elements,
+                           smartlist_get(parsed_headers, 0),
+                           " ", SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 3);
+    tor_assert(smartlist_len(status_line_elements) <= 3);
+    if (smartlist_len(status_line_elements) == 3) {
+      *reason = smartlist_get(status_line_elements, 2);
+      smartlist_set(status_line_elements, 2, NULL); /* Prevent free */
+    }
+    SMARTLIST_FOREACH(status_line_elements, char *, cp, tor_free(cp));
+    smartlist_free(status_line_elements);
+  }
   if (date) {
     *date = 0;
     SMARTLIST_FOREACH(parsed_headers, const char *, s,
       if (!strcmpstart(s, "Date: ")) {
         strlcpy(datestr, s+6, sizeof(datestr));
         /* This will do nothing on failure, so we don't need to check
-           the result.  We shouldn't warn, since there are many other valid
+           the result.   We shouldn't warn, since there are many other valid
            date formats besides the one we use. */
         parse_rfc1123_time(datestr, date);
         break;
@@ -585,6 +612,7 @@ connection_dir_client_reached_eof(connection_t *conn)
 {
   char *body;
   char *headers;
+  char *reason = NULL;
   size_t body_len=0;
   int status_code;
   time_t now, date_header=0;
@@ -597,7 +625,7 @@ connection_dir_client_reached_eof(connection_t *conn)
                               &headers, MAX_HEADERS_SIZE,
                               &body, &body_len, MAX_DIR_SIZE)) {
     case -1: /* overflow */
-      log_fn(LOG_WARN,"'fetch' response too large. Failing.");
+      log_fn(LOG_WARN,"'fetch' response too large (server '%s'). Failing.", conn->address);
       return -1;
     case 0:
       log_fn(LOG_INFO,"'fetch' response not all here, but we're at eof. Closing.");
@@ -606,16 +634,25 @@ connection_dir_client_reached_eof(connection_t *conn)
   }
 
   if (parse_http_response(headers, &status_code, &date_header,
-                          &compression) < 0) {
-    log_fn(LOG_WARN,"Unparseable headers. Closing.");
+                          &compression, &reason) < 0) {
+    log_fn(LOG_WARN,"Unparseable headers (server '%s'). Closing.", conn->address);
     tor_free(body); tor_free(headers);
     return -1;
   }
+  if (!reason) reason = tor_strdup("[no reason given]");
+
+  log_fn(LOG_DEBUG,
+         "Received response from directory servers '%s': %d \"%s\"",
+         conn->address, status_code, reason);
+
   if (date_header > 0) {
     now = time(NULL);
     delta = now-date_header;
     if (abs(delta)>ALLOW_DIRECTORY_TIME_SKEW) {
-      log_fn(LOG_WARN, "Received directory with skewed time: we are %d minutes %s, or the directory is %d minutes %s.",
+      routerinfo_t *router = router_get_by_digest(conn->identity_digest);
+      log_fn((router && router->is_verified) ? LOG_WARN : LOG_INFO,
+             "Received directory with skewed time (server '%s'): we are %d minutes %s, or the directory is %d minutes %s.",
+             conn->address,
              abs(delta)/60, delta>0 ? "ahead" : "behind",
              abs(delta)/60, delta>0 ? "behind" : "ahead");
       skewed = 1; /* don't check the recommended-versions line */
@@ -665,7 +702,7 @@ connection_dir_client_reached_eof(connection_t *conn)
      * we didn't manage to uncompress it, then warn and bail. */
     if (!plausible && !new_body) {
       log_fn(LOG_WARN, "Unable to decompress HTTP body (server '%s').", conn->address);
-      tor_free(body); tor_free(headers);
+      tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (new_body) {
@@ -677,16 +714,18 @@ connection_dir_client_reached_eof(connection_t *conn)
 
   if (conn->purpose == DIR_PURPOSE_FETCH_DIR) {
     /* fetch/process the directory to learn about new routers. */
-    log_fn(LOG_INFO,"Received directory (size %d)", (int)body_len);
+    log_fn(LOG_INFO,"Received directory (size %d) from server '%s'",
+           (int)body_len, conn->address);
     if (status_code == 503 || body_len == 0) {
-      log_fn(LOG_INFO,"Empty directory. Ignoring.");
-      tor_free(body); tor_free(headers);
+      log_fn(LOG_INFO,"Empty directory; status %d (\"%s\") Ignoring.",
+             status_code, reason);
+      tor_free(body); tor_free(headers); tor_free(reason);
       return 0;
     }
     if (status_code != 200) {
-      log_fn(LOG_WARN,"Received http status code %d from dirserver. Failing.",
-             status_code);
-      tor_free(body); tor_free(headers);
+      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s'. Failing.",
+             status_code, reason, conn->address);
+      tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (router_load_routerlist_from_directory(body, NULL, !skewed, 0) < 0) {
@@ -694,7 +733,8 @@ connection_dir_client_reached_eof(connection_t *conn)
     } else {
       log_fn(LOG_INFO,"updated routers.");
     }
-    directory_has_arrived(time(NULL)); /* do things we've been waiting to do */
+    /* do things we've been waiting to do */
+    directory_has_arrived(time(NULL), conn->identity_digest);
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RUNNING_LIST) {
@@ -703,20 +743,22 @@ connection_dir_client_reached_eof(connection_t *conn)
     /* just update our list of running routers, if this list is new info */
     log_fn(LOG_INFO,"Received running-routers list (size %d)", (int)body_len);
     if (status_code != 200) {
-      log_fn(LOG_WARN,"Received http status code %d from dirserver. Failing.",
-             status_code);
-      tor_free(body); tor_free(headers);
+      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s'. Failing.",
+             status_code, reason, conn->address);
+      tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (!(rrs = router_parse_runningrouters(body, 1))) {
-      log_fn(LOG_WARN, "Can't parse runningrouters list");
-      tor_free(body); tor_free(headers);
+      log_fn(LOG_WARN, "Can't parse runningrouters list (server '%s')", conn->address);
+      tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     router_get_routerlist(&rl);
-    if (rl)
-      routerlist_update_from_runningrouters(rl,rrs);
-    running_routers_free(rrs);
+    if (rl) {
+      routerlist_set_runningrouters(rl,rrs);
+    } else {
+      running_routers_free(rrs);
+    }
   }
 
   if (conn->purpose == DIR_PURPOSE_UPLOAD_DIR) {
@@ -725,20 +767,20 @@ connection_dir_client_reached_eof(connection_t *conn)
         log_fn(LOG_INFO,"eof (status 200) after uploading server descriptor: finished.");
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (bad request) response from dirserver. Malformed server descriptor?");
+        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s'. Malformed server descriptor?", reason, conn->address);
         break;
       case 403:
-        log_fn(LOG_WARN,"http status 403 (unapproved server) response from dirserver '%s'. Is your clock skewed? Have you mailed us your identity fingerprint? Are you using the right key? Are you using a private IP address? See http://tor.eff.org/doc/tor-doc.html#server.", conn->address);
+        log_fn(LOG_WARN,"http status 403 (\"%s\") response from dirserver '%s'. Is your clock skewed? Have you mailed us your key fingerprint? Are you using the right key? Are you using a private IP address? See http://tor.eff.org/doc/tor-doc.html#server.", reason, conn->address);
         break;
       default:
-        log_fn(LOG_WARN,"http status %d response unrecognized.", status_code);
+        log_fn(LOG_WARN,"http status %d (\"%s\") reason unexpected (server '%s').", status_code, reason, conn->address);
         break;
     }
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RENDDESC) {
-    log_fn(LOG_INFO,"Received rendezvous descriptor (size %d, status code %d)",
-           (int)body_len, status_code);
+    log_fn(LOG_INFO,"Received rendezvous descriptor (size %d, status %d (\"%s\"))",
+           (int)body_len, status_code, reason);
     switch (status_code) {
       case 200:
         if (rend_cache_store(body, body_len) < 0) {
@@ -747,8 +789,8 @@ connection_dir_client_reached_eof(connection_t *conn)
            * cleans it up */
         } else {
           /* success. notify pending connections about this. */
-          rend_client_desc_fetched(conn->rend_query, 1);
           conn->purpose = DIR_PURPOSE_HAS_FETCHED_RENDDESC;
+          rend_client_desc_here(conn->rend_query);
         }
         break;
       case 404:
@@ -756,7 +798,10 @@ connection_dir_client_reached_eof(connection_t *conn)
          * connection_mark_for_close cleans it up. */
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (bad request). Dirserver didn't like our rendezvous query?");
+        log_fn(LOG_WARN,"http status 400 (\"%s\"). Dirserver didn't like our rendezvous query?", reason);
+        break;
+      default:
+        log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s').", status_code, reason, conn->address);
         break;
     }
   }
@@ -764,17 +809,17 @@ connection_dir_client_reached_eof(connection_t *conn)
   if (conn->purpose == DIR_PURPOSE_UPLOAD_RENDDESC) {
     switch (status_code) {
       case 200:
-        log_fn(LOG_INFO,"eof (status 200) after uploading rendezvous descriptor: finished.");
+        log_fn(LOG_INFO,"Uploading rendezvous descriptor: finished with status 200 (\"%s\")", reason);
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (bad request) response from dirserver. Malformed rendezvous descriptor?");
+        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver. Malformed rendezvous descriptor?", reason);
         break;
       default:
-        log_fn(LOG_WARN,"http status %d response unrecognized.", status_code);
+        log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s').", status_code, reason, conn->address);
         break;
     }
   }
-  tor_free(body); tor_free(headers);
+  tor_free(body); tor_free(headers); tor_free(reason);
   return 0;
 }
 
@@ -821,11 +866,21 @@ int connection_dir_process_inbuf(connection_t *conn) {
   return 0;
 }
 
-static char answer200[] = "HTTP/1.0 200 OK\r\n\r\n";
-static char answer400[] = "HTTP/1.0 400 Bad request\r\n\r\n";
-static char answer403[] = "HTTP/1.0 403 Unapproved server\r\n\r\n";
-static char answer404[] = "HTTP/1.0 404 Not found\r\n\r\n";
-static char answer503[] = "HTTP/1.0 503 Directory unavailable\r\n\r\n";
+/** Create an http response for the client <b>conn</b> out of
+ * <b>status</b> and <b>reason_phrase</b>. Write it to <b>conn</b>.
+ */
+static void
+write_http_status_line(connection_t *conn, int status,
+                       const char *reason_phrase)
+{
+  char buf[256];
+  if (tor_snprintf(buf, sizeof(buf), "HTTP/1.0 %d %s\r\n\r\n",
+      status, reason_phrase) < 0) {
+    log_fn(LOG_WARN,"Bug: status line too long.");
+    return;
+  }
+  connection_write_to_buf(buf, strlen(buf), conn);
+}
 
 /** Helper function: called when a dirserver gets a complete HTTP GET
  * request.  Look for a request for a directory or for a rendezvous
@@ -847,7 +902,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
   conn->state = DIR_CONN_STATE_SERVER_WRITING;
 
   if (parse_http_url(headers, &url) < 0) {
-    connection_write_to_buf(answer400, strlen(answer400), conn);
+    write_http_status_line(conn, 400, "Bad request");
     return 0;
   }
   log_fn(LOG_INFO,"rewritten url as '%s'.", url);
@@ -860,7 +915,10 @@ directory_handle_command_get(connection_t *conn, char *headers,
 
     if (dlen == 0) {
       log_fn(LOG_NOTICE,"Client asked for the mirrored directory, but we don't have a good one yet. Sending 503 Dir not available.");
-      connection_write_to_buf(answer503, strlen(answer503), conn);
+      write_http_status_line(conn, 503, "Directory unavailable");
+      /* try to get a new one now */
+      if (!connection_get_by_type_purpose(CONN_TYPE_DIR, DIR_PURPOSE_FETCH_DIR))
+        directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
       return 0;
     }
 
@@ -882,7 +940,11 @@ directory_handle_command_get(connection_t *conn, char *headers,
     tor_free(url);
     dlen = dirserv_get_runningrouters(&cp, deflated);
     if (!dlen) { /* we failed to create/cache cp */
-      connection_write_to_buf(answer503, strlen(answer503), conn);
+      write_http_status_line(conn, 503, "Directory unavailable");
+      /* try to get a new one now */
+      if (!connection_get_by_type_purpose(CONN_TYPE_DIR,
+                                          DIR_PURPOSE_FETCH_RUNNING_LIST))
+        directory_get_from_dirserver(DIR_PURPOSE_FETCH_RUNNING_LIST, NULL, 1);
       return 0;
     }
 
@@ -907,7 +969,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
        * if we're gone to the site recently, and 404 if we haven't.
        *
        * Reject. */
-      connection_write_to_buf(answer400, strlen(answer400), conn);
+      write_http_status_line(conn, 400, "Nonauthoritative directory does not not store rendezvous descriptors.");
       tor_free(url);
       return 0;
     }
@@ -921,10 +983,10 @@ directory_handle_command_get(connection_t *conn, char *headers,
         connection_write_to_buf(descp, desc_len, conn);
         break;
       case 0: /* well-formed but not present */
-        connection_write_to_buf(answer404, strlen(answer404), conn);
+        write_http_status_line(conn, 404, "Not found");
         break;
       case -1: /* not well-formed */
-        connection_write_to_buf(answer400, strlen(answer400), conn);
+        write_http_status_line(conn, 400, "Bad request");
         break;
     }
     tor_free(url);
@@ -932,7 +994,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
   }
 
   /* we didn't recognize the url */
-  connection_write_to_buf(answer404, strlen(answer404), conn);
+  write_http_status_line(conn, 404, "Not found");
   tor_free(url);
   return 0;
 }
@@ -956,30 +1018,32 @@ directory_handle_command_post(connection_t *conn, char *headers,
   if (!authdir_mode(get_options())) {
     /* we just provide cached directories; we don't want to
      * receive anything. */
-    connection_write_to_buf(answer400, strlen(answer400), conn);
+    write_http_status_line(conn, 400, "Nonauthoritative directory does not not store server descriptors.");
     return 0;
   }
 
   if (parse_http_url(headers, &url) < 0) {
-    connection_write_to_buf(answer400, strlen(answer400), conn);
+    write_http_status_line(conn, 400, "Bad request");
     return 0;
   }
   log_fn(LOG_INFO,"rewritten url as '%s'.", url);
 
   if (!strcmp(url,"/tor/")) { /* server descriptor post */
+    const char *msg;
     cp = body;
-    switch (dirserv_add_descriptor(&cp)) {
+    switch (dirserv_add_descriptor(&cp, &msg)) {
+      case -2:
       case -1:
         /* malformed descriptor, or something wrong */
-        connection_write_to_buf(answer400, strlen(answer400), conn);
+        write_http_status_line(conn, 400, msg?msg:"Malformed or unacceptable server descriptor");
         break;
       case 0:
         /* descriptor was well-formed but server has not been approved */
-        connection_write_to_buf(answer403, strlen(answer403), conn);
+        write_http_status_line(conn, 200, msg?msg:"Unverified server descriptor accepted");
         break;
       case 1:
         dirserv_get_directory(&cp, 0); /* rebuild and write to disk */
-        connection_write_to_buf(answer200, strlen(answer200), conn);
+        write_http_status_line(conn, 200, msg?msg:"Verified server descriptor accepted");
         break;
     }
     tor_free(url);
@@ -989,15 +1053,15 @@ directory_handle_command_post(connection_t *conn, char *headers,
   if (!strcmpstart(url,"/tor/rendezvous/publish")) {
     /* rendezvous descriptor post */
     if (rend_cache_store(body, body_len) < 0)
-      connection_write_to_buf(answer400, strlen(answer400), conn);
+      write_http_status_line(conn, 400, "Invalid service descriptor rejected");
     else
-      connection_write_to_buf(answer200, strlen(answer200), conn);
+      write_http_status_line(conn, 200, "Service descriptor stored");
     tor_free(url);
     return 0;
   }
 
   /* we didn't recognize the url */
-  connection_write_to_buf(answer404, strlen(answer404), conn);
+  write_http_status_line(conn, 404, "Not found");
   tor_free(url);
   return 0;
 }
@@ -1019,7 +1083,7 @@ static int directory_handle_command(connection_t *conn) {
                               &headers, MAX_HEADERS_SIZE,
                               &body, &body_len, MAX_BODY_SIZE)) {
     case -1: /* overflow */
-      log_fn(LOG_WARN,"Invalid input. Closing.");
+      log_fn(LOG_WARN,"Invalid input from address '%s'. Closing.", conn->address);
       return -1;
     case 0:
       log_fn(LOG_DEBUG,"command not all here yet.");
@@ -1063,6 +1127,9 @@ int connection_dir_finished_flushing(connection_t *conn) {
       return 0;
     default:
       log_fn(LOG_WARN,"Bug: called in unexpected state %d.", conn->state);
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
       return -1;
   }
   return 0;
