@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.129.2.5 2005/03/23 06:43:02 nickm Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.129.2.5 2005/03/23 06:43:02 nickm Exp $";
+/* $Id: dirserv.c,v 1.148 2005/03/23 19:15:10 nickm Exp $ */
+const char dirserv_c_id[] = "$Id: dirserv.c,v 1.148 2005/03/23 19:15:10 nickm Exp $";
 
 #include "or.h"
 
@@ -20,8 +20,6 @@ const char dirserv_c_id[] = "$Id: dirserv.c,v 1.129.2.5 2005/03/23 06:43:02 nick
 static int the_directory_is_dirty = 1;
 static int runningrouters_is_dirty = 1;
 
-static int list_server_status(char **running_routers_out,
-                              char **router_status_out);
 static void directory_remove_unrecognized(void);
 static int dirserv_regenerate_directory(void);
 /* Should be static; exposed for testing */
@@ -249,29 +247,12 @@ dirserv_free_fingerprint_list()
  *    Descriptor list
  */
 
-/** A directory server's view of a server descriptor.  Contains both
- * parsed and unparsed versions. */
-typedef struct descriptor_entry_t {
-  char *nickname;
-  time_t published;
-  size_t desc_len;
-  char *descriptor;
-  int verified;
-  routerinfo_t *router;
-} descriptor_entry_t;
-
-/** List of all server descriptors that this dirserv is holding. */
+/** List of routerinfo_t for all server descriptors that this dirserv
+ * is holding.
+ * XXXX This should eventually get coalesced into routerlist.c
+ */
 static smartlist_t *descriptor_list = NULL;
 
-/** Release the storage held by <b>desc</b> */
-static void free_descriptor_entry(descriptor_entry_t *desc)
-{
-  tor_free(desc->descriptor);
-  tor_free(desc->nickname);
-  routerinfo_free(desc->router);
-  tor_free(desc);
-}
-
 /** Release all storage that the dirserv is holding for server
  * descriptors. */
 void
@@ -279,39 +260,62 @@ dirserv_free_descriptors()
 {
   if (!descriptor_list)
     return;
-  SMARTLIST_FOREACH(descriptor_list, descriptor_entry_t *, d,
-                    free_descriptor_entry(d));
+  SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri,
+                    routerinfo_free(ri));
   smartlist_clear(descriptor_list);
 }
 
+/** Return -1 if <b>ri</b> has a private or otherwise bad address,
+ * unless we're configured to not care. Return 0 if all ok. */
+static int
+dirserv_router_has_valid_address(routerinfo_t *ri)
+{
+  struct in_addr iaddr;
+  if (get_options()->DirAllowPrivateAddresses)
+    return 0; /* whatever it is, we're fine with it */
+  if (!tor_inet_aton(ri->address, &iaddr)) {
+    log_fn(LOG_INFO,"Router '%s' published non-IP address '%s'. Refusing.",
+           ri->nickname, ri->address);
+    return -1;
+  }
+  if (is_internal_IP(ntohl(iaddr.s_addr))) {
+    log_fn(LOG_INFO,"Router '%s' published internal IP address '%s'. Refusing.",
+           ri->nickname, ri->address);
+    return -1; /* it's a private IP, we should reject it */
+  }
+  return 0;
+}
+
 /** Parse the server descriptor at *desc and maybe insert it into the
  * list of server descriptors, and (if the descriptor is well-formed)
- * advance *desc immediately past the descriptor's end.
+ * advance *desc immediately past the descriptor's end.  Set msg to a
+ * message that should be passed back to the origin of this descriptor, or
+ * to NULL.
  *
  * Return 1 if descriptor is well-formed and accepted;
- * 0 if well-formed and server is unapproved;
- * -1 if not well-formed or other error.
+ * 0 if well-formed and server is unapproved but accepted;
+ * -1 if well-formed but rejected;
+ * -2 if not well-formed.
  */
 int
-dirserv_add_descriptor(const char **desc)
+dirserv_add_descriptor(const char **desc, const char **msg)
 {
-  descriptor_entry_t *ent = NULL;
-  routerinfo_t *ri = NULL;
+  routerinfo_t *ri = NULL, *ri_old=NULL;
   int i, r, found=-1;
   char *start, *end;
   char *desc_tmp = NULL;
-  const char *cp;
   size_t desc_len;
   time_t now;
   int verified=1; /* whether we knew its fingerprint already */
-
+  tor_assert(msg);
+  *msg = NULL;
   if (!descriptor_list)
     descriptor_list = smartlist_create();
 
   start = strstr(*desc, "router ");
   if (!start) {
     log_fn(LOG_WARN, "no 'router' line found. This is not a descriptor.");
-    return -1;
+    return -2;
   }
   if ((end = strstr(start+6, "\nrouter "))) {
     ++end; /* Include NL. */
@@ -321,26 +325,28 @@ dirserv_add_descriptor(const char **desc)
     end = start+strlen(start);
   }
   desc_len = end-start;
-  cp = desc_tmp = tor_strndup(start, desc_len);
+  desc_tmp = tor_strndup(start, desc_len); /* Is this strndup still needed???*/
 
   /* Check: is the descriptor syntactically valid? */
-  ri = router_parse_entry_from_string(cp, NULL);
+  ri = router_parse_entry_from_string(desc_tmp, NULL);
   tor_free(desc_tmp);
   if (!ri) {
     log(LOG_WARN, "Couldn't parse descriptor");
+    *msg = "Rejected: Couldn't parse server descriptor.";
     return -1;
   }
   /* Okay.  Now check whether the fingerprint is recognized. */
   r = dirserv_router_fingerprint_is_known(ri);
   if (r==-1) {
     log_fn(LOG_WARN, "Known nickname '%s', wrong fingerprint. Not adding.", ri->nickname);
+    *msg = "Rejected: There is already a verified server with this nickname and a different fingerprint.";
     routerinfo_free(ri);
     *desc = end;
-    return 0;
+    return -1;
   }
   if (r==0) {
     char fp[FINGERPRINT_LEN+1];
-    log_fn(LOG_INFO, "Unknown nickname '%s' (%s:%d). Adding.",
+    log_fn(LOG_INFO, "Unknown nickname '%s' (%s:%d). Will try to add.",
            ri->nickname, ri->address, ri->or_port);
     if (crypto_pk_get_fingerprint(ri->identity_pkey, fp, 1) < 0) {
       log_fn(LOG_WARN, "Error computing fingerprint for '%s'", ri->nickname);
@@ -353,57 +359,63 @@ dirserv_add_descriptor(const char **desc)
   now = time(NULL);
   if (ri->published_on > now+ROUTER_ALLOW_SKEW) {
     log_fn(LOG_NOTICE, "Publication time for nickname '%s' is too far in the future; possible clock skew. Not adding.", ri->nickname);
+    *msg = "Rejected: Your clock is set too far in the future, or your timezone is not correct.";
     routerinfo_free(ri);
     *desc = end;
-    return 0;
+    return -1;
   }
   if (ri->published_on < now-ROUTER_MAX_AGE) {
     log_fn(LOG_NOTICE, "Publication time for router with nickname '%s' is too far in the past. Not adding.", ri->nickname);
+    *msg = "Rejected: Server is expired, or your clock is too far in the past, or your timezone is not correct.";
     routerinfo_free(ri);
     *desc = end;
-    return 0;
+    return -1;
+  }
+  if (dirserv_router_has_valid_address(ri) < 0) {
+    log_fn(LOG_NOTICE, "Router with nickname '%s' has invalid address '%s'. Not adding.", ri->nickname, ri->address);
+    *msg = "Rejected: Address is not an IP, or IP is a private address.";
+    routerinfo_free(ri);
+    *desc = end;
+    return -1;
   }
 
   /* Do we already have an entry for this router? */
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
-    ent = smartlist_get(descriptor_list, i);
-    if (!strcasecmp(ri->nickname, ent->nickname)) {
+    ri_old = smartlist_get(descriptor_list, i);
+    if (!strcasecmp(ri->nickname, ri_old->nickname)) {
       found = i;
       break;
     }
   }
   if (found >= 0) {
     /* if so, decide whether to update it. */
-    if (ent->published >= ri->published_on) {
+    if (ri_old->published_on >= ri->published_on) {
       /* We already have a newer or equal-time descriptor */
       log_fn(LOG_INFO,"We already have a new enough desc for nickname '%s'. Not adding.",ri->nickname);
+      *msg = "We already have a newer descriptor.";
       /* This isn't really an error; return success. */
       routerinfo_free(ri);
       *desc = end;
-      return 1;
+      return verified;
     }
-    /* We don't have a newer one; we'll update this one. */
+    /* We don't alrady have a newer one; we'll update this one. */
     log_fn(LOG_INFO,"Dirserv updating desc for nickname '%s'",ri->nickname);
-    free_descriptor_entry(ent);
+    *msg = verified?"Verified server updated":"Unverified server updated. (Have you sent us your key fingerprint?)";
+    routerinfo_free(ri_old);
     smartlist_del_keeporder(descriptor_list, found);
   } else {
     /* Add at the end. */
     log_fn(LOG_INFO,"Dirserv adding desc for nickname '%s'",ri->nickname);
+    *msg = verified?"Verified server added":"Unverified server added. (Have you sent us your key fingerprint?)";
   }
 
-  ent = tor_malloc(sizeof(descriptor_entry_t));
-  ent->nickname = tor_strdup(ri->nickname);
-  ent->published = ri->published_on;
-  ent->desc_len = desc_len;
-  ent->descriptor = tor_strndup(start,desc_len);
-  ent->router = ri;
-  ent->verified = verified;
-  smartlist_add(descriptor_list, ent);
+  ri->is_verified = verified;
+  smartlist_add(descriptor_list, ri);
 
   *desc = end;
   directory_set_dirty();
 
-  return 1;
+  return verified;
 }
 
 /** Remove all descriptors whose nicknames or fingerprints we don't
@@ -414,16 +426,16 @@ static void
 directory_remove_unrecognized(void)
 {
   int i;
-  descriptor_entry_t *ent;
+  routerinfo_t *ent;
   if (!descriptor_list)
     descriptor_list = smartlist_create();
 
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
     ent = smartlist_get(descriptor_list, i);
-    if (dirserv_router_fingerprint_is_known(ent->router)<=0) {
+    if (dirserv_router_fingerprint_is_known(ent)<=0) {
       log(LOG_INFO, "Router '%s' is no longer recognized",
           ent->nickname);
-      free_descriptor_entry(ent);
+      routerinfo_free(ent);
       smartlist_del(descriptor_list, i--);
     }
   }
@@ -450,12 +462,13 @@ directory_set_dirty()
 int
 dirserv_load_from_directory_string(const char *dir)
 {
-  const char *cp = dir;
+  const char *cp = dir, *m;
   while (1) {
     cp = strstr(cp, "\nrouter ");
     if (!cp) break;
     ++cp;
-    if (dirserv_add_descriptor(&cp) < 0) {
+    if (dirserv_add_descriptor(&cp,&m) < -1) {
+      /* only fail if parsing failed; keep going if simply rejected */
       return -1;
     }
     --cp; /*Back up to newline.*/
@@ -470,39 +483,40 @@ dirserv_load_from_directory_string(const char *dir)
  * as running iff <b>is_live</b> is true.
  */
 static char *
-list_single_server_status(descriptor_entry_t *desc, int is_live,
+list_single_server_status(routerinfo_t *desc, int is_live,
                           int rr_format)
 {
   char buf[MAX_NICKNAME_LEN+HEX_DIGEST_LEN+4]; /* !nickname=$hexdigest\0 */
   char *cp;
 
   tor_assert(desc);
-  tor_assert(desc->router);
 
   cp = buf;
   if (!is_live) {
     *cp++ = '!';
   }
-  if (desc->verified) {
+  if (desc->is_verified) {
     strlcpy(cp, desc->nickname, sizeof(buf)-(cp-buf));
     cp += strlen(cp);
     if (!rr_format)
       *cp++ = '=';
   }
-  if (!desc->verified || !rr_format) {
+  if (!desc->is_verified || !rr_format) {
     *cp++ = '$';
-    base16_encode(cp, HEX_DIGEST_LEN+1, desc->router->identity_digest,
+    base16_encode(cp, HEX_DIGEST_LEN+1, desc->identity_digest,
                   DIGEST_LEN);
   }
   return tor_strdup(buf);
 }
 
-/** Allocate the contents of a running-routers line and a router-status line,
- * and store them in *<b>running_routers_out</b> and *<b>router_status_out</b>
- * respectively.  Return 0 on success, -1 on failure.
+/** Based on the routerinfo_ts in <b>routers</b>, allocate the
+ * contents of a running-routers line and a router-status line, and
+ * store them in *<b>running_routers_out</b> and
+ * *<b>router_status_out</b> respectively.  If either is NULL, skip
+ * it.  Return 0 on success, -1 on failure.
  */
-static int
-list_server_status(char **running_routers_out, char **router_status_out)
+int
+list_server_status(smartlist_t *routers, char **running_routers_out, char **router_status_out)
 {
   /* List of entries in running-routers style: An optional !, then either
    * a nickname or a dollar-prefixed hexdigest. */
@@ -510,26 +524,34 @@ list_server_status(char **running_routers_out, char **router_status_out)
   /* List of entries in a router-status style: An optional !, then an optional
    * equals-suffixed nickname, then a dollar-prefixed hexdigest. */
   smartlist_t *rs_entries;
-
+  /* XXXX Really, we should merge descriptor_list into routerlist.  But
+   * this is potentially tricky, since the semantics of the two lists
+   * are not quite the same.  In any case, it's not for the 0.1.0.x
+   * series.
+   */
+  int authdir_mode = get_options()->AuthoritativeDir;
   tor_assert(running_routers_out || router_status_out);
 
   rr_entries = smartlist_create();
   rs_entries = smartlist_create();
 
-  SMARTLIST_FOREACH(descriptor_list, descriptor_entry_t *, d,
+  SMARTLIST_FOREACH(routers, routerinfo_t *, ri,
   {
     int is_live;
     connection_t *conn;
-    tor_assert(d->router);
     conn = connection_get_by_identity_digest(
-                    d->router->identity_digest, CONN_TYPE_OR);
-    /* Treat a router as alive if
-     *    - It's me, and I'm not hibernating.
-     * or - we're connected to it. */
-    is_live = (router_is_me(d->router) && !we_are_hibernating()) ||
-      (conn && conn->state == OR_CONN_STATE_OPEN);
-    smartlist_add(rr_entries, list_single_server_status(d, is_live, 1));
-    smartlist_add(rs_entries, list_single_server_status(d, is_live, 0));
+                    ri->identity_digest, CONN_TYPE_OR);
+    if (authdir_mode) {
+      /* Treat a router as alive if
+       *    - It's me, and I'm not hibernating.
+       * or - we're connected to it. */
+      is_live = (router_is_me(ri) && !we_are_hibernating()) ||
+        (conn && conn->state == OR_CONN_STATE_OPEN);
+    } else {
+      is_live = ri->is_running;
+    }
+    smartlist_add(rr_entries, list_single_server_status(ri, is_live, 1));
+    smartlist_add(rs_entries, list_single_server_status(ri, is_live, 0));
   });
 
   if (running_routers_out)
@@ -553,16 +575,16 @@ dirserv_remove_old_servers(int age)
 {
   int i;
   time_t cutoff;
-  descriptor_entry_t *ent;
+  routerinfo_t *ent;
   if (!descriptor_list)
     descriptor_list = smartlist_create();
 
   cutoff = time(NULL) - age;
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
     ent = smartlist_get(descriptor_list, i);
-    if (ent->published <= cutoff) {
+    if (ent->published_on <= cutoff) {
       /* descriptor_list[i] is too old.  Remove it. */
-      free_descriptor_entry(ent);
+      routerinfo_free(ent);
       smartlist_del(descriptor_list, i--);
       directory_set_dirty();
     }
@@ -596,7 +618,7 @@ dirserv_dump_directory_to_string(char **dir_out,
   if (!descriptor_list)
     descriptor_list = smartlist_create();
 
-  if (list_server_status(&running_routers, &router_status))
+  if (list_server_status(descriptor_list, &running_routers, &router_status))
     return -1;
 
   /* ASN.1-encode the public key.  This is a temporary measure; once
@@ -637,8 +659,8 @@ dirserv_dump_directory_to_string(char **dir_out,
 
   buf_len = 2048+strlen(recommended_versions)+strlen(running_routers)+
     strlen(router_status);
-  SMARTLIST_FOREACH(descriptor_list, descriptor_entry_t *, d,
-                    buf_len += strlen(d->descriptor));
+  SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri,
+                    buf_len += strlen(ri->signed_descriptor));
   buf = tor_malloc(buf_len);
   /* We'll be comparing against buf_len throughout the rest of the
      function, though strictly speaking we shouldn't be able to exceed
@@ -662,8 +684,8 @@ dirserv_dump_directory_to_string(char **dir_out,
   i = strlen(buf);
   cp = buf+i;
 
-  SMARTLIST_FOREACH(descriptor_list, descriptor_entry_t *, d,
-    if (strlcat(buf, d->descriptor, buf_len) >= buf_len)
+  SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri,
+    if (strlcat(buf, ri->signed_descriptor, buf_len) >= buf_len)
       goto truncated);
 
   /* These multiple strlcat calls are inefficient, but dwarfed by the RSA
@@ -855,7 +877,10 @@ static int generate_runningrouters(crypto_pk_env_t *private_key)
   time_t published_on;
   char *identity_pkey; /* Identity key, DER64-encoded. */
 
-  if (list_server_status(NULL, &router_status)) {
+  if (!descriptor_list)
+    descriptor_list = smartlist_create();
+
+  if (list_server_status(descriptor_list, NULL, &router_status)) {
     goto err;
   }
   /* ASN.1-encode the public key.  This is a temporary measure; once
@@ -958,3 +983,35 @@ size_t dirserv_get_runningrouters(const char **rr, int compress)
   return compress ? the_runningrouters_z_len : the_runningrouters_len;
 }
 
+void
+dirserv_free_all(void)
+{
+  if (fingerprint_list) {
+    SMARTLIST_FOREACH(fingerprint_list, fingerprint_entry_t*, fp,
+                      { tor_free(fp->nickname);
+                        tor_free(fp->fingerprint);
+                        tor_free(fp); });
+    smartlist_free(fingerprint_list);
+    fingerprint_list = NULL;
+  }
+  if (descriptor_list) {
+    SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri,
+                      routerinfo_free(ri));
+    smartlist_free(descriptor_list);
+    descriptor_list = NULL;
+  }
+  tor_free(the_directory);
+  tor_free(the_directory_z);
+  the_directory_len = 0;
+  the_directory_z_len = 0;
+  tor_free(the_runningrouters);
+  tor_free(the_runningrouters_z);
+  the_runningrouters_len = 0;
+  the_runningrouters_z_len = 0;
+  tor_free(cached_directory.dir);
+  tor_free(cached_directory.dir_z);
+  tor_free(cached_runningrouters.dir);
+  tor_free(cached_runningrouters.dir_z);
+  memset(&cached_directory, 0, sizeof(cached_directory));
+  memset(&cached_runningrouters, 0, sizeof(cached_runningrouters));
+}
