@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c,v 1.126.2.9 2005/02/05 21:24:39 nickm Exp $ */
-const char dns_c_id[] = "$Id: dns.c,v 1.126.2.9 2005/02/05 21:24:39 nickm Exp $";
+/* $Id: dns.c,v 1.139 2005/03/14 04:42:52 arma Exp $ */
+const char dns_c_id[] = "$Id: dns.c,v 1.139 2005/03/14 04:42:52 arma Exp $";
 
 /**
  * \file dns.c
@@ -99,6 +99,27 @@ void dns_init(void) {
   spawn_enough_dnsworkers();
 }
 
+static void
+_free_cached_resolve(struct cached_resolve *r) {
+  while (r->pending_connections) {
+    struct pending_connection_t *victim = r->pending_connections;
+    r->pending_connections = victim->next;
+    tor_free(victim);
+  }
+  tor_free(r);
+}
+
+void
+dns_free_all(void)
+{
+  struct cached_resolve *ptr, *next;
+  for (ptr = SPLAY_MIN(cache_tree, &cache_root); ptr != NULL; ptr = next) {
+    next = SPLAY_NEXT(cache_tree, &cache_root, ptr);
+    SPLAY_REMOVE(cache_tree, &cache_root, ptr);
+    _free_cached_resolve(ptr);
+  }
+}
+
 /** Linked list of resolved addresses, oldest to newest. */
 static struct cached_resolve *oldest_cached_resolve = NULL;
 static struct cached_resolve *newest_cached_resolve = NULL;
@@ -118,16 +139,22 @@ static void purge_expired_resolves(uint32_t now) {
     log(LOG_DEBUG,"Forgetting old cached resolve (expires %lu)", (unsigned long)resolve->expire);
     if (resolve->state == CACHE_STATE_PENDING) {
       log_fn(LOG_WARN,"Bug: Expiring a dns resolve that's still pending. Forgot to cull it?");
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
     }
     if (resolve->pending_connections) {
       log_fn(LOG_WARN, "Closing pending connections on expiring DNS resolve!");
+#ifdef TOR_FRAGILE
+      tor_assert(0);
+#endif
       while (resolve->pending_connections) {
         pend = resolve->pending_connections;
         resolve->pending_connections = pend->next;
         /* Connections should only be pending if they have no socket. */
         tor_assert(pend->conn->s == -1);
         pendconn = pend->conn;
-        connection_edge_end(pendconn, END_STREAM_REASON_MISC,
+        connection_edge_end(pendconn, END_STREAM_REASON_TIMEOUT,
                             pendconn->cpath_layer);
         circuit_detach_stream(circuit_get_by_conn(pendconn), pendconn);
         connection_free(pendconn);
@@ -250,7 +277,8 @@ int dns_resolve(connection_t *exitconn) {
         circ = circuit_get_by_conn(exitconn);
         if (circ)
           circuit_detach_stream(circ, exitconn);
-        connection_free(exitconn);
+        if (!exitconn->marked_for_close)
+          connection_free(exitconn);
         return -1;
     }
     tor_assert(0);
@@ -289,7 +317,7 @@ static int assign_to_dnsworker(connection_t *exitconn) {
     log_fn(LOG_WARN,"no idle dns workers. Failing.");
     if (exitconn->purpose == EXIT_PURPOSE_RESOLVE)
       send_resolved_cell(exitconn, RESOLVED_TYPE_ERROR_TRANSIENT);
-    dns_cancel_pending_resolve(exitconn->address); /* also *frees* it */
+    dns_cancel_pending_resolve(exitconn->address); /* also sends end and frees! */
     return -1;
   }
 
@@ -408,6 +436,9 @@ void dns_cancel_pending_resolve(char *address) {
   if (!resolve->pending_connections) {
     /* XXX this should never trigger, but sometimes it does */
     log_fn(LOG_WARN,"Bug: Address '%s' is pending but has no pending connections!", address);
+#ifdef TOR_FRAGILE
+    tor_assert(0);
+#endif
     return;
   }
   tor_assert(resolve->pending_connections);
@@ -520,8 +551,7 @@ static void dns_found_answer(char *address, uint32_t addr, char outcome) {
       /* prevent double-remove. */
       pendconn->state = EXIT_CONN_STATE_RESOLVEFAILED;
       if (pendconn->purpose == EXIT_PURPOSE_CONNECT) {
-        if (!pendconn->has_sent_end) /* XXX prevent warn, but not fix bug */
-          connection_edge_end(pendconn, END_STREAM_REASON_RESOLVEFAILED, pendconn->cpath_layer);
+        connection_edge_end(pendconn, END_STREAM_REASON_RESOLVEFAILED, pendconn->cpath_layer);
         /* This detach must happen after we send the end cell. */
         circuit_detach_stream(circuit_get_by_conn(pendconn), pendconn);
       } else {
@@ -604,6 +634,9 @@ int connection_dns_process_inbuf(connection_t *conn) {
 
   if (conn->state != DNSWORKER_STATE_BUSY && buf_datalen(conn->inbuf)) {
     log_fn(LOG_WARN,"Bug: read data from an idle dns worker.  Please report.");
+#ifdef TOR_FRAGILE
+    tor_assert(0);
+#endif
     return 0;
   }
   if (buf_datalen(conn->inbuf) < 5) /* entire answer available? */
@@ -686,10 +719,15 @@ static int dnsworker_main(void *data) {
   tor_free(data);
 
   for (;;) {
+    int r;
 
-    if (recv(fd, &address_len, 1, 0) != 1) {
-      log_fn(LOG_INFO,"dnsworker exiting because tor process closed connection (either pruned idle dnsworker or died).");
-      log_fn(LOG_INFO,"Error on %d was %s", fd, tor_socket_strerror(tor_socket_errno(fd)));
+    if ((r = recv(fd, &address_len, 1, 0)) != 1) {
+      if (r == 0) {
+        log_fn(LOG_INFO,"DNS worker exiting because Tor process closed connection (either pruned idle dnsworker or died).");
+      } else {
+        log_fn(LOG_INFO,"DNS worker exiting because of error on connection to Tor process.");
+        log_fn(LOG_INFO,"(Error on %d was %s)", fd, tor_socket_strerror(tor_socket_errno(fd)));
+      }
       spawn_exit();
     }
 
