@@ -1,7 +1,7 @@
 /* Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: hibernate.c,v 1.41.2.1 2005/02/03 23:46:05 arma Exp $ */
-const char hibernate_c_id[] = "$Id: hibernate.c,v 1.41.2.1 2005/02/03 23:46:05 arma Exp $";
+/* $Id: hibernate.c,v 1.49 2005/03/27 04:55:12 arma Exp $ */
+const char hibernate_c_id[] = "$Id: hibernate.c,v 1.49 2005/03/27 04:55:12 arma Exp $";
 
 /**
  * \file hibernate.c
@@ -29,7 +29,7 @@ hibernating, phase 2:
 #define HIBERNATE_STATE_LOWBANDWIDTH 3
 #define HIBERNATE_STATE_DORMANT 4
 
-#define SHUTDOWN_WAIT_LENGTH 30 /* seconds */
+extern long stats_n_seconds_working; /* published uptime */
 
 static int hibernate_state = HIBERNATE_STATE_LIVE;
 /** If are hibernating, when do we plan to wake up? Set to 0 if we
@@ -235,13 +235,13 @@ static time_t
 edge_of_accounting_period_containing(time_t now, int get_end)
 {
   int before;
-  struct tm *tm;
-  tm = localtime(&now);
+  struct tm tm;
+  tor_localtime_r(&now, &tm);
 
   /* Set 'before' to true iff the current time is before the hh:mm
    * changeover time for today. */
-  before = tm->tm_hour < cfg_start_hour ||
-    (tm->tm_hour == cfg_start_hour && tm->tm_min < cfg_start_min);
+  before = tm.tm_hour < cfg_start_hour ||
+    (tm.tm_hour == cfg_start_hour && tm.tm_min < cfg_start_min);
 
   /* Dispatch by unit.  First, find the start day of the given period;
    * then, if get_end is true, increment to the end day. */
@@ -249,14 +249,14 @@ edge_of_accounting_period_containing(time_t now, int get_end)
     {
     case UNIT_MONTH: {
       /* If this is before the Nth, we want the Nth of last month. */
-      if (tm->tm_mday < cfg_start_day ||
-          (tm->tm_mday < cfg_start_day && before)) {
-        --tm->tm_mon;
+      if (tm.tm_mday < cfg_start_day ||
+          (tm.tm_mday < cfg_start_day && before)) {
+        --tm.tm_mon;
       }
       /* Otherwise, the month is correct. */
-      tm->tm_mday = cfg_start_day;
+      tm.tm_mday = cfg_start_day;
       if (get_end)
-        ++tm->tm_mon;
+        ++tm.tm_mon;
       break;
     }
     case UNIT_WEEK: {
@@ -264,31 +264,31 @@ edge_of_accounting_period_containing(time_t now, int get_end)
          say Sunday==7; struct tm says Sunday==0.) */
       int wday = cfg_start_day % 7;
       /* How many days do we subtract from today to get to the right day? */
-      int delta = (7+tm->tm_wday-wday)%7;
+      int delta = (7+tm.tm_wday-wday)%7;
       /* If we are on the right day, but the changeover hasn't happened yet,
        * then subtract a whole week. */
       if (delta == 0 && before)
         delta = 7;
-      tm->tm_mday -= delta;
+      tm.tm_mday -= delta;
       if (get_end)
-        tm->tm_mday += 7;
+        tm.tm_mday += 7;
       break;
     }
     case UNIT_DAY:
       if (before)
-        --tm->tm_mday;
+        --tm.tm_mday;
       if (get_end)
-        ++tm->tm_mday;
+        ++tm.tm_mday;
       break;
     default:
       tor_assert(0);
   }
 
-  tm->tm_hour = cfg_start_hour;
-  tm->tm_min = cfg_start_min;
-  tm->tm_sec = 0;
-  tm->tm_isdst = -1; /* Autodetect DST */
-  return mktime(tm);
+  tm.tm_hour = cfg_start_hour;
+  tm.tm_min = cfg_start_min;
+  tm.tm_sec = 0;
+  tm.tm_isdst = -1; /* Autodetect DST */
+  return mktime(&tm);
 }
 
 /** Return the start of the accounting period containing the time
@@ -409,6 +409,9 @@ accounting_run_housekeeping(time_t now)
   if (time_to_record_bandwidth_usage(now)) {
     if (accounting_record_bandwidth_usage(now)) {
       log_fn(LOG_ERR, "Couldn't record bandwidth usage; exiting.");
+      /* XXX this can fail when you're out of fd's, causing a crash.
+       * Perhaps the better answer is to hold the file open all the
+       * time? */
       exit(1);
     }
   }
@@ -651,10 +654,13 @@ static int hibernate_soft_limit_reached(void)
  * connections, but we continue handling old ones. */
 static void hibernate_begin(int new_state, time_t now) {
   connection_t *conn;
+  or_options_t *options = get_options();
 
-  if (hibernate_state == HIBERNATE_STATE_EXITING) {
-    /* we've been called twice now. close immediately. */
-    log(LOG_NOTICE,"Second sigint received; exiting now.");
+  if (new_state == HIBERNATE_STATE_EXITING &&
+      hibernate_state != HIBERNATE_STATE_LIVE) {
+    log(LOG_NOTICE,"Sigint received %s; exiting now.",
+        hibernate_state == HIBERNATE_STATE_EXITING ?
+          "a second time" : "while hibernating");
     tor_cleanup();
     exit(0);
   }
@@ -671,8 +677,8 @@ static void hibernate_begin(int new_state, time_t now) {
   /* XXX upload rendezvous service descriptors with no intro points */
 
   if (new_state == HIBERNATE_STATE_EXITING) {
-    log(LOG_NOTICE,"Interrupt: will shut down in %d seconds. Interrupt again to exit now.", SHUTDOWN_WAIT_LENGTH);
-    hibernate_end_time = time(NULL) + SHUTDOWN_WAIT_LENGTH;
+    log(LOG_NOTICE,"Interrupt: will shut down in %d seconds. Interrupt again to exit now.", options->ShutdownWaitLength);
+    hibernate_end_time = time(NULL) + options->ShutdownWaitLength;
   } else { /* soft limit reached */
     hibernate_end_time = interval_end_time;
   }
@@ -693,6 +699,7 @@ hibernate_end(int new_state) {
 
   hibernate_state = new_state;
   hibernate_end_time = 0; /* no longer hibernating */
+  stats_n_seconds_working = 0; /* reset published uptime */
 }
 
 /** A wrapper around hibernate_begin, for when we get SIGINT. */
@@ -731,9 +738,13 @@ hibernate_go_dormant(time_t now) {
          (conn = connection_get_by_type(CONN_TYPE_AP)) ||
          (conn = connection_get_by_type(CONN_TYPE_EXIT))) {
     if (CONN_IS_EDGE(conn))
-      connection_edge_end(conn, END_STREAM_REASON_MISC, conn->cpath_layer);
+      connection_edge_end(conn, END_STREAM_REASON_HIBERNATING,
+                          conn->cpath_layer);
     log_fn(LOG_INFO,"Closing conn type %d", conn->type);
-    connection_mark_for_close(conn);
+    if (conn->type == CONN_TYPE_AP) /* send socks failure if needed */
+      connection_close_unattached_ap(conn, END_STREAM_REASON_HIBERNATING);
+    else
+      connection_mark_for_close(conn);
   }
 
   accounting_record_bandwidth_usage(now);
