@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: onion.c,v 1.173.2.2 2005/02/20 08:54:59 arma Exp $ */
-const char onion_c_id[] = "$Id: onion.c,v 1.173.2.2 2005/02/20 08:54:59 arma Exp $";
+/* $Id: onion.c,v 1.176 2005/02/20 09:27:48 arma Exp $ */
+const char onion_c_id[] = "$Id: onion.c,v 1.176 2005/02/20 09:27:48 arma Exp $";
 
 /**
  * \file onion.c
@@ -329,3 +329,15 @@ onion_skin_client_handshake(crypto_dh_env_t *handshake_state,
   return 0;
 }
 
+/** Remove all circuits from the pending list.  Called from tor_free_all. */
+void
+clear_pending_onions(void)
+{
+  while (ol_list) {
+    struct onion_queue_t *victim = ol_list;
+    ol_list = victim->next;
+    tor_free(victim);
+  }
+  ol_list = ol_tail = NULL;
+  ol_length = 0;
+}
