@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: relay.c,v 1.32.2.4 2005/03/24 21:30:33 arma Exp $ */
-const char relay_c_id[] = "$Id: relay.c,v 1.32.2.4 2005/03/24 21:30:33 arma Exp $";
+/* $Id: relay.c,v 1.58 2005/03/27 06:37:56 arma Exp $ */
+const char relay_c_id[] = "$Id: relay.c,v 1.58 2005/03/27 06:37:56 arma Exp $";
 
 /**
  * \file relay.c
@@ -107,18 +107,13 @@ static int relay_digest_matches(crypto_digest_env_t *digest, cell_t *cell) {
 static int relay_crypt_one_payload(crypto_cipher_env_t *cipher, char *in,
                                    int encrypt_mode) {
   char out[CELL_PAYLOAD_SIZE]; /* 'in' must be this size too */
-  relay_header_t rh;
 
-  relay_header_unpack(&rh, in);
-//  log_fn(LOG_DEBUG,"before crypt: %d",rh.recognized);
   if (( encrypt_mode && crypto_cipher_encrypt(cipher, out, in, CELL_PAYLOAD_SIZE)) ||
       (!encrypt_mode && crypto_cipher_decrypt(cipher, out, in, CELL_PAYLOAD_SIZE))) {
     log_fn(LOG_WARN,"Error during relay encryption");
     return -1;
   }
   memcpy(in,out,CELL_PAYLOAD_SIZE);
-  relay_header_unpack(&rh, in);
-//  log_fn(LOG_DEBUG,"after crypt: %d",rh.recognized);
   return 0;
 }
 
@@ -349,7 +344,7 @@ relay_lookup_conn(circuit_t *circ, cell_t *cell, int cell_direction)
    */
 
   for (tmpconn = circ->n_streams; tmpconn; tmpconn=tmpconn->next_stream) {
-    if (rh.stream_id == tmpconn->stream_id) {
+    if (rh.stream_id == tmpconn->stream_id && !tmpconn->marked_for_close) {
       log_fn(LOG_DEBUG,"found conn for stream %d.", rh.stream_id);
       if (cell_direction == CELL_DIRECTION_OUT ||
           connection_edge_is_rendezvous_stream(tmpconn))
@@ -357,13 +352,13 @@ relay_lookup_conn(circuit_t *circ, cell_t *cell, int cell_direction)
     }
   }
   for (tmpconn = circ->p_streams; tmpconn; tmpconn=tmpconn->next_stream) {
-    if (rh.stream_id == tmpconn->stream_id) {
+    if (rh.stream_id == tmpconn->stream_id && !tmpconn->marked_for_close) {
       log_fn(LOG_DEBUG,"found conn for stream %d.", rh.stream_id);
       return tmpconn;
     }
   }
   for (tmpconn = circ->resolving_streams; tmpconn; tmpconn=tmpconn->next_stream) {
-    if (rh.stream_id == tmpconn->stream_id) {
+    if (rh.stream_id == tmpconn->stream_id && !tmpconn->marked_for_close) {
       log_fn(LOG_DEBUG,"found conn for stream %d.", rh.stream_id);
       return tmpconn;
     }
@@ -421,8 +416,12 @@ int connection_edge_send_command(connection_t *fromconn, circuit_t *circ,
   if (!circ) {
     log_fn(LOG_WARN,"no circ. Closing conn.");
     tor_assert(fromconn);
-    fromconn->has_sent_end = 1; /* no circ to send to */
-    connection_mark_for_close(fromconn);
+    if (fromconn->type == CONN_TYPE_AP) {
+      connection_close_unattached_ap(fromconn, END_STREAM_REASON_INTERNAL);
+    } else {
+      fromconn->has_sent_end = 1; /* no circ to send to */
+      connection_mark_for_close(fromconn);
+    }
     return -1;
   }
 
@@ -442,8 +441,10 @@ int connection_edge_send_command(connection_t *fromconn, circuit_t *circ,
     rh.stream_id = fromconn->stream_id; /* else it's 0 */
   rh.length = payload_len;
   relay_header_pack(cell.payload, &rh);
-  if (payload_len)
+  if (payload_len) {
+    tor_assert(payload_len <= RELAY_PAYLOAD_SIZE);
     memcpy(cell.payload+RELAY_HEADER_SIZE, payload, payload_len);
+  }
 
   log_fn(LOG_DEBUG,"delivering %d cell %s.", relay_command,
          cell_direction == CELL_DIRECTION_OUT ? "forward" : "backward");
@@ -461,7 +462,7 @@ int connection_edge_send_command(connection_t *fromconn, circuit_t *circ,
  * why the stream is closing.
  */
 static const char *
-connection_edge_end_reason(char *payload, uint16_t length) {
+connection_edge_end_reason_str(char *payload, uint16_t length) {
   if (length < 1) {
     log_fn(LOG_WARN,"End cell arrived with length 0. Should be at least 1.");
     return "MALFORMED";
@@ -485,6 +486,98 @@ connection_edge_end_reason(char *payload, uint16_t length) {
   }
 }
 
+/** Translate <b>reason</b> (as from a relay 'end' cell) into an
+ * appropriate SOCKS5 reply code.
+ */
+socks5_reply_status_t
+connection_edge_end_reason_socks5_response(int reason)
+{
+  switch (reason) {
+    case END_STREAM_REASON_MISC:
+      return SOCKS5_GENERAL_ERROR;
+    case END_STREAM_REASON_RESOLVEFAILED:
+      return SOCKS5_HOST_UNREACHABLE;
+    case END_STREAM_REASON_CONNECTREFUSED:
+      return SOCKS5_CONNECTION_REFUSED;
+    case END_STREAM_REASON_EXITPOLICY:
+      return SOCKS5_NOT_ALLOWED;
+    case END_STREAM_REASON_DESTROY:
+      return SOCKS5_GENERAL_ERROR;
+    case END_STREAM_REASON_DONE:
+      return SOCKS5_SUCCEEDED;
+    case END_STREAM_REASON_TIMEOUT:
+      return SOCKS5_TTL_EXPIRED;
+    case END_STREAM_REASON_RESOURCELIMIT:
+      return SOCKS5_GENERAL_ERROR;
+    case END_STREAM_REASON_HIBERNATING:
+      return SOCKS5_GENERAL_ERROR;
+    case END_STREAM_REASON_INTERNAL:
+      return SOCKS5_GENERAL_ERROR;
+    case END_STREAM_REASON_CONNRESET:
+      return SOCKS5_CONNECTION_REFUSED;
+    case END_STREAM_REASON_TORPROTOCOL:
+      return SOCKS5_GENERAL_ERROR;
+
+    case END_STREAM_REASON_ALREADY_SOCKS_REPLIED:
+      return SOCKS5_SUCCEEDED; /* never used */
+    case END_STREAM_REASON_CANT_ATTACH:
+      return SOCKS5_GENERAL_ERROR;
+    case END_STREAM_REASON_NET_UNREACHABLE:
+      return SOCKS5_NET_UNREACHABLE;
+    default:
+      log_fn(LOG_WARN,"Reason for ending (%d) not recognized.",reason);
+      return SOCKS5_GENERAL_ERROR;
+  }
+}
+
+/* We need to use a few macros to deal with the fact that Windows
+ * decided that their sockets interface should be a permakludge.
+ * E_CASE is for errors where windows has both a EFOO and a WSAEFOO
+ * version, and S_CASE is for errors where windows has only a WSAEFOO
+ * version.  (The E is for 'error', the S is for 'socket'). */
+#ifdef MS_WINDOWS
+#define E_CASE(s) case s: case WSA ## s
+#define S_CASE(s) case WSA ## s
+#else
+#define E_CASE(s) case s
+#define S_CASE(s) case s
+#endif
+
+int
+errno_to_end_reason(int e)
+{
+  switch (e) {
+    case EPIPE:
+      return END_STREAM_REASON_DONE;
+    E_CASE(EBADF):
+    E_CASE(EFAULT):
+    E_CASE(EINVAL):
+    S_CASE(EISCONN):
+    S_CASE(ENOTSOCK):
+    S_CASE(EPROTONOSUPPORT):
+    S_CASE(EAFNOSUPPORT):
+    E_CASE(EACCES):
+    S_CASE(ENOTCONN):
+    S_CASE(ENETUNREACH):
+      return END_STREAM_REASON_INTERNAL;
+    S_CASE(ECONNREFUSED):
+      return END_STREAM_REASON_CONNECTREFUSED;
+    S_CASE(ECONNRESET):
+      return END_STREAM_REASON_CONNRESET;
+    S_CASE(ETIMEDOUT):
+      return END_STREAM_REASON_TIMEOUT;
+    S_CASE(ENOBUFS):
+    case ENOMEM:
+    case ENFILE:
+    E_CASE(EMFILE):
+      return END_STREAM_REASON_RESOURCELIMIT;
+    default:
+      log_fn(LOG_INFO, "Didn't recognize errno %d (%s); telling the OP that we are ending a stream for 'misc' reason.",
+             e, tor_socket_strerror(e));
+      return END_STREAM_REASON_MISC;
+  }
+}
+
 /** How many times will I retry a stream that fails due to DNS
  * resolve failure?
  */
@@ -501,6 +594,7 @@ static int
 connection_edge_process_relay_cell_not_open(
     relay_header_t *rh, cell_t *cell, circuit_t *circ,
     connection_t *conn, crypt_path_t *layer_hint) {
+  struct in_addr in;
   uint32_t addr;
   int reason;
   routerinfo_t *exitrouter;
@@ -515,16 +609,16 @@ connection_edge_process_relay_cell_not_open(
       }
       addr = ntohl(get_uint32(cell->payload+RELAY_HEADER_SIZE+1));
       if (addr) {
-        log_fn(LOG_INFO,"Address %s refused due to exit policy. Retrying.",
+        log_fn(LOG_INFO,"Address '%s' refused due to exit policy. Retrying.",
                conn->socks_request->address);
       } else {
-        log_fn(LOG_INFO,"Address %s resolved to 0.0.0.0. Closing,",
+        log_fn(LOG_INFO,"Address '%s' resolved to 0.0.0.0. Closing,",
                conn->socks_request->address);
-        conn->has_sent_end = 1; /* we just got an 'end', don't need to send one */
-        connection_mark_for_close(conn);
+        connection_close_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
         return 0;
       }
-      client_dns_set_entry(conn->socks_request->address, addr);
+      client_dns_set_addressmap(conn->socks_request->address, addr,
+                                conn->chosen_exit_name);
 
       /* check if he *ought* to have allowed it */
       exitrouter = router_get_by_digest(circ->build_state->chosen_exit_digest);
@@ -532,34 +626,36 @@ connection_edge_process_relay_cell_not_open(
         log_fn(LOG_INFO,"Skipping broken circ (exit router vanished)");
         return 0; /* this circuit is screwed and doesn't know it yet */
       }
-      if (connection_ap_can_use_exit(conn, exitrouter)) {
-        log_fn(LOG_NOTICE,"Exitrouter %s seems to be more restrictive than its exit policy. Not using this router as exit for now,", exitrouter->nickname);
+      if (!tor_inet_aton(conn->socks_request->address, &in) &&
+          !conn->chosen_exit_name) {
+        log_fn(LOG_NOTICE,"Exitrouter '%s' seems to be more restrictive than its exit policy. Not using this router as exit for now.", exitrouter->nickname);
         addr_policy_free(exitrouter->exit_policy);
         exitrouter->exit_policy =
           router_parse_addr_policy_from_string("reject *:*");
       }
 
-      conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
-      circuit_detach_stream(circ,conn);
-      if (connection_ap_handshake_attach_circuit(conn) >= 0)
+      if (connection_ap_detach_retriable(conn, circ) >= 0)
         return 0;
+
       log_fn(LOG_INFO,"Giving up on retrying (from exitpolicy); conn can't be handled.");
       /* else, conn will get closed below */
     } else if (rh->length && reason == END_STREAM_REASON_RESOLVEFAILED) {
+      if (conn->type != CONN_TYPE_AP) {
+        log_fn(LOG_WARN,"Got an end because of resolvefailed, but we're not an AP. Closing.");
+        return -1;
+      }
       if (client_dns_incr_failures(conn->socks_request->address)
           < MAX_RESOLVE_FAILURES) {
         /* We haven't retried too many times; reattach the connection. */
         log_fn(LOG_INFO,"Resolve of '%s' failed, trying again.",
                conn->socks_request->address);
         circuit_log_path(LOG_INFO,circ);
-        conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
-        circuit_detach_stream(circ,conn);
         tor_assert(circ->timestamp_dirty);
-        circ->timestamp_dirty -= get_options()->NewCircuitPeriod;
-        /* make sure not to expire/retry the stream quite yet */
-        conn->timestamp_lastread = time(NULL);
-        if (connection_ap_handshake_attach_circuit(conn) >= 0)
+        circ->timestamp_dirty -= get_options()->MaxCircuitDirtiness;
+
+        if (connection_ap_detach_retriable(conn, circ) >= 0)
           return 0;
+
         /* else, conn will get closed below */
         log_fn(LOG_INFO,"Giving up on retrying (from resolvefailed); conn can't be handled.");
       } else {
@@ -568,11 +664,17 @@ connection_edge_process_relay_cell_not_open(
       }
     }
     log_fn(LOG_INFO,"Edge got end (%s) before we're connected. Marking for close.",
-      connection_edge_end_reason(cell->payload+RELAY_HEADER_SIZE, rh->length));
+           connection_edge_end_reason_str(cell->payload+RELAY_HEADER_SIZE,
+                                          rh->length));
     if (CIRCUIT_IS_ORIGIN(circ))
       circuit_log_path(LOG_INFO,circ);
-    conn->has_sent_end = 1; /* we just got an 'end', don't need to send one */
-    connection_mark_for_close(conn);
+    if (conn->type == CONN_TYPE_AP) {
+      connection_close_unattached_ap(conn,
+        *(char *)(cell->payload+RELAY_HEADER_SIZE));
+    } else {
+      conn->has_sent_end = 1; /* we just got an 'end', don't need to send one */
+      connection_mark_for_close(conn);
+    }
     return 0;
   }
 
@@ -589,18 +691,18 @@ connection_edge_process_relay_cell_not_open(
       addr = ntohl(get_uint32(cell->payload+RELAY_HEADER_SIZE));
       if (!addr) {
         log_fn(LOG_INFO,"...but it claims the IP address was 0.0.0.0. Closing.");
-        connection_edge_end(conn, END_STREAM_REASON_MISC, conn->cpath_layer);
-        connection_mark_for_close(conn);
+        connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL, conn->cpath_layer);
+        connection_close_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
         return 0;
       }
-      client_dns_set_entry(conn->socks_request->address, addr);
+      client_dns_set_addressmap(conn->socks_request->address, addr,
+                                conn->chosen_exit_name);
     }
     circuit_log_path(LOG_INFO,circ);
-    connection_ap_handshake_socks_reply(conn, NULL, 0, 1);
-    conn->socks_request->has_finished = 1;
+    connection_ap_handshake_socks_reply(conn, NULL, 0, SOCKS5_SUCCEEDED);
     /* handle anything that might have queued */
     if (connection_edge_package_raw_inbuf(conn, 1) < 0) {
-      connection_edge_end(conn, END_STREAM_REASON_MISC, conn->cpath_layer);
+      /* (We already sent an end cell if possible) */
       connection_mark_for_close(conn);
       return 0;
     }
@@ -614,23 +716,20 @@ connection_edge_process_relay_cell_not_open(
     tor_assert(conn->socks_request->command == SOCKS_COMMAND_RESOLVE);
     if (rh->length < 2 || cell->payload[RELAY_HEADER_SIZE+1]+2>rh->length) {
       log_fn(LOG_WARN, "Dropping malformed 'resolved' cell");
-      conn->has_sent_end = 1;
-      connection_mark_for_close(conn);
+      connection_close_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
       return 0;
     }
     connection_ap_handshake_socks_resolved(conn,
                    cell->payload[RELAY_HEADER_SIZE], /*answer_type*/
                    cell->payload[RELAY_HEADER_SIZE+1], /*answer_len*/
                    cell->payload+RELAY_HEADER_SIZE+2); /* answer */
-    conn->has_sent_end = 1;
-    connection_mark_for_close(conn);
-    conn->hold_open_until_flushed = 1;
+    connection_close_unattached_ap(conn, END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
     return 0;
   }
 
   log_fn(LOG_WARN,"Got an unexpected relay command %d, in state %d (%s). Closing.",
          rh->command, conn->state, conn_state_to_string[conn->type][conn->state]);
-  connection_edge_end(conn, END_STREAM_REASON_MISC, conn->cpath_layer);
+  connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL, conn->cpath_layer);
   connection_mark_for_close(conn);
   return -1;
 }
@@ -691,7 +790,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
       if (( layer_hint && --layer_hint->deliver_window < 0) ||
           (!layer_hint && --circ->deliver_window < 0)) {
         log_fn(LOG_WARN,"(relay data) circ deliver_window below 0. Killing.");
-        connection_edge_end(conn, END_STREAM_REASON_MISC, conn->cpath_layer);
+        connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL, conn->cpath_layer);
         connection_mark_for_close(conn);
         return -1;
       }
@@ -722,15 +821,27 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
     case RELAY_COMMAND_END:
       if (!conn) {
         log_fn(LOG_INFO,"end cell (%s) dropped, unknown stream.",
-          connection_edge_end_reason(cell->payload+RELAY_HEADER_SIZE, rh.length));
+               connection_edge_end_reason_str(cell->payload+RELAY_HEADER_SIZE,
+                                              rh.length));
         return 0;
       }
 /* XXX add to this log_fn the exit node's nickname? */
       log_fn(LOG_INFO,"%d: end cell (%s) for stream %d. Removing stream. Size %d.",
-        conn->s,
-        connection_edge_end_reason(cell->payload+RELAY_HEADER_SIZE, rh.length),
-        conn->stream_id, (int)conn->stream_size);
-
+             conn->s,
+             connection_edge_end_reason_str(cell->payload+RELAY_HEADER_SIZE,
+                                            rh.length),
+             conn->stream_id, (int)conn->stream_size);
+      if (conn->socks_request && !conn->socks_request->has_finished) {
+        socks5_reply_status_t status;
+        if (rh.length < 1) {
+          log_fn(LOG_WARN,"End cell arrived with length 0. Should be at least 1.");
+          status = SOCKS5_GENERAL_ERROR;
+        } else {
+          status = connection_edge_end_reason_socks5_response(
+                        *(uint8_t*)cell->payload+RELAY_HEADER_SIZE);
+        }
+        connection_ap_handshake_socks_reply(conn, NULL, 0, status);
+      }
 #ifdef HALF_OPEN
       conn->done_sending = 1;
       shutdown(conn->s, 1); /* XXX check return; refactor NM */
@@ -817,7 +928,12 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
       conn->package_window += STREAMWINDOW_INCREMENT;
       log_fn(LOG_DEBUG,"stream-level sendme, packagewindow now %d.", conn->package_window);
       connection_start_reading(conn);
-      connection_edge_package_raw_inbuf(conn, 1); /* handle whatever might still be on the inbuf */
+      /* handle whatever might still be on the inbuf */
+      if (connection_edge_package_raw_inbuf(conn, 1) < 0) {
+        /* (We already sent an end cell if possible) */
+        connection_mark_for_close(conn);
+        return 0;
+      }
       return 0;
     case RELAY_COMMAND_RESOLVE:
       if (layer_hint) {
@@ -866,7 +982,8 @@ uint64_t stats_n_data_bytes_received = 0;
  * and the appropriate package windows aren't empty, grab a cell
  * and send it down the circuit.
  *
- * Return -1 if conn should be marked for close, else return 0.
+ * Return -1 (and send a RELAY_END cell if necessary) if conn should
+ * be marked for close, else return 0.
  */
 int connection_edge_package_raw_inbuf(connection_t *conn, int package_partial) {
   size_t amount_to_process, length;
@@ -875,12 +992,17 @@ int connection_edge_package_raw_inbuf(connection_t *conn, int package_partial) {
 
   tor_assert(conn);
   tor_assert(!connection_speaks_cells(conn));
+  if (conn->marked_for_close) {
+    log_fn(LOG_WARN,"Bug: called on conn that's already marked for close at %s:%d.",
+           conn->marked_for_close_file, conn->marked_for_close);
+    return 0;
+  }
 
 repeat_connection_edge_package_raw_inbuf:
 
   circ = circuit_get_by_conn(conn);
   if (!circ) {
-    log_fn(LOG_INFO,"conn has no circuits! Closing.");
+    log_fn(LOG_INFO,"conn has no circuit! Closing.");
     return -1;
   }
 
@@ -920,7 +1042,8 @@ repeat_connection_edge_package_raw_inbuf:
 
   if (connection_edge_send_command(conn, circ, RELAY_COMMAND_DATA,
                                    payload, length, conn->cpath_layer) < 0)
-    return 0; /* circuit is closed, don't continue */
+    /* circuit got marked for close, don't continue, don't need to mark conn */
+    return 0;
 
   if (!conn->cpath_layer) { /* non-rendezvous exit */
     tor_assert(circ->package_window > 0);
@@ -1005,7 +1128,11 @@ circuit_resume_edge_reading_helper(connection_t *conn,
         (layer_hint && conn->package_window > 0 && conn->cpath_layer == layer_hint)) {
       connection_start_reading(conn);
       /* handle whatever might still be on the inbuf */
-      connection_edge_package_raw_inbuf(conn, 1);
+      if (connection_edge_package_raw_inbuf(conn, 1)<0) {
+        /* (We already sent an end cell if possible) */
+        connection_mark_for_close(conn);
+        continue;
+      }
 
       /* If the circuit won't accept any more data, return without looking
        * at any more of the streams. Any connections that should be stopped
