@@ -1,7 +1,7 @@
 /* Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendcommon.c,v 1.43 2004/11/29 22:25:30 nickm Exp $ */
-const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.43 2004/11/29 22:25:30 nickm Exp $";
+/* $Id: rendcommon.c,v 1.45 2005/02/28 22:38:00 nickm Exp $ */
+const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.45 2005/02/28 22:38:00 nickm Exp $";
 
 /**
  * \file rendcommon.c
@@ -170,6 +170,22 @@ void rend_cache_init(void)
   rend_cache = strmap_new();
 }
 
+static void
+_rend_cache_entry_free(void *p)
+{
+  rend_cache_entry_t *e = p;
+  rend_service_descriptor_free(e->parsed);
+  tor_free(e->desc);
+  tor_free(e);
+}
+
+void
+rend_cache_free_all(void)
+{
+  strmap_free(rend_cache, _rend_cache_entry_free);
+  rend_cache = NULL;
+}
+
 /** Removes all old entries from the service descriptor cache.
  */
 void rend_cache_clean(void)
@@ -185,9 +201,7 @@ void rend_cache_clean(void)
     ent = (rend_cache_entry_t*)val;
     if (ent->parsed->timestamp < cutoff) {
       iter = strmap_iter_next_rmv(rend_cache, iter);
-      rend_service_descriptor_free(ent->parsed);
-      tor_free(ent->desc);
-      tor_free(ent);
+      _rend_cache_entry_free(ent);
     } else {
       iter = strmap_iter_next(rend_cache, iter);
     }
@@ -242,7 +256,9 @@ int rend_cache_lookup_desc(const char *query, const char **desc, size_t *desc_le
 /** Parse *desc, calculate its service id, and store it in the cache.
  * If we have a newer descriptor with the same ID, ignore this one.
  * If we have an older descriptor with the same ID, replace it.
- * Returns -1 if it's malformed or otherwise rejected, else return 0.
+ * Return -1 if it's malformed or otherwise rejected; return 0 if
+ * it's the same or older than one we've already got; return 1 if
+ * it's novel.
  */
 int rend_cache_store(const char *desc, size_t desc_len)
 {
@@ -250,6 +266,7 @@ int rend_cache_store(const char *desc, size_t desc_len)
   rend_service_descriptor_t *parsed;
   char query[REND_SERVICE_ID_LEN+1];
   time_t now;
+
   tor_assert(rend_cache);
   parsed = rend_parse_service_descriptor(desc,desc_len);
   if (!parsed) {
@@ -298,7 +315,7 @@ int rend_cache_store(const char *desc, size_t desc_len)
   memcpy(e->desc, desc, desc_len);
 
   log_fn(LOG_INFO,"Successfully stored rend desc '%s', len %d", query, (int)desc_len);
-  return 0;
+  return 1;
 }
 
 /** Called when we get a rendezvous-related relay cell on circuit
