@@ -1,7 +1,7 @@
 /* Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendmid.c,v 1.34 2004/12/01 03:48:13 nickm Exp $ */
-const char rendmid_c_id[] = "$Id: rendmid.c,v 1.34 2004/12/01 03:48:13 nickm Exp $";
+/* $Id: rendmid.c,v 1.35 2005/01/13 20:21:11 arma Exp $ */
+const char rendmid_c_id[] = "$Id: rendmid.c,v 1.35 2005/01/13 20:21:11 arma Exp $";
 
 /**
  * \file rendmid.c
@@ -123,8 +123,7 @@ rend_mid_introduce(circuit_t *circ, const char *request, size_t request_len)
     goto err;
   }
 
-/* change MAX_NICKNAME_LEN to MAX_HEX_NICKNAME_LEN when 0.0.8.1 and
- * 0.0.9pre3-4 are obsolete. */
+  /* change to MAX_HEX_NICKNAME_LEN once 0.0.9.x is obsolete */
   if (request_len < (DIGEST_LEN+(MAX_NICKNAME_LEN+1)+REND_COOKIE_LEN+
                      DH_KEY_LEN+CIPHER_KEY_LEN+PKCS1_OAEP_PADDING_OVERHEAD)) {
     log_fn(LOG_WARN,
