@@ -1,7 +1,7 @@
 /* Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendservice.c,v 1.111.2.1 2005/01/17 18:50:45 arma Exp $ */
-const char rendservice_c_id[] = "$Id: rendservice.c,v 1.111.2.1 2005/01/17 18:50:45 arma Exp $";
+/* $Id: rendservice.c,v 1.120 2005/03/23 06:21:48 nickm Exp $ */
+const char rendservice_c_id[] = "$Id: rendservice.c,v 1.120 2005/03/23 06:21:48 nickm Exp $";
 
 /**
  * \file rendservice.c
@@ -31,7 +31,10 @@ typedef struct rend_service_port_config_t {
 #define MAX_INTRO_CIRCS_PER_PERIOD 10
 /** How many times will a hidden service operator attempt to connect to
  * a requested rendezvous point before giving up? */
-#define MAX_REND_FAILURES 3
+#define MAX_REND_FAILURES 30
+/** How many seconds should we spend trying to connect to a requested
+ * rendezvous point before giving up? */
+#define MAX_REND_TIMEOUT 30
 
 /** Represents a single hidden service running at this OP. */
 typedef struct rend_service_t {
@@ -77,19 +80,17 @@ static void rend_service_free(rend_service_t *service)
   tor_free(service);
 }
 
-/** Release all the storage held in rend_service_list, and allocate a new,
- * empty rend_service_list.
+/** Release all the storage held in rend_service_list.
  */
-static void rend_service_free_all(void)
+void rend_service_free_all(void)
 {
   if (!rend_service_list) {
-    rend_service_list = smartlist_create();
     return;
   }
   SMARTLIST_FOREACH(rend_service_list, rend_service_t*, ptr,
                     rend_service_free(ptr));
   smartlist_free(rend_service_list);
-  rend_service_list = smartlist_create();
+  rend_service_list = NULL;
 }
 
 /** Validate <b>service</b> and add it to rend_service_list if possible.
@@ -113,10 +114,12 @@ static void add_service(rend_service_t *service)
     smartlist_add(rend_service_list, service);
     log_fn(LOG_DEBUG,"Configuring service with directory %s",service->directory);
     for (i = 0; i < smartlist_len(service->ports); ++i) {
+      char addrbuf[INET_NTOA_BUF_LEN];
       p = smartlist_get(service->ports, i);
       addr.s_addr = htonl(p->real_address);
+      tor_inet_ntoa(&addr, addrbuf, sizeof(addrbuf));
       log_fn(LOG_DEBUG,"Service maps port %d to %s:%d",
-             p->virtual_port, inet_ntoa(addr), p->real_port);
+             p->virtual_port, addrbuf, p->real_port);
     }
   }
 }
@@ -190,8 +193,10 @@ int rend_config_services(or_options_t *options, int validate_only)
   rend_service_t *service = NULL;
   rend_service_port_config_t *portcfg;
 
-  if (!validate_only)
+  if (!validate_only) {
     rend_service_free_all();
+    rend_service_list = smartlist_create();
+  }
 
   for (line = options->RendConfigLines; line; line = line->next) {
     if (!strcasecmp(line->key, "HiddenServiceDir")) {
@@ -342,12 +347,28 @@ rend_service_get_by_pk_digest(const char* digest)
   return NULL;
 }
 
+/** Return 1 if any virtual port in <b>service</b> wants a circuit
+ * to have good uptime. Else return 0.
+ */
+static int
+rend_service_requires_uptime(rend_service_t *service) {
+  int i;
+  rend_service_port_config_t *p;
+
+  for (i=0; i < smartlist_len(service->ports); ++i) {
+    p = smartlist_get(service->ports, i);
+    if (smartlist_string_num_isin(get_options()->LongLivedPorts, p->virtual_port))
+      return 1;
+  }
+  return 0;
+}
+
 /******
  * Handle cells
  ******/
 
 /** Respond to an INTRODUCE2 cell by launching a circuit to the chosen
- * rendezvous points.
+ * rendezvous point.
  */
 int
 rend_service_introduce(circuit_t *circuit, const char *request, size_t request_len)
@@ -365,6 +386,7 @@ rend_service_introduce(circuit_t *circuit, const char *request, size_t request_l
   char hexcookie[9];
   int version;
   size_t nickname_field_len;
+  int circ_needs_uptime;
 
   base32_encode(serviceid, REND_SERVICE_ID_LEN+1,
                 circuit->rend_pk_digest,10);
@@ -422,8 +444,7 @@ rend_service_introduce(circuit_t *circuit, const char *request, size_t request_l
     rp_nickname = buf;
     version = 0;
   }
-  /* XXX when 0.0.8.1 and 0.0.9pre3-4 are obsolete, change this to
-   * reject version != 1. */
+  /* XXX when 0.0.9.x is obsolete, change this to reject version != 1. */
   ptr=memchr(rp_nickname,0,nickname_field_len);
   if (!ptr || ptr == rp_nickname) {
     log_fn(LOG_WARN, "Couldn't find a null-padded nickname in INTRODUCE2 cell");
@@ -437,6 +458,7 @@ rend_service_introduce(circuit_t *circuit, const char *request, size_t request_l
   /* Okay, now we know that a nickname is at the start of the buffer. */
   ptr = rp_nickname+nickname_field_len;
   len -= nickname_field_len;
+  len -= rp_nickname - buf; /* also remove header space used by version, if any */
   if (len != REND_COOKIE_LEN+DH_KEY_LEN) {
     log_fn(LOG_WARN, "Bad length %u for INTRODUCE2 cell.", (int)len);
     return -1;
@@ -456,10 +478,16 @@ rend_service_introduce(circuit_t *circuit, const char *request, size_t request_l
     goto err;
   }
 
+  circ_needs_uptime = rend_service_requires_uptime(service);
+
+  /* help predict this next time */
+  rep_hist_note_used_hidserv(time(NULL), circ_needs_uptime, 1);
+
   /* Launch a circuit to alice's chosen rendezvous point.
    */
   for (i=0;i<MAX_REND_FAILURES;i++) {
-    launched = circuit_launch_by_nickname(CIRCUIT_PURPOSE_S_CONNECT_REND, rp_nickname);
+    launched = circuit_launch_by_nickname(CIRCUIT_PURPOSE_S_CONNECT_REND, rp_nickname,
+                                          circ_needs_uptime, 1, 1);
     if (launched)
       break;
   }
@@ -480,6 +508,8 @@ rend_service_introduce(circuit_t *circuit, const char *request, size_t request_l
           sizeof(launched->rend_query));
   launched->build_state->pending_final_cpath = cpath =
     tor_malloc_zero(sizeof(crypt_path_t));
+  cpath->magic = CRYPT_PATH_MAGIC;
+  launched->build_state->expiry_time = time(NULL) + MAX_REND_TIMEOUT;
 
   cpath->handshake_state = dh;
   dh = NULL;
@@ -506,8 +536,9 @@ rend_service_relaunch_rendezvous(circuit_t *oldcirc)
   tor_assert(oldcirc->purpose == CIRCUIT_PURPOSE_S_CONNECT_REND);
 
   if (!oldcirc->build_state ||
-      oldcirc->build_state->failure_count > MAX_REND_FAILURES) {
-    log_fn(LOG_INFO,"Attempt to build circuit to %s for rendezvous has failed too many times; giving up.",
+      oldcirc->build_state->failure_count > MAX_REND_FAILURES ||
+      oldcirc->build_state->expiry_time < time(NULL)) {
+    log_fn(LOG_INFO,"Attempt to build circuit to %s for rendezvous has failed too many times or expired; giving up.",
            oldcirc->build_state->chosen_exit_name);
     return;
   }
@@ -524,7 +555,7 @@ rend_service_relaunch_rendezvous(circuit_t *oldcirc)
          oldstate->chosen_exit_name);
 
   newcirc = circuit_launch_by_nickname(CIRCUIT_PURPOSE_S_CONNECT_REND,
-                               oldstate->chosen_exit_name);
+                               oldstate->chosen_exit_name, 0, 1, 1);
   if (!newcirc) {
     log_fn(LOG_WARN,"Couldn't relaunch rendezvous circuit to %s",
            oldstate->chosen_exit_name);
@@ -533,6 +564,7 @@ rend_service_relaunch_rendezvous(circuit_t *oldcirc)
   newstate = newcirc->build_state;
   tor_assert(newstate);
   newstate->failure_count = oldstate->failure_count+1;
+  newstate->expiry_time = oldstate->expiry_time;
   newstate->pending_final_cpath = oldstate->pending_final_cpath;
   oldstate->pending_final_cpath = NULL;
 
@@ -552,8 +584,10 @@ rend_service_launch_establish_intro(rend_service_t *service, const char *nicknam
   log_fn(LOG_INFO, "Launching circuit to introduction point %s for service %s",
          nickname, service->service_id);
 
+  rep_hist_note_used_hidserv(time(NULL), 1, 0);
+
   ++service->n_intro_circuits_launched;
-  launched = circuit_launch_by_nickname(CIRCUIT_PURPOSE_S_ESTABLISH_INTRO, nickname);
+  launched = circuit_launch_by_nickname(CIRCUIT_PURPOSE_S_ESTABLISH_INTRO, nickname, 1, 0, 1);
   if (!launched) {
     log_fn(LOG_WARN, "Can't launch circuit to establish introduction at '%s'",
            nickname);
@@ -807,7 +841,7 @@ void rend_services_introduce(void) {
   exclude_routers = smartlist_create();
   now = time(NULL);
 
-  for (i=0; i< smartlist_len(rend_service_list); ++i) {
+  for (i=0; i < smartlist_len(rend_service_list); ++i) {
     smartlist_clear(intro_routers);
     service = smartlist_get(rend_service_list, i);
 
@@ -825,7 +859,7 @@ void rend_services_introduce(void) {
     }
 
     /* Find out which introduction points we have in progress for this service. */
-    for (j=0;j< smartlist_len(service->intro_nodes); ++j) {
+    for (j=0; j < smartlist_len(service->intro_nodes); ++j) {
       intro = smartlist_get(service->intro_nodes, j);
       router = router_get_by_nickname(intro);
       if (!router || !find_intro_circuit(router,service->pk_digest)) {
@@ -833,8 +867,8 @@ void rend_services_introduce(void) {
                 intro, service->service_id);
         tor_free(intro);
         smartlist_del(service->intro_nodes,j--);
-        service->desc_is_dirty = now;
         changed = 1;
+        service->desc_is_dirty = now;
       }
       smartlist_add(intro_routers, router);
     }
@@ -908,7 +942,7 @@ rend_consider_services_upload(time_t now) {
   rend_service_t *service;
   int rendpostperiod = get_options()->RendPostPeriod;
 
-  for (i=0; i< smartlist_len(rend_service_list); ++i) {
+  for (i=0; i < smartlist_len(rend_service_list); ++i) {
     service = smartlist_get(rend_service_list, i);
     if (!service->next_upload_time) { /* never been uploaded yet */
       service->next_upload_time =
