@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.137.2.2 2005/02/03 23:34:43 arma Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.137.2.2 2005/02/03 23:34:43 arma Exp $";
+/* $Id: router.c,v 1.165 2005/04/01 09:28:14 arma Exp $ */
+const char router_c_id[] = "$Id: router.c,v 1.165 2005/04/01 09:28:14 arma Exp $";
 
 #include "or.h"
 
@@ -13,7 +13,7 @@ const char router_c_id[] = "$Id: router.c,v 1.137.2.2 2005/02/03 23:34:43 arma E
  * and uploading server descriptors, retrying OR connections.
  **/
 
-extern long stats_n_seconds_uptime;
+extern long stats_n_seconds_working;
 
 /** Exposed for test.c. */ void get_platform_str(char *platform, size_t len);
 
@@ -57,6 +57,9 @@ crypto_pk_env_t *get_previous_onion_key(void) {
   return lastonionkey;
 }
 
+/** Store a copy of the current onion key into *<b>key</b>, and a copy
+ * of the most recent onion key into *<b>last</b>.
+ */
 void dup_onion_keys(crypto_pk_env_t **key, crypto_pk_env_t **last)
 {
   tor_assert(key);
@@ -129,31 +132,20 @@ void rotate_onion_key(void)
     log(LOG_ERR, "Couldn't write generated key to %s.", fname);
     goto error;
   }
+  log_fn(LOG_INFO, "Rotating onion key");
   tor_mutex_acquire(key_lock);
   if (lastonionkey)
     crypto_free_pk_env(lastonionkey);
-  log_fn(LOG_INFO, "Rotating onion key");
   lastonionkey = onionkey;
-  set_onion_key(prkey);
+  onionkey = prkey;
+  onionkey_set_at = time(NULL);
   tor_mutex_release(key_lock);
+  mark_my_descriptor_dirty();
   return;
  error:
   log_fn(LOG_WARN, "Couldn't rotate onion key.");
 }
 
-/** The latest calculated bandwidth usage for our node. */
-static int bw_capacity = 0;
-/** Tuck <b>bw</b> away so we can produce it when somebody
- * calls router_get_bandwidth_capacity() below.
- */
-void router_set_bandwidth_capacity(int bw) {
-  bw_capacity = bw;
-}
-/** Return the value we tucked away above, or zero by default. */
-int router_get_bandwidth_capacity(void) {
-  return bw_capacity;
-}
-
 /* Read an RSA secret key key from a file that was once named fname_old,
  * but is now named fname_new.  Rename the file from old to new as needed.
  */
@@ -240,7 +232,8 @@ int init_keys(void) {
    */
   char keydir[512];
   char keydir2[512];
-  char fingerprint[FINGERPRINT_LEN+MAX_NICKNAME_LEN+3];
+  char fingerprint[FINGERPRINT_LEN+1];
+  char fingerprint_line[FINGERPRINT_LEN+MAX_NICKNAME_LEN+3];/*nickname fp\n\0 */
   char *cp;
   const char *tmp, *mydesc, *datadir;
   crypto_pk_env_t *prkey;
@@ -314,13 +307,15 @@ int init_keys(void) {
     return -1;
   }
   if (authdir_mode(options)) {
+    const char *m;
     /* We need to add our own fingerprint so it gets recognized. */
     if (dirserv_add_own_fingerprint(options->Nickname, get_identity_key())) {
       log_fn(LOG_ERR, "Error adding own fingerprint to approved set");
       return -1;
     }
-    if (dirserv_add_descriptor(&tmp) != 1) {
-      log(LOG_ERR, "Unable to add own descriptor to directory.");
+    if (dirserv_add_descriptor(&tmp, &m) != 1) {
+      log(LOG_ERR, "Unable to add own descriptor to directory: %s",
+          m?m:"<unknown error>");
       return -1;
     }
   }
@@ -333,16 +328,17 @@ int init_keys(void) {
   /* 5. Dump fingerprint to 'fingerprint' */
   tor_snprintf(keydir,sizeof(keydir),"%s/fingerprint", datadir);
   log_fn(LOG_INFO,"Dumping fingerprint to %s...",keydir);
-  tor_assert(strlen(options->Nickname) <= MAX_NICKNAME_LEN);
-  strlcpy(fingerprint, options->Nickname, sizeof(fingerprint));
-  strlcat(fingerprint, " ", sizeof(fingerprint));
-  if (crypto_pk_get_fingerprint(get_identity_key(),
-                                fingerprint+strlen(fingerprint), 1)<0) {
+  if (crypto_pk_get_fingerprint(get_identity_key(), fingerprint, 1)<0) {
     log_fn(LOG_ERR, "Error computing fingerprint");
     return -1;
   }
-  strlcat(fingerprint, "\n", sizeof(fingerprint));
-  if (write_str_to_file(keydir, fingerprint, 0))
+  tor_assert(strlen(options->Nickname) <= MAX_NICKNAME_LEN);
+  if (tor_snprintf(fingerprint_line, sizeof(fingerprint_line),
+                   "%s %s\n",options->Nickname, fingerprint) < 0) {
+    log_fn(LOG_ERR, "Error writing fingerprint line");
+    return -1;
+  }
+  if (write_str_to_file(keydir, fingerprint_line, 0))
     return -1;
   if (!authdir_mode(options))
     return 0;
@@ -366,7 +362,7 @@ int init_keys(void) {
     log_fn(LOG_INFO,"Cached directory %s not present. Ok.",keydir);
   } else {
     if (dirserv_load_from_directory_string(cp) < 0) {
-      log_fn(LOG_WARN, "Cached directory %s is corrupt, only loaded part of it", keydir);
+      log_fn(LOG_WARN, "Cached directory %s is corrupt, only loaded part of it.", keydir);
       tor_free(cp);
       return 0;
     }
@@ -376,6 +372,136 @@ int init_keys(void) {
   return 0;
 }
 
+/* Keep track of whether we should upload our server descriptor,
+ * and what type of server we are.
+ */
+
+/** Whether we can reach our ORPort from the outside. */
+static int can_reach_or_port = 0;
+/** Whether we can reach our DirPort from the outside. */
+static int can_reach_dir_port = 0;
+
+/** Return 1 if or port is known reachable; else return 0. */
+int check_whether_orport_reachable(void) {
+  return clique_mode(get_options()) || can_reach_or_port;
+}
+/** Return 1 if we don't have a dirport configured, or if it's reachable. */
+int check_whether_dirport_reachable(void) {
+  return !get_options()->DirPort || can_reach_dir_port;
+}
+
+void consider_testing_reachability(void) {
+  routerinfo_t *me = router_get_my_routerinfo();
+  if (!me) {
+    log_fn(LOG_WARN,"Bug: router_get_my_routerinfo() did not find my routerinfo?");
+    return;
+  }
+
+  if (!check_whether_orport_reachable()) {
+    circuit_launch_by_router(CIRCUIT_PURPOSE_TESTING, me, 0, 1, 1);
+  }
+
+  if (!check_whether_dirport_reachable()) {
+    if (me) {
+      directory_initiate_command_router(me, DIR_PURPOSE_FETCH_DIR, 1, NULL, NULL, 0);
+    } else {
+      log_fn(LOG_NOTICE,"Delaying checking DirPort reachability; can't build descriptor.");
+    }
+  }
+}
+
+/** Annotate that we found our ORPort reachable. */
+void router_orport_found_reachable(void) {
+  if (!can_reach_or_port) {
+    if (!clique_mode(get_options()))
+      log_fn(LOG_NOTICE,"Your ORPort is reachable from the outside. Excellent. Publishing server descriptor.");
+    can_reach_or_port = 1;
+    consider_publishable_server(time(NULL), 1);
+  }
+}
+
+/** Annotate that we found our DirPort reachable. */
+void router_dirport_found_reachable(void) {
+  if (!can_reach_dir_port) {
+    log_fn(LOG_NOTICE,"Your DirPort is reachable from the outside. Excellent.");
+    can_reach_dir_port = 1;
+  }
+}
+
+/** Our router has just moved to a new IP. Reset stats. */
+void server_has_changed_ip(void) {
+  stats_n_seconds_working = 0;
+  can_reach_or_port = 0;
+  can_reach_dir_port = 0;
+  mark_my_descriptor_dirty();
+}
+
+/** Return true iff we believe ourselves to be an authoritative
+ * directory server.
+ */
+int authdir_mode(or_options_t *options) {
+  return options->AuthoritativeDir != 0;
+}
+/** Return true iff we try to stay connected to all ORs at once.
+ */
+int clique_mode(or_options_t *options) {
+  return authdir_mode(options);
+}
+
+/** Return true iff we are trying to be a server.
+ */
+int server_mode(or_options_t *options) {
+  return (options->ORPort != 0 || options->ORBindAddress);
+}
+
+/** Remember if we've advertised ourselves to the dirservers. */
+static int server_is_advertised=0;
+
+/** Return true iff we have published our descriptor lately.
+ */
+int advertised_server_mode(void) {
+  return server_is_advertised;
+}
+
+static void set_server_advertised(int s) {
+  server_is_advertised = s;
+}
+
+/** Return true iff we are trying to be a socks proxy. */
+int proxy_mode(or_options_t *options) {
+  return (options->SocksPort != 0 || options->SocksBindAddress);
+}
+
+/** Decide if we're a publishable server or just a client. We are a server if:
+ * - We have the AuthoritativeDirectory option set.
+ * or
+ * - We don't have the ClientOnly option set; and
+ * - We have ORPort set; and
+ * - We believe we are reachable from the outside.
+ */
+static int decide_if_publishable_server(time_t now) {
+  or_options_t *options = get_options();
+
+  if (options->ClientOnly)
+    return 0;
+  if (!server_mode(options))
+    return 0;
+  if (options->AuthoritativeDir)
+    return 1;
+
+  return check_whether_orport_reachable();
+}
+
+void consider_publishable_server(time_t now, int force) {
+  if (decide_if_publishable_server(now)) {
+    set_server_advertised(1);
+    router_rebuild_descriptor(force);
+    router_upload_dir_desc_to_dirservers(force);
+  } else {
+    set_server_advertised(0);
+  }
+}
+
 /*
  * Clique maintenance
  */
@@ -409,6 +535,8 @@ void router_retry_connections(void) {
   }
 }
 
+/** Return true iff this OR should try to keep connections open to all
+ * other ORs. */
 int router_is_clique_mode(routerinfo_t *router) {
   if (router_digest_is_trusted_dir(router->identity_digest))
     return 1;
@@ -421,15 +549,14 @@ int router_is_clique_mode(routerinfo_t *router) {
 
 /** My routerinfo. */
 static routerinfo_t *desc_routerinfo = NULL;
-/** String representation of my descriptor, signed by me. */
-static char descriptor[8192];
 /** Boolean: do we need to regenerate the above? */
 static int desc_is_dirty = 1;
 /** Boolean: do we need to regenerate the above? */
 static int desc_needs_upload = 0;
 
-/** OR only: try to upload our signed descriptor to all the directory servers
- * we know about. DOCDOC force
+/** OR only: If <b>force</b> is true, or we haven't uploaded this
+ * descriptor successfully yet, try to upload our signed descriptor to
+ * all the directory servers we know about.
  */
 void router_upload_dir_desc_to_dirservers(int force) {
   const char *s;
@@ -445,7 +572,7 @@ void router_upload_dir_desc_to_dirservers(int force) {
   directory_post_to_dirservers(DIR_PURPOSE_UPLOAD_DIR, s, strlen(s));
 }
 
-#define DEFAULT_EXIT_POLICY "reject 0.0.0.0/8,reject 169.254.0.0/16,reject 127.0.0.0/8,reject 192.168.0.0/16,reject 10.0.0.0/8,reject 172.16.0.0/12,accept *:20-22,accept *:53,accept *:79-81,accept *:110,accept *:143,accept *:443,accept *:706,accept *:873,accept *:993,accept *:995,reject *:1214,reject *:4661-4666,reject *:6346-6429,reject *:6881-6999,accept *:1024-65535,reject *:*"
+#define DEFAULT_EXIT_POLICY "reject 0.0.0.0/8,reject 169.254.0.0/16,reject 127.0.0.0/8,reject 192.168.0.0/16,reject 10.0.0.0/8,reject 172.16.0.0/12,reject *:25,reject *:119,reject *:135-139,reject *:445,reject *:1214,reject *:4661-4666,reject *:6346-6429,reject *:6699,reject *:6881-6999,accept *:*"
 
 /** Set the exit policy on <b>router</b> to match the exit policy in the
  * current configuration file.  If the exit policy doesn't have a catch-all
@@ -470,8 +597,8 @@ static void router_add_exit_policy_from_config(routerinfo_t *router) {
   config_parse_addr_policy(&default_policy, &router->exit_policy);
 }
 
-/** OR only: Return false if my exit policy says to allow connection to
- * conn.  Else return true.
+/** OR only: Check whether my exit policy says to allow connection to
+ * conn.  Return false if we accept; true if we reject.
  */
 int router_compare_to_my_exit_policy(connection_t *conn)
 {
@@ -483,10 +610,11 @@ int router_compare_to_my_exit_policy(connection_t *conn)
     return -1;
 
   return router_compare_addr_to_addr_policy(conn->addr, conn->port,
-                   desc_routerinfo->exit_policy);
+                   desc_routerinfo->exit_policy) != ADDR_POLICY_ACCEPTED;
 
 }
 
+
 /** Return true iff <b>router</b> has the same nickname as this OR.  (For an
  * OP, always returns false.)
  */
@@ -521,12 +649,13 @@ const char *router_get_my_descriptor(void) {
     if (router_rebuild_descriptor(1))
       return NULL;
   }
-  log_fn(LOG_DEBUG,"my desc is '%s'",descriptor);
-  return descriptor;
+  log_fn(LOG_DEBUG,"my desc is '%s'",desc_routerinfo->signed_descriptor);
+  return desc_routerinfo->signed_descriptor;
 }
 
-/** Rebuild a fresh routerinfo and signed server descriptor for this
- * OR.  Return 0 on success, -1 on error. DOCDOC force
+/** If <b>force</b> is true, or our descriptor is out-of-date, rebuild
+ * a fresh routerinfo and signed server descriptor for this OR.
+ * Return 0 on success, -1 on error.
  */
 int router_rebuild_descriptor(int force) {
   routerinfo_t *ri;
@@ -535,6 +664,7 @@ int router_rebuild_descriptor(int force) {
   struct in_addr in;
   int hibernating = we_are_hibernating();
   or_options_t *options = get_options();
+  char addrbuf[INET_NTOA_BUF_LEN];
 
   if (!desc_is_dirty && !force)
     return 0;
@@ -546,12 +676,13 @@ int router_rebuild_descriptor(int force) {
 
   ri = tor_malloc_zero(sizeof(routerinfo_t));
   in.s_addr = htonl(addr);
-  ri->address = tor_strdup(inet_ntoa(in));
+  tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
+  ri->address = tor_strdup(addrbuf);
   ri->nickname = tor_strdup(options->Nickname);
   ri->addr = addr;
   ri->or_port = options->ORPort;
-  ri->socks_port = hibernating ? 0 : options->SocksPort;
-  ri->dir_port = hibernating ? 0 : options->DirPort;
+  ri->dir_port = hibernating ?
+                 0 : options->DirPort;
   ri->published_on = time(NULL);
   ri->onion_pkey = crypto_pk_dup_key(get_onion_key()); /* must invoke from main thread */
   ri->identity_pkey = crypto_pk_dup_key(get_identity_key());
@@ -563,7 +694,11 @@ int router_rebuild_descriptor(int force) {
   ri->platform = tor_strdup(platform);
   ri->bandwidthrate = (int)options->BandwidthRate;
   ri->bandwidthburst = (int)options->BandwidthBurst;
-  ri->bandwidthcapacity = hibernating ? 0 : router_get_bandwidth_capacity();
+  ri->bandwidthcapacity = hibernating ? 0 : rep_hist_bandwidth_assess();
+
+  if (options->BandwidthRate > options->MaxAdvertisedBandwidth)
+    ri->bandwidthrate = (int)options->MaxAdvertisedBandwidth;
+
   router_add_exit_policy_from_config(ri);
   if (desc_routerinfo) /* inherit values */
     ri->is_verified = desc_routerinfo->is_verified;
@@ -572,20 +707,23 @@ int router_rebuild_descriptor(int force) {
     smartlist_split_string(ri->declared_family, options->MyFamily, ",",
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
   }
+  ri->signed_descriptor = tor_malloc(8192);
+  if (router_dump_router_to_string(ri->signed_descriptor, 8192,
+                                   ri, get_identity_key())<0) {
+    log_fn(LOG_WARN, "Couldn't dump router to string.");
+    return -1;
+  }
 
   if (desc_routerinfo)
     routerinfo_free(desc_routerinfo);
   desc_routerinfo = ri;
-  if (router_dump_router_to_string(descriptor, 8192, ri, get_identity_key())<0) {
-    log_fn(LOG_WARN, "Couldn't dump router to string.");
-    return -1;
-  }
+
   desc_is_dirty = 0;
   desc_needs_upload = 1;
   return 0;
 }
 
-/** DOCDOC */
+/** Call when the current descriptor is out of date. */
 void
 mark_my_descriptor_dirty(void)
 {
@@ -623,6 +761,7 @@ int router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
   char published[32];
   char fingerprint[FINGERPRINT_LEN+1];
   struct in_addr in;
+  char addrbuf[INET_NTOA_BUF_LEN];
   size_t onion_pkeylen, identity_pkeylen;
   size_t written;
   int result=0;
@@ -681,28 +820,28 @@ int router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
 
   /* Generate the easy portion of the router descriptor. */
   result = tor_snprintf(s, maxlen,
-                    "router %s %s %d %d %d\n"
+                    "router %s %s %d 0 %d\n"
                     "platform %s\n"
                     "published %s\n"
                     "opt fingerprint %s\n"
                     "opt uptime %ld\n"
                     "bandwidth %d %d %d\n"
                     "onion-key\n%s"
-                    "signing-key\n%s%s%s",
+                    "signing-key\n%s%s%s%s",
     router->nickname,
     router->address,
     router->or_port,
-    router->socks_port,
-    router->dir_port,
+    check_whether_dirport_reachable ? router->dir_port : 0,
     router->platform,
     published,
     fingerprint,
-    stats_n_seconds_uptime,
+    stats_n_seconds_working,
     (int) router->bandwidthrate,
     (int) router->bandwidthburst,
     (int) router->bandwidthcapacity,
     onion_pkey, identity_pkey,
-    family_line, bandwidth_usage);
+    family_line, bandwidth_usage,
+    we_are_hibernating() ? "opt hibernating 1\n" : "");
   tor_free(family_line);
   tor_free(onion_pkey);
   tor_free(identity_pkey);
@@ -725,16 +864,18 @@ int router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
   for (tmpe=router->exit_policy; tmpe; tmpe=tmpe->next) {
     in.s_addr = htonl(tmpe->addr);
     /* Write: "accept 1.2.3.4" */
+    tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
     result = tor_snprintf(s+written, maxlen-written, "%s %s",
         tmpe->policy_type == ADDR_POLICY_ACCEPT ? "accept" : "reject",
-        tmpe->msk == 0 ? "*" : inet_ntoa(in));
+        tmpe->msk == 0 ? "*" : addrbuf);
     if (result < 0)
       return -1;
     written += result;
     if (tmpe->msk != 0xFFFFFFFFu && tmpe->msk != 0) {
       /* Write "/255.255.0.0" */
+      tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
       in.s_addr = htonl(tmpe->msk);
-      result = tor_snprintf(s+written, maxlen-written, "/%s", inet_ntoa(in));
+      result = tor_snprintf(s+written, maxlen-written, "/%s", addrbuf);
       if (result<0)
         return -1;
       written += result;
@@ -808,6 +949,7 @@ int router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
   return written+1;
 }
 
+/** Return true iff <b>s</b> is a legally valid server nickname. */
 int is_legal_nickname(const char *s)
 {
   size_t len;
@@ -816,6 +958,8 @@ int is_legal_nickname(const char *s)
   return len > 0 && len <= MAX_NICKNAME_LEN &&
     strspn(s,LEGAL_NICKNAME_CHARACTERS)==len;
 }
+/** Return true iff <b>s</b> is a legally valid server nickname or
+ * hex-encoded identity-key digest. */
 int is_legal_nickname_or_hexdigest(const char *s)
 {
   size_t len;
@@ -827,3 +971,17 @@ int is_legal_nickname_or_hexdigest(const char *s)
   return len == HEX_DIGEST_LEN+1 && strspn(s+1,HEX_CHARACTERS)==len-1;
 }
 
+/** Release all resources held in router keys. */
+void router_free_all_keys(void)
+{
+  if (onionkey)
+    crypto_free_pk_env(onionkey);
+  if (lastonionkey)
+    crypto_free_pk_env(lastonionkey);
+  if (identitykey)
+    crypto_free_pk_env(identitykey);
+  if (key_lock)
+    tor_mutex_free(key_lock);
+  if (desc_routerinfo)
+    routerinfo_free(desc_routerinfo);
+}
