@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerlist.c,v 1.200.2.4 2005/02/23 04:21:04 arma Exp $ */
-const char routerlist_c_id[] = "$Id: routerlist.c,v 1.200.2.4 2005/02/23 04:21:04 arma Exp $";
+/* $Id: routerlist.c,v 1.222 2005/04/01 08:42:26 arma Exp $ */
+const char routerlist_c_id[] = "$Id: routerlist.c,v 1.222 2005/04/01 08:42:26 arma Exp $";
 
 #include "or.h"
 
@@ -64,10 +64,9 @@ int router_reload_router_list(void)
       log_fn(LOG_WARN, "Cached directory at '%s' was unparseable; ignoring.", filename);
     }
     if (routerlist &&
-        ((routerlist->published_on > time(NULL) - OLD_MIN_ONION_KEY_LIFETIME/2)
+        ((routerlist->published_on > time(NULL) - MIN_ONION_KEY_LIFETIME/2)
          || is_recent)) {
-      /* XXX use new onion key lifetime when 0.0.8 servers are obsolete */
-      directory_has_arrived(st.st_mtime); /* do things we've been waiting to do */
+      directory_has_arrived(st.st_mtime, NULL); /* do things we've been waiting to do */
     }
     tor_free(s);
   }
@@ -166,7 +165,6 @@ router_pick_directory_server_impl(int requireothers, int fascistfirewall,
   int i;
   routerinfo_t *router;
   smartlist_t *sl;
-  char buf[16];
 
   if (!routerlist)
     return NULL;
@@ -176,15 +174,14 @@ router_pick_directory_server_impl(int requireothers, int fascistfirewall,
 
   /* Find all the running dirservers we know about. */
   sl = smartlist_create();
-  for (i=0;i< smartlist_len(routerlist->routers); i++) {
+  for (i=0;i < smartlist_len(routerlist->routers); i++) {
     router = smartlist_get(routerlist->routers, i);
     if (!router->is_running || !router->dir_port || !router->is_verified)
       continue;
     if (requireothers && router_is_me(router))
       continue;
     if (fascistfirewall) {
-      tor_snprintf(buf,sizeof(buf),"%d",router->dir_port);
-      if (!smartlist_string_isin(get_options()->FirewallPorts, buf))
+      if (!smartlist_string_num_isin(get_options()->FirewallPorts, router->dir_port))
         continue;
     }
     /* before 0.0.9rc5-cvs, only trusted dirservers served status info. */
@@ -205,7 +202,6 @@ router_pick_trusteddirserver_impl(int requireother, int fascistfirewall)
 {
   smartlist_t *sl;
   routerinfo_t *me;
-  char buf[16];
   trusted_dir_server_t *ds;
   sl = smartlist_create();
   me = router_get_my_routerinfo();
@@ -223,8 +219,7 @@ router_pick_trusteddirserver_impl(int requireother, int fascistfirewall)
           !memcmp(me->identity_digest, d->digest, DIGEST_LEN))
         continue;
       if (fascistfirewall) {
-        tor_snprintf(buf,sizeof(buf),"%d",d->dir_port);
-        if (!smartlist_string_isin(get_options()->FirewallPorts, buf))
+        if (!smartlist_string_num_isin(get_options()->FirewallPorts, d->dir_port))
           continue;
       }
       smartlist_add(sl, d);
@@ -313,8 +308,8 @@ add_nickname_list_to_smartlist(smartlist_t *sl, const char *list, int warn_if_do
                          SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
 
   SMARTLIST_FOREACH(nickname_list, const char *, nick, {
-    if (strlen(nick) > MAX_HEX_NICKNAME_LEN) {
-      log_fn(LOG_WARN,"Nickname too long; skipping");
+    if (!is_legal_nickname_or_hexdigest(nick)) {
+      log_fn(LOG_WARN,"Nickname %s is misformed; skipping", nick);
       continue;
     }
     router = router_get_by_nickname(nick);
@@ -360,7 +355,7 @@ router_nickname_is_in_list(routerinfo_t *router, const char *list)
  */
 static void
 router_add_running_routers_to_smartlist(smartlist_t *sl, int allow_unverified,
-                                        int preferuptime, int preferbandwidth)
+                                        int need_uptime, int need_capacity)
 {
   routerinfo_t *router;
   int i;
@@ -373,7 +368,7 @@ router_add_running_routers_to_smartlist(smartlist_t *sl, int allow_unverified,
     if (router->is_running &&
         (router->is_verified ||
         (allow_unverified &&
-         !router_is_unreliable_router(router, preferuptime, preferbandwidth)))) {
+         !router_is_unreliable(router, need_uptime, need_capacity)))) {
       /* If it's running, and either it's verified or we're ok picking
        * unverified routers and this one is suitable.
        */
@@ -398,18 +393,12 @@ routerlist_find_my_routerinfo(void) {
   return NULL;
 }
 
-/** How many seconds a router must be up before we'll use it for
- * reliability-critical node positions.
- */
-#define ROUTER_REQUIRED_MIN_UPTIME 3600 /* an hour */
-#define ROUTER_REQUIRED_MIN_BANDWIDTH 10000
-
 int
-router_is_unreliable_router(routerinfo_t *router, int need_uptime, int need_bw)
+router_is_unreliable(routerinfo_t *router, int need_uptime, int need_capacity)
 {
   if (need_uptime && router->uptime < ROUTER_REQUIRED_MIN_UPTIME)
     return 1;
-  if (need_bw && router->bandwidthcapacity < ROUTER_REQUIRED_MIN_BANDWIDTH)
+  if (need_capacity && router->bandwidthcapacity < ROUTER_REQUIRED_MIN_BANDWIDTH)
     return 1;
   return 0;
 }
@@ -422,7 +411,7 @@ routerlist_sl_remove_unreliable_routers(smartlist_t *sl)
 
   for (i = 0; i < smartlist_len(sl); ++i) {
     router = smartlist_get(sl, i);
-    if (router_is_unreliable_router(router, 1, 0)) {
+    if (router_is_unreliable(router, 1, 0)) {
       log(LOG_DEBUG, "Router '%s' has insufficient uptime; deleting.",
           router->nickname);
       smartlist_del(sl, i--);
@@ -486,7 +475,7 @@ routerlist_sl_choose_by_bandwidth(smartlist_t *sl)
 routerinfo_t *router_choose_random_node(const char *preferred,
                                         const char *excluded,
                                         smartlist_t *excludedsmartlist,
-                                        int preferuptime, int preferbandwidth,
+                                        int need_uptime, int need_capacity,
                                         int allow_unverified, int strict)
 {
   smartlist_t *sl, *excludednodes;
@@ -501,9 +490,9 @@ routerinfo_t *router_choose_random_node(const char *preferred,
   smartlist_subtract(sl,excludednodes);
   if (excludedsmartlist)
     smartlist_subtract(sl,excludedsmartlist);
-  if (preferuptime)
+  if (need_uptime)
     routerlist_sl_remove_unreliable_routers(sl);
-  if (preferbandwidth)
+  if (need_capacity)
     choice = routerlist_sl_choose_by_bandwidth(sl);
   else
     choice = smartlist_choose(sl);
@@ -511,13 +500,13 @@ routerinfo_t *router_choose_random_node(const char *preferred,
   if (!choice && !strict) {
     sl = smartlist_create();
     router_add_running_routers_to_smartlist(sl, allow_unverified,
-                                            preferuptime, preferbandwidth);
+                                            need_uptime, need_capacity);
     smartlist_subtract(sl,excludednodes);
     if (excludedsmartlist)
       smartlist_subtract(sl,excludedsmartlist);
-    if (preferuptime)
+    if (need_uptime)
       routerlist_sl_remove_unreliable_routers(sl);
-    if (preferbandwidth)
+    if (need_capacity)
       choice = routerlist_sl_choose_by_bandwidth(sl);
     else
       choice = smartlist_choose(sl);
@@ -593,6 +582,9 @@ routerinfo_t *router_get_by_nickname(const char *nickname)
     return NULL;
   if (nickname[0] == '$')
     return router_get_by_hexdigest(nickname);
+  if (server_mode(get_options()) &&
+      !strcasecmp(nickname, get_options()->Nickname))
+    return router_get_my_routerinfo();
 
   maybedigest = (strlen(nickname) == HEX_DIGEST_LEN) &&
     (base16_decode(digest,DIGEST_LEN,nickname,HEX_DIGEST_LEN) == 0);
@@ -636,12 +628,16 @@ routerinfo_t *router_get_by_hexdigest(const char *hexdigest) {
 }
 
 /** Return the router in our routerlist whose 20-byte key digest
- * is <b>hexdigest</b>.  Return NULL if no such router is known. */
+ * is <b>digest</b>.  Return NULL if no such router is known. */
 routerinfo_t *router_get_by_digest(const char *digest) {
   int i;
   routerinfo_t *router;
 
   tor_assert(digest);
+  if (server_mode(get_options()) &&
+      (router = router_get_my_routerinfo()) &&
+      !memcmp(digest, router->identity_digest, DIGEST_LEN))
+    return router;
   if (!routerlist) return NULL;
 
   for (i=0;i<smartlist_len(routerlist->routers);i++) {
@@ -670,6 +666,7 @@ void routerinfo_free(routerinfo_t *router)
   if (!router)
     return;
 
+  tor_free(router->signed_descriptor);
   tor_free(router->address);
   tor_free(router->nickname);
   tor_free(router->platform);
@@ -697,6 +694,8 @@ routerinfo_t *routerinfo_copy(const routerinfo_t *router)
   r->address = tor_strdup(r->address);
   r->nickname = tor_strdup(r->nickname);
   r->platform = tor_strdup(r->platform);
+  if (r->signed_descriptor)
+    r->signed_descriptor = tor_strdup(r->signed_descriptor);
   if (r->onion_pkey)
     r->onion_pkey = crypto_pk_dup_key(r->onion_pkey);
   if (r->identity_pkey)
@@ -720,13 +719,32 @@ routerinfo_t *routerinfo_copy(const routerinfo_t *router)
 /** Free all storage held by a routerlist <b>rl</b> */
 void routerlist_free(routerlist_t *rl)
 {
+  tor_assert(rl);
   SMARTLIST_FOREACH(rl->routers, routerinfo_t *, r,
                     routerinfo_free(r));
   smartlist_free(rl->routers);
+  running_routers_free(rl->running_routers);
   tor_free(rl->software_versions);
   tor_free(rl);
 }
 
+void routerlist_free_current(void)
+{
+  if (routerlist)
+    routerlist_free(routerlist);
+  routerlist = NULL;
+}
+
+void free_trusted_dir_servers(void)
+{
+  if (trusted_dir_servers) {
+    SMARTLIST_FOREACH(trusted_dir_servers, trusted_dir_server_t *, ds,
+                      { tor_free(ds->address); tor_free(ds); });
+    smartlist_free(trusted_dir_servers);
+    trusted_dir_servers = NULL;
+  }
+}
+
 /** Mark the router with ID <b>digest</b> as non-running in our routerlist. */
 void router_mark_as_down(const char *digest) {
   routerinfo_t *router;
@@ -849,12 +867,44 @@ routerlist_remove_old_routers(int age)
 /*
  * Code to parse router descriptors and directories.
  */
+int
+router_load_single_router(const char *s)
+{
+  routerinfo_t *ri;
+
+  if (!(ri = router_parse_entry_from_string(s, NULL))) {
+    log_fn(LOG_WARN, "Error parsing router descriptor; dropping.");
+    return -1;
+  }
+  if (routerlist && routerlist->running_routers) {
+    running_routers_t *rr = routerlist->running_routers;
+    router_update_status_from_smartlist(ri,
+                                        rr->published_on,
+                                        rr->running_routers,
+                                        rr->is_running_routers_format);
+  }
+  if (router_add_to_routerlist(ri)<0) {
+    log_fn(LOG_WARN, "Couldn't add router to list; dropping.");
+    return -1;
+  } else {
+    smartlist_t *changed = smartlist_create();
+    smartlist_add(changed, ri);
+    control_event_descriptors_changed(changed);
+    smartlist_free(changed);
+  }
+  log_fn(LOG_DEBUG, "Added router to list");
+  return 0;
+}
 
 /** Add to the current routerlist each router stored in the
  * signed directory <b>s</b>.  If pkey is provided, check the signature against
  * pkey; else check against the pkey of the signing directory server.
  *
- * DOCDOC dir_is_recent/cached
+ * If <b>dir_is_recent</b> is non-zero, then examine the
+ * Recommended-versions line and take appropriate action.
+ *
+ * If <b>dir_is_cached</b> is non-zero, then we're reading it
+ * from the cache so don't bother to re-write it to the cache.
  */
 int router_load_routerlist_from_directory(const char *s,
                                           crypto_pk_env_t *pkey,
@@ -869,16 +919,23 @@ int router_load_routerlist_from_directory(const char *s,
     return -1;
   }
   if (routerlist) {
+    smartlist_t *changed = smartlist_create();
     SMARTLIST_FOREACH(new_list->routers, routerinfo_t *, r,
-                      router_add_to_routerlist(r));
+    {
+      if (router_add_to_routerlist(r)==0)
+        smartlist_add(changed, r);
+    });
     smartlist_clear(new_list->routers);
     routerlist->published_on = new_list->published_on;
     tor_free(routerlist->software_versions);
     routerlist->software_versions = new_list->software_versions;
     new_list->software_versions = NULL;
     routerlist_free(new_list);
+    control_event_descriptors_changed(changed);
+    smartlist_free(changed);
   } else {
     routerlist = new_list;
+    control_event_descriptors_changed(routerlist->routers);
   }
   if (router_resolve_routerlist(routerlist)) {
     log_fn(LOG_WARN, "Error resolving routerlist");
@@ -944,16 +1001,26 @@ router_resolve_routerlist(routerlist_t *rl)
   return 0;
 }
 
-/** Decide whether a given addr:port is definitely accepted, definitely
- * rejected, or neither by a given policy.  If <b>addr</b> is 0, we
- * don't know the IP of the target address. If <b>port</b> is 0, we
- * don't know the port of the target address.
+/** Decide whether a given addr:port is definitely accepted,
+ * definitely rejected, probably accepted, or probably rejected by a
+ * given policy.  If <b>addr</b> is 0, we don't know the IP of the
+ * target address. If <b>port</b> is 0, we don't know the port of the
+ * target address.
  *
- * Returns -1 for "rejected", 0 for "accepted", 1 for "maybe" (since IP or
- * port is unknown).
+ * For now, the algorithm is pretty simple: we look for definite and
+ * uncertain matches.  The first definite match is what we guess; if
+ * it was proceded by no uncertain matches of the opposite policy,
+ * then the guess is definite; otherwise it is probable.  (If we
+ * have a known addr and port, all matches are definite; if we have an
+ * unknown addr/port, any address/port ranges other than "all" are
+ * uncertain.)
+ *
+ * We could do better by assuming that some ranges never match typical
+ * addresses (127.0.0.1, and so on).  But we'll try this for now.
  */
-int router_compare_addr_to_addr_policy(uint32_t addr, uint16_t port,
-                                       addr_policy_t *policy)
+addr_policy_result_t
+router_compare_addr_to_addr_policy(uint32_t addr, uint16_t port,
+                                   addr_policy_t *policy)
 {
   int maybe_reject = 0;
   int maybe_accept = 0;
@@ -1003,32 +1070,121 @@ int router_compare_addr_to_addr_policy(uint32_t addr, uint16_t port,
       if (tmpe->policy_type == ADDR_POLICY_ACCEPT) {
         /* If we already hit a clause that might trigger a 'reject', than we
          * can't be sure of this certain 'accept'.*/
-        return maybe_reject ? ADDR_POLICY_UNKNOWN : ADDR_POLICY_ACCEPTED;
+        return maybe_reject ? ADDR_POLICY_PROBABLY_ACCEPTED : ADDR_POLICY_ACCEPTED;
       } else {
-        return maybe_accept ? ADDR_POLICY_UNKNOWN : ADDR_POLICY_REJECTED;
+        return maybe_accept ? ADDR_POLICY_PROBABLY_REJECTED : ADDR_POLICY_REJECTED;
       }
     }
   }
   /* accept all by default. */
-  return maybe_reject ? ADDR_POLICY_UNKNOWN : ADDR_POLICY_ACCEPTED;
+  return maybe_reject ? ADDR_POLICY_PROBABLY_ACCEPTED : ADDR_POLICY_ACCEPTED;
 }
 
-/** Return 1 if all running routers will reject addr:port, return 0 if
- * any might accept it. */
-int router_exit_policy_all_routers_reject(uint32_t addr, uint16_t port) {
+/** Return 1 if all running sufficiently-stable routers will reject
+ * addr:port, return 0 if any might accept it. */
+int router_exit_policy_all_routers_reject(uint32_t addr, uint16_t port,
+                                          int need_uptime) {
   int i;
   routerinfo_t *router;
+  addr_policy_result_t r;
   if (!routerlist) return 1;
 
   for (i=0;i<smartlist_len(routerlist->routers);i++) {
     router = smartlist_get(routerlist->routers, i);
-    if (router->is_running && router_compare_addr_to_addr_policy(
-             addr, port, router->exit_policy) != ADDR_POLICY_REJECTED)
-      return 0; /* this one could be ok. good enough. */
+    if (router->is_running &&
+        !router_is_unreliable(router, need_uptime, 0)) {
+      r = router_compare_addr_to_addr_policy(addr, port, router->exit_policy);
+      if (r != ADDR_POLICY_REJECTED && r != ADDR_POLICY_PROBABLY_REJECTED)
+        return 0; /* this one could be ok. good enough. */
+    }
   }
   return 1; /* all will reject. */
 }
 
+/**
+ * If <b>policy</b> implicitly allows connections to any port in the
+ * IP set <b>addr</b>/<b>mask</b>, then set *<b>policy_out</b> to the
+ * part of the policy that allows it, and return 1.  Else return 0.
+ *
+ * A policy allows an IP:Port combination <em>implicitly</em> if
+ * it is included in a *: pattern, or in a fallback pattern.
+ */
+static int
+policy_includes_addr_mask_implicitly(addr_policy_t *policy,
+                                     uint32_t addr, uint32_t mask,
+                                     addr_policy_t **policy_out)
+{
+  uint32_t addr2;
+  tor_assert(policy_out);
+  addr &= mask;
+  addr2 = addr | ~mask;
+  for (; policy; policy=policy->next) {
+    /* Does this policy cover all of the address range we're looking at? */
+    /* Boolean logic time: range X is contained in range Y if, for
+     * each bit B, all possible values of B in X are values of B in Y.
+     * In "addr", we have every fixed bit set to its value, and every
+     * free bit set to 0.  In "addr2", we have every fixed bit set to
+     * its value, and every free bit set to 1.  So if addr and addr2 are
+     * both in the policy, the range is covered by the policy.
+     */
+    if ((policy->addr & policy->msk) == (addr & policy->msk) &&
+        (policy->addr & policy->msk) == (addr2 & policy->msk) &&
+        (policy->prt_min <= 1 && policy->prt_max == 65535)) {
+      return 0;
+    }
+    /* Does this policy cover some of the address range we're looking at? */
+    /* Boolean logic time: range X and range Y intersect if there is
+     * some z such that z & Xmask == Xaddr and z & Ymask == Yaddr.
+     * This is FALSE iff there is some bit b where Xmask == yMask == 1
+     * and Xaddr != Yaddr.  So if X intersects with Y iff at every
+     * place where Xmask&Ymask==1, Xaddr == Yaddr, or equivalently,
+     * Xaddr&Xmask&Ymask == Yaddr&Xmask&Ymask.
+     */
+    if ((policy->addr & policy->msk & mask) == (addr & policy->msk) &&
+        policy->policy_type == ADDR_POLICY_ACCEPT) {
+      *policy_out = policy;
+      return 1;
+    }
+  }
+  *policy_out = NULL;
+  return 1;
+}
+
+/** If <b>policy</b> implicitly allows connections to any port on
+ * 127.*, 192.168.*, etc, then warn (if <b>warn</b> is set) and return
+ * true.  Else return false.
+ **/
+int
+exit_policy_implicitly_allows_local_networks(addr_policy_t *policy,
+                                             int warn)
+{
+  addr_policy_t *p;
+  int r=0,i;
+  static struct {
+    uint32_t addr; uint32_t mask; const char *network;
+  } private_networks[] = {
+    { 0x7f000000, 0xff000000, "localhost (127.x)" },
+    { 0x0a000000, 0xff000000, "addresses in private network 10.x" },
+    { 0xa9fe0000, 0xffff0000, "addresses in private network 169.254.x" },
+    { 0xac100000, 0xfff00000, "addresses in private network 172.16.x" },
+    { 0xc0a80000, 0xffff0000, "addresses in private network 192.168.x" },
+    { 0,0,NULL},
+  };
+  for (i=0; private_networks[i].addr; ++i) {
+    p = NULL;
+      if (policy_includes_addr_mask_implicitly(
+              policy, private_networks[i].addr, private_networks[i].mask, &p)) {
+        if (warn)
+          log_fn(LOG_WARN, "Exit policy %s implicitly accepts %s",
+                 p?p->string:"(default)",
+                 private_networks[i].network);
+        r = 1;
+      }
+  }
+
+  return r;
+}
+
 /** Return true iff <b>router</b> does not permit exit streams.
  */
 int router_exit_policy_rejects_all(routerinfo_t *router) {
@@ -1039,7 +1195,8 @@ int router_exit_policy_rejects_all(routerinfo_t *router) {
 /** Release all space held in <b>rr</b>. */
 void running_routers_free(running_routers_t *rr)
 {
-  tor_assert(rr);
+  if (!rr)
+    return;
   if (rr->running_routers) {
     SMARTLIST_FOREACH(rr->running_routers, char *, s, tor_free(s));
     smartlist_free(rr->running_routers);
@@ -1047,6 +1204,18 @@ void running_routers_free(running_routers_t *rr)
   tor_free(rr);
 }
 
+/** We've just got a running routers list in <b>rr</b>; update the
+ * status of the routers in <b>list</b>, and cache <b>rr</b> */
+void
+routerlist_set_runningrouters(routerlist_t *list, running_routers_t *rr)
+{
+  routerlist_update_from_runningrouters(list,rr);
+  if (list->running_routers != rr) {
+    running_routers_free(list->running_routers);
+    list->running_routers = rr;
+  }
+}
+
 /** Update the running/not-running status of every router in <b>list</b>, based
  * on the contents of <b>rr</b>. */
 /* Note: this function is not yet used, since nobody publishes just
@@ -1233,6 +1402,7 @@ add_trusted_dir_server(const char *address, uint16_t port, const char *digest)
   smartlist_add(trusted_dir_servers, ent);
 }
 
+/** Remove all members from the list of trusted dir servers. */
 void clear_trusted_dir_servers(void)
 {
   if (trusted_dir_servers) {
