@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.94.2.2 2005/01/06 20:30:09 nickm Exp $ */
-const char routerparse_c_id[] = "$Id: routerparse.c,v 1.94.2.2 2005/01/06 20:30:09 nickm Exp $";
+/* $Id: routerparse.c,v 1.104 2005/03/31 20:52:43 arma Exp $ */
+const char routerparse_c_id[] = "$Id: routerparse.c,v 1.104 2005/03/31 20:52:43 arma Exp $";
 
 /**
  * \file routerparse.c
@@ -142,6 +142,7 @@ static int check_directory_signature(const char *digest,
 static crypto_pk_env_t *find_dir_signing_key(const char *str);
 /* static */ int is_obsolete_version(const char *myversion,
                                      const char *versionlist);
+static int tor_version_same_series(tor_version_t *a, tor_version_t *b);
 
 /** Set <b>digest</b> to the SHA-1 digest of the hash of the directory in
  * <b>s</b>.  Return 0 on success, nonzero on failure.
@@ -199,6 +200,12 @@ get_recommended_software_from_directory(const char *str)
 
 /** Return 1 if <b>myversion</b> is not in <b>versionlist</b>, and if at least
  * one version of Tor on <b>versionlist</b> is newer than <b>myversion</b>.
+
+ * Return 1 if no version from the same series as <b>myversion</b> is
+ * in <b>versionlist</b> (and <b>myversion</b> is not the newest
+ * version), or if a newer version from the same series is in
+ * <b>versionlist</b>.
+ *
  * Otherwise return 0.
  * (versionlist is a comma-separated list of version strings,
  * optionally prefixed with "Tor".  Versions that can't be parsed are
@@ -207,9 +214,11 @@ get_recommended_software_from_directory(const char *str)
                            const char *versionlist) {
   const char *vl;
   tor_version_t mine, other;
-  int found_newer = 0, r, ret;
+  int found_newer = 0, found_newer_in_series = 0, found_any_in_series = 0,
+    r, ret, same;
   static int warned_too_new=0;
   smartlist_t *version_sl;
+  int XXXpath;
 
   vl = versionlist;
 
@@ -229,26 +238,63 @@ get_recommended_software_from_directory(const char *str)
     if (tor_version_parse(cp, &other)) {
       /* Couldn't parse other; it can't be a match. */
     } else {
+      same = tor_version_same_series(&mine, &other);
+      if (same)
+        found_any_in_series = 1;
       r = tor_version_compare(&mine, &other);
       if (r==0) {
         ret = 0;
         goto done;
       } else if (r<0) {
         found_newer = 1;
+        if (same)
+          found_newer_in_series = 1;
       }
     }
   });
 
-  if (!found_newer) {
-    if (!warned_too_new) {
-      log(LOG_WARN, "This version of Tor (%s) is newer than any on the recommended list (%s)",
-             myversion, versionlist);
-      warned_too_new=1;
+  /* We didn't find the listed version. Is it new or old? */
+
+  if (found_any_in_series) {
+    if (!found_newer_in_series) {
+      /* We belong to a series with recommended members, and we are newer than
+       * any recommended member. We're probably okay. */
+      if (!warned_too_new) {
+        log(LOG_WARN, "This version of Tor (%s) is newer than any in the same series on the recommended list (%s)",
+            myversion, versionlist);
+        warned_too_new = 1;
+      }
+      ret = 0;
+      XXXpath = 1;
+    } else {
+      /* We found a newer one in the same series; we're obsolete. */
+      ret = 1;
+      XXXpath = 2;
     }
-    ret = 0;
   } else {
-    ret = 1;
+    if (found_newer) {
+      /* We belong to a series with no recommended members, and
+       * a newer series is recommended. We're obsolete. */
+      ret = 1;
+      XXXpath = 3;
+    } else {
+      /* We belong to a series with no recommended members, and it's
+       * newer than any recommended series. We're probably okay. */
+      if (!warned_too_new) {
+        log(LOG_WARN, "This version of Tor (%s) is newer than any on the recommended list (%s)",
+            myversion, versionlist);
+        warned_too_new = 1;
+      }
+      ret = 0;
+      XXXpath = 4;
+    }
   }
+  /*
+  log_fn(LOG_DEBUG,
+         "Decided that %s is %sobsolete relative to %s: %d, %d, %d\n",
+         myversion, ret?"":"not ", versionlist, found_newer,
+         found_any_in_series, found_newer_in_series);
+  */
 
  done:
   SMARTLIST_FOREACH(version_sl, char *, version, tor_free(version));
@@ -289,10 +335,17 @@ int check_software_version_against_directory(const char *directory,
 }
 
 /** Parse a directory from <b>str</b> and, when done, store the
- * resulting routerlist in *<b>dest</b>, freeing the old value if necessary.
+ * resulting routerlist in *<b>dest</b>, freeing the old value if
+ * necessary.
+ *
  * If <b>pkey</b> is provided, we check the directory signature with pkey.
  *
- * DOCDOC check_version, write_to_cache.
+ * If <b>check_version</b> is non-zero, then examine the
+ * Recommended-versions * line in the directory, and warn or quit
+ * as needed.
+ *
+ * If <b>write_to_cache</b> is non-zero, then store this directory in
+ * memory and/or disk as well.
  */
 int /* Should be static; exposed for unit tests */
 router_parse_routerlist_from_directory(const char *str,
@@ -305,6 +358,7 @@ router_parse_routerlist_from_directory(const char *str,
   char digest[DIGEST_LEN];
   routerlist_t *new_dir = NULL;
   char *versions = NULL;
+  int nickname_list_is_running_routers;
   smartlist_t *good_nickname_list = NULL;
   time_t published_on;
   int i, r;
@@ -343,6 +397,10 @@ router_parse_routerlist_from_directory(const char *str,
     goto err;
 
   /* now we know tok->n_args == 1, so it's safe to access tok->args[0] */
+  if (!is_legal_nickname(tok->args[0])) {
+    log_fn(LOG_WARN, "Directory nickname '%s' is misformed", tok->args[0]);
+    goto err;
+  }
   strlcpy(dirnickname, tok->args[0], sizeof(dirnickname));
 
   SMARTLIST_FOREACH(tokens, directory_token_t *, tok, token_free(tok));
@@ -415,6 +473,7 @@ router_parse_routerlist_from_directory(const char *str,
     }
   }
 
+  nickname_list_is_running_routers = (tok->tp == K_RUNNING_ROUTERS);
   good_nickname_list = smartlist_create();
   for (i=0; i<tok->n_args; ++i) {
     smartlist_add(good_nickname_list, tok->args[i]);
@@ -448,6 +507,11 @@ router_parse_routerlist_from_directory(const char *str,
 
   new_dir->software_versions = versions; versions = NULL;
   new_dir->published_on = published_on;
+  new_dir->running_routers = tor_malloc_zero(sizeof(running_routers_t));
+  new_dir->running_routers->published_on = published_on;
+  new_dir->running_routers->running_routers = good_nickname_list;
+  new_dir->running_routers->is_running_routers_format =
+    nickname_list_is_running_routers;
 
   SMARTLIST_FOREACH(tokens, directory_token_t *, tok, token_free(tok));
   smartlist_free(tokens);
@@ -464,20 +528,22 @@ router_parse_routerlist_from_directory(const char *str,
   if (new_dir)
     routerlist_free(new_dir);
   tor_free(versions);
+  if (good_nickname_list) {
+    SMARTLIST_FOREACH(good_nickname_list, char *, n, tor_free(n));
+    smartlist_free(good_nickname_list);
+  }
  done:
   if (declared_key) crypto_free_pk_env(declared_key);
   if (tokens) {
     SMARTLIST_FOREACH(tokens, directory_token_t *, tok, token_free(tok));
     smartlist_free(tokens);
   }
-  if (good_nickname_list) {
-    SMARTLIST_FOREACH(good_nickname_list, char *, n, tor_free(n));
-    smartlist_free(good_nickname_list);
-  }
   return r;
 }
 
-/* DOCDOC */
+/** Read a signed router status statement from <b>str</b>.  On
+ * success, return it, and cache the original string if
+ * <b>write_to_cache</b> is set.  Otherwise, return NULL.  */
 running_routers_t *
 router_parse_runningrouters(const char *str, int write_to_cache)
 {
@@ -791,6 +857,7 @@ routerinfo_t *router_parse_entry_from_string(const char *s,
   }
 
   router = tor_malloc_zero(sizeof(routerinfo_t));
+  router->signed_descriptor = tor_strndup(s, end-s);
   ports_set = bw_set = 0;
 
   if (tok->n_args == 2 || tok->n_args == 5 || tok->n_args == 6) {
@@ -804,7 +871,6 @@ routerinfo_t *router_parse_entry_from_string(const char *s,
 
     if (tok->n_args >= 5) {
       router->or_port = (uint16_t) tor_parse_long(tok->args[2],10,0,65535,NULL,NULL);
-      router->socks_port = (uint16_t) tor_parse_long(tok->args[3],10,0,65535,NULL,NULL);
       router->dir_port = (uint16_t) tor_parse_long(tok->args[4],10,0,65535,NULL,NULL);
       ports_set = 1;
     }
@@ -823,7 +889,6 @@ routerinfo_t *router_parse_entry_from_string(const char *s,
       goto err;
     }
     router->or_port = (uint16_t) tor_parse_long(tok->args[0],10,0,65535,NULL,NULL);
-    router->socks_port = (uint16_t) tor_parse_long(tok->args[1],10,0,65535,NULL,NULL);
     router->dir_port = (uint16_t) tor_parse_long(tok->args[2],10,0,65535,NULL,NULL);
     ports_set = 1;
   }
@@ -950,8 +1015,8 @@ routerinfo_t *router_parse_entry_from_string(const char *s,
     router->platform = tor_strdup("<unknown>");
   }
 
-//  log_fn(LOG_DEBUG,"or_port %d, socks_port %d, dir_port %d, bandwidthrate %u, bandwidthburst %u.",
-//    router->or_port, router->socks_port, router->dir_port,
+//  log_fn(LOG_DEBUG,"or_port %d, dir_port %d, bandwidthrate %u, bandwidthburst %u.",
+//    router->or_port, router->dir_port,
 //    (unsigned) router->bandwidthrate, (unsigned) router->bandwidthburst);
 
   goto done;
@@ -1045,8 +1110,10 @@ static addr_policy_t *
 router_parse_addr_policy(directory_token_t *tok) {
 
   addr_policy_t *newe;
-  struct in_addr in;
-  char *arg, *address;
+//  struct in_addr in;
+  char *arg;
+//  char *address;
+//  char buf[INET_NTOA_BUF_LEN];
 
   tor_assert(tok->tp == K_REJECT || tok->tp == K_ACCEPT);
 
@@ -1067,13 +1134,14 @@ router_parse_addr_policy(directory_token_t *tok) {
                                 &newe->prt_min, &newe->prt_max))
     goto policy_read_failed;
 
-  in.s_addr = htonl(newe->addr);
-  address = tor_strdup(inet_ntoa(in));
+//  in.s_addr = htonl(newe->addr);
+//  tor_inet_ntoa(&in, buf, sizeof(buf));
+//  address = tor_strdup(buf);
 //  in.s_addr = htonl(newe->msk);
 //  log_fn(LOG_DEBUG,"%s %s/%s:%d-%d",
 //         newe->policy_type == ADDR_POLICY_REJECT ? "reject" : "accept",
 //         address, inet_ntoa(in), newe->prt_min, newe->prt_max);
-  tor_free(address);
+//  tor_free(address);
 
   return newe;
 
@@ -1466,13 +1534,16 @@ int tor_version_parse(const char *s, tor_version_t *out)
 {
   char *eos=NULL, *cp=NULL;
   /* Format is:
-   *   NUM dot NUM dot NUM [ ( pre | rc | dot ) NUM [ -cvs ] ]
+   *   "Tor " ? NUM dot NUM dot NUM [ ( pre | rc | dot ) NUM [ -cvs ] ]
    */
   tor_assert(s);
   tor_assert(out);
 
   memset(out, 0, sizeof(tor_version_t));
 
+  if (!strcasecmpstart(s, "Tor "))
+    cp += 4;
+
   /* Get major. */
   out->major = strtol(s,&eos,10);
   if (!eos || eos==s || *eos != '.') return -1;
@@ -1551,3 +1622,12 @@ int tor_version_compare(tor_version_t *a, tor_version_t *b)
   }
 }
 
+static int
+tor_version_same_series(tor_version_t *a, tor_version_t *b)
+{
+  tor_assert(a);
+  tor_assert(b);
+  return ((a->major == b->major) &&
+          (a->minor == b->minor) &&
+          (a->micro == b->micro));
+}
