@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: test.c,v 1.160.2.2 2005/01/06 20:30:09 nickm Exp $ */
-const char test_c_id[] = "$Id: test.c,v 1.160.2.2 2005/01/06 20:30:09 nickm Exp $";
+/* $Id: test.c,v 1.170 2005/03/01 01:44:57 nickm Exp $ */
+const char test_c_id[] = "$Id: test.c,v 1.170 2005/03/01 01:44:57 nickm Exp $";
 
 #include "orconfig.h"
 #include <stdio.h>
@@ -791,6 +791,14 @@ test_util(void) {
   test_assert(strcmpend("abcdef", "dee")>0);
   test_assert(strcmpend("ab", "abb")<0);
 
+  {
+    char tmpbuf[INET_NTOA_BUF_LEN];
+    struct in_addr in;
+    tor_inet_aton("18.244.0.188",&in);
+    tor_inet_ntoa(&in, tmpbuf, sizeof(tmpbuf));
+    test_streq(tmpbuf, "18.244.0.188");
+  }
+
   /* XXXX test older functions. */
   smartlist_free(sl);
 }
@@ -802,11 +810,13 @@ test_gzip(void)
   size_t len1, len2;
 
   buf1 = tor_strdup("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA");
+  test_eq(detect_compression_method(buf1, strlen(buf1)), 0);
   if (is_gzip_supported()) {
     test_assert(!tor_gzip_compress(&buf2, &len1, buf1, strlen(buf1)+1,
                                    GZIP_METHOD));
     test_assert(buf2);
     test_assert(!memcmp(buf2, "\037\213", 2)); /* Gzip magic. */
+    test_eq(detect_compression_method(buf2, len1), GZIP_METHOD);
 
     test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1, GZIP_METHOD));
     test_assert(buf3);
@@ -820,6 +830,7 @@ test_gzip(void)
                                  ZLIB_METHOD));
   test_assert(buf2);
   test_assert(!memcmp(buf2, "\x78\xDA", 2)); /* deflate magic. */
+  test_eq(detect_compression_method(buf2, len1), ZLIB_METHOD);
 
   test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1, ZLIB_METHOD));
   test_assert(buf3);
@@ -995,6 +1006,7 @@ test_dir_format(void)
   routerlist_t *dir1 = NULL, *dir2 = NULL;
   tor_version_t ver1;
   char *bw_lines = NULL;
+  const char *m;
 
   test_assert( (pk1 = crypto_new_pk_env()) );
   test_assert( (pk2 = crypto_new_pk_env()) );
@@ -1026,11 +1038,10 @@ test_dir_format(void)
   get_platform_str(platform, sizeof(platform));
   memset(&r1,0,sizeof(r1));
   memset(&r2,0,sizeof(r2));
-  r1.address = tor_strdup("testaddr1.foo.bar");
+  r1.address = tor_strdup("18.244.0.1");
   r1.addr = 0xc0a80001u; /* 192.168.0.1 */
   r1.published_on = 0;
   r1.or_port = 9000;
-  r1.socks_port = 9002;
   r1.dir_port = 9003;
   r1.onion_pkey = pk1;
   r1.identity_pkey = pk2;
@@ -1052,12 +1063,11 @@ test_dir_format(void)
   ex2.msk = 0xFF000000u;
   ex2.prt_min = ex2.prt_max = 24;
   ex2.next = NULL;
-  r2.address = tor_strdup("tor.tor.tor");
+  r2.address = tor_strdup("1.1.1.1");
   r2.addr = 0x0a030201u; /* 10.3.2.1 */
   r2.platform = tor_strdup(platform);
   r2.published_on = 5;
   r2.or_port = 9005;
-  r2.socks_port = 0;
   r2.dir_port = 0;
   r2.onion_pkey = pk2;
   r2.identity_pkey = pk1;
@@ -1079,7 +1089,7 @@ test_dir_format(void)
   memset(buf, 0, 2048);
   test_assert(router_dump_router_to_string(buf, 2048, &r1, pk2)>0);
 
-  strcpy(buf2, "router Magri testaddr1.foo.bar 9000 9002 9003\n"
+  strcpy(buf2, "router Magri 18.244.0.1 9000 0 9003\n"
          "platform Tor "VERSION" on ");
   strcat(buf2, get_uname());
   strcat(buf2, "\n"
@@ -1109,7 +1119,6 @@ test_dir_format(void)
   test_assert(rp1);
   test_streq(rp1->address, r1.address);
   test_eq(rp1->or_port, r1.or_port);
-  test_eq(rp1->socks_port, r1.socks_port);
   test_eq(rp1->dir_port, r1.dir_port);
   test_eq(rp1->bandwidthrate, r1.bandwidthrate);
   test_eq(rp1->bandwidthburst, r1.bandwidthburst);
@@ -1133,7 +1142,6 @@ test_dir_format(void)
   test_assert(rp2);
   test_streq(rp2->address, r2.address);
   test_eq(rp2->or_port, r2.or_port);
-  test_eq(rp2->socks_port, r2.socks_port);
   test_eq(rp2->dir_port, r2.dir_port);
   test_eq(rp2->bandwidth, r2.bandwidth);
   test_assert(crypto_pk_cmp_keys(rp2->onion_pkey, pk2) == 0);
@@ -1159,10 +1167,10 @@ test_dir_format(void)
   r2.published_on = time(NULL)-3*60*60;
   test_assert(router_dump_router_to_string(buf, 2048, &r1, pk2)>0);
   cp = buf;
-  test_eq(dirserv_add_descriptor((const char**)&cp), 1);
+  test_eq(dirserv_add_descriptor((const char**)&cp,&m), 1);
   test_assert(router_dump_router_to_string(buf, 2048, &r2, pk1)>0);
   cp = buf;
-  test_eq(dirserv_add_descriptor((const char**)&cp), 1);
+  test_eq(dirserv_add_descriptor((const char**)&cp,&m), 1);
   get_options()->Nickname = tor_strdup("DirServer");
   test_assert(!dirserv_dump_directory_to_string(&cp,pk3));
   test_assert(!router_parse_routerlist_from_directory(cp, &dir1, pk3, 1, 0));
@@ -1231,11 +1239,22 @@ test_dir_format(void)
   test_eq(1, is_obsolete_version("0.0.1", "0.0.2,Tor 0.0.3"));
   test_eq(1, is_obsolete_version("0.0.1", "0.0.3,BetterTor 0.0.1"));
   test_eq(0, is_obsolete_version("0.0.2", "Tor 0.0.2,Tor 0.0.3"));
-  test_eq(1, is_obsolete_version("0.0.2", "Tor 0.0.2pre1,Tor 0.0.3"));
+  test_eq(0, is_obsolete_version("0.0.2", "Tor 0.0.2pre1,Tor 0.0.3"));
+  test_eq(1, is_obsolete_version("0.0.2", "Tor 0.0.2.1,Tor 0.0.3"));
   test_eq(0, is_obsolete_version("0.1.0", "Tor 0.0.2,Tor 0.0.3"));
   test_eq(0, is_obsolete_version("0.0.7rc2", "0.0.7,Tor 0.0.7rc2,Tor 0.0.8"));
   test_eq(0, is_obsolete_version("0.0.5", "0.0.5-cvs"));
   test_eq(0, is_obsolete_version("0.0.5.1-cvs", "0.0.5"));
+  /* Not on list, but newer than any in same series. */
+  test_eq(0, is_obsolete_version("0.1.0.3", "Tor 0.1.0.2,Tor 0.0.9.5,Tor 0.1.1.0"));
+  /* Series newer than any on list. */
+  test_eq(0, is_obsolete_version("0.1.1.3", "Tor 0.1.0.2,Tor 0.0.9.5,Tor 0.1.1.0"));
+  /* Series older than any on list. */
+  test_eq(1, is_obsolete_version("0.0.1.3", "Tor 0.1.0.2,Tor 0.0.9.5,Tor 0.1.1.0"));
+  /* Not on list, not newer than any on same series. */
+  test_eq(1, is_obsolete_version("0.1.0.1", "Tor 0.1.0.2,Tor 0.0.9.5,Tor 0.1.1.0"));
+  /* On list, not newer than any on same series. */
+  test_eq(1, is_obsolete_version("0.1.0.1", "Tor 0.1.0.2,Tor 0.0.9.5,Tor 0.1.1.0"));
 
   test_eq(0, tor_version_as_new_as("Tor 0.0.5", "0.0.9pre1-cvs"));
   test_eq(1, tor_version_as_new_as(
@@ -1299,6 +1318,8 @@ main(int c, char**v) {
   rep_hist_init();
   atexit(remove_directory);
 
+  printf("Running Tor unit tests on %s\n", get_uname());
+
 //  puts("========================== Buffers =========================");
   if (0) test_buffers();
   puts("\n========================== Crypto ==========================");
