@@ -1,6 +1,7 @@
 /* Copyright 2001-2004 Roger Dingledine. */
 /* See LICENSE for licensing information */
-/* $Id: tor_main.c,v 1.5 2004/05/10 02:43:41 arma Exp $ */
+/* $Id: tor_main.c,v 1.7 2004/11/29 22:25:31 nickm Exp $ */
+const char tor_main_c_id[] = "$Id: tor_main.c,v 1.7 2004/11/29 22:25:31 nickm Exp $";
 
 /**
  * \file tor_main.c
@@ -17,10 +18,3 @@ int main(int argc, char *argv[])
   return tor_main(argc, argv);
 }
 
-/*
-  Local Variables:
-  mode:c
-  indent-tabs-mode:nil
-  c-basic-offset:2
-  End:
-*/
