@@ -1,6 +1,6 @@
 /* Copyright 2004 Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tor-resolve.c,v 1.12.2.1 2005/02/04 07:16:13 arma Exp $ */
+/* $Id: tor-resolve.c,v 1.16 2005/04/01 07:03:44 nickm Exp $ */
 
 #include "orconfig.h"
 
@@ -74,15 +74,21 @@ parse_socks4a_resolve_response(const char *response, size_t len,
   tor_assert(response);
   tor_assert(addr_out);
 
-  if (len < RESPONSE_LEN)
+  if (len < RESPONSE_LEN) {
+    log_fn(LOG_WARN,"Truncated socks response.");
     return -1;
-  if (((uint8_t)response[0])!=0) /* version: 0 */
+  }
+  if (((uint8_t)response[0])!=0) { /* version: 0 */
+    log_fn(LOG_WARN,"Nonzero version in socks response: bad format.");
     return -1;
+  }
   status = (uint8_t)response[1];
-  if (get_uint16(response+2)!=0) /* port: 0 */
+  if (get_uint16(response+2)!=0) { /* port: 0 */
+    log_fn(LOG_WARN,"Nonzero port in socks response: bad format.");
     return -1;
+  }
   if (status != 90) {
-    log_fn(LOG_WARN,"Got status response '%d', meaning not success.", status);
+    log_fn(LOG_WARN,"Got status response '%d': socks request failed.", status);
     return -1;
   }
 
@@ -151,7 +157,6 @@ do_resolve(const char *hostname, uint32_t sockshost, uint16_t socksport,
   }
 
   if (parse_socks4a_resolve_response(response_buf, RESPONSE_LEN,result_addr)<0){
-    log_fn(LOG_ERR, "Error parsing SOCKS response");
     return -1;
   }
 
@@ -174,6 +179,7 @@ main(int argc, char **argv)
   int n_args;
   struct in_addr a;
   uint32_t result;
+  char buf[INET_NTOA_BUF_LEN];
 
   arg = &argv[1];
   n_args = argc-1;
@@ -225,6 +231,7 @@ main(int argc, char **argv)
     return 1;
 
   a.s_addr = htonl(result);
-  printf("%s\n", inet_ntoa(a));
+  tor_inet_ntoa(&a, buf, sizeof(buf));
+  printf("%s\n", buf);
   return 0;
 }
